 /*
 Linear Technology DC2343A Demonstration Board.
 Reference Application File for Controlling the LTC3335 Nanopower Buck-Boost DC/DC with Integrated Coulomb Counter on the DC2343A PCB.

 @verbatim
 Demonstration Circuit DC2343A is a complete system level solution for a Nanopower Buck-Boost DC/DC with Integrated Coulomb Counter.
 The DC2343A contains a PIC16F1459 embedded processor for communication to the PC over USB and the LTC3335 via its I2C port.
 The GUI is capable of reading and writing all the control registers on the LTC3335 as well as displaying and resetting all its
 alarm registers.  The firmware and software for the embedded system and the GUI are available at the LTC3335 solutions page
 located at http://www.linear.com/product/LTC3335#demoboards.  The DC2343A uses two Analog-to-Digital Converter channels to sample
 the Battery and Output voltag-es.  The voltage samples improve the functionality of the GUI, and allow optimal software correction
 to the measured coulombs.  By adding the software correction the first order known errors are compensated for over the operating
 range and the resultant coulomb count is accurate to within 3%.

 The DC2343A contains an LTC3335 which is a high efficiency, low quiescent current (680nA) buck-boost DC/DC converter with an
 integrated precision coulomb counter which monitors accumulated battery discharge in long life battery powered applications.
 The buck-boost can operate down to 1.8V on its input and provides eight pin selectable output voltages with up to 50mA of output
 current. The coulomb counter stores the accumulated battery discharge in an internal register accessible via an I2C interface.
 The LTC3335 features a programmable discharge alarm threshold. When the threshold is reached, an interrupt is generated at the
 IRQ pin. To accommodate a wide range of battery types and sizes, the peak input current can be selected from as low as 5mA to
 as high as 250mA and the full-scale coulomb counter has a range from 1.1mAh (with 5mA IPEAK) to 1793Ah (with 250mA IPEAK).
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 860 $
 $Date: 2014-11-20 11:19:57 -0500 (Thu, 20 Nov 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

*/

#if !defined(__XC8)
#error This project was built for the XC8 compiler from Microchip Technology Inc.
#error www.microchip.com
#endif

// Fuse configuration for Debug and Release versions
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)
// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config CPUDIV = NOCLKDIV// CPU System Clock Selection Bit (NO CPU system divide)
#pragma config USBLSCLK = 48MHz // USB Low Speed Clock Selection bit (System clock expects 48 MHz, FS/LS USB CLKENs divide-by is set to 8.)
#pragma config PLLMULT = 3x     // PLL Multipler Selection Bit (3x Output Frequency Selected)
#pragma config PLLEN = ENABLED  // PLL Enable Bit (3x or 4x PLL Enabled)
#pragma config STVREN = ON      // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Include basic types
#include "Typedefs.h"

// Include hardware definition
#include <xc.h>         // Defines the registers in the PIC.
#include "DC2343A.h"     // Defines the usage of the PIC hardware on the DC2343A.

// PIC Driver Modules
#include "PIC16F_Counter.h"
#include "PIC16F_Timer.h"
#include "PIC16F_I2C.h"
#include "PIC16F_ADC.h"
#include "PIC16F_Flash.h"
#include "USB_Driver/usb_config.h"
#include "USB_Driver/usb.h"
#include "USB_Driver/usb_device.h"

// IC interface Modules
#include "LTC3335.h"

// High Level Modules
#include "USB_Parser.h"
#include "Error.h"
#include "Status.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void main(void)
{
    // Configure the PIC hardware
    DC2343_DISABLE_INTERRUPTS;
    DC2343_CONFIGURE_CLOCK;
    LED_GREEN_OUT_PIN_SETUP;
    LED_AMBER_OUT_PIN_SETUP;
    DVCC_OUT_PIN_SETUP;
    IRQ_IN_PIN_SETUP;
    PGOOD_IN_PIN_SETUP;
    ENABLE_ADC_OUT_PIN_SETUP
    VBAT_ADC_PIN_SETUP
    VOUT_ADC_PIN_SETUP
    PIC16F_Timer_Init();
    PIC16F_Counter_Init();
    PIC16F_I2C_Init();
    PIC16F_ADC_Init();
    PIC16F_Flash_Init();
    DC2343_ENABLE_INTERRUPTS;

    // Init the Error handler first, so that HW issues upon startup can be recorded
    Error_Init();
    Status_Init();

    // Initialize the Hardware Modules.
    LTC3335_Init();                                   // Init after I2C, as LTC3335 communication is through I2C

    // Initialize USB and Attach.
    USB_Parser_Init();
    USBDeviceInit();
    USBDeviceAttach();

    // Start executing the tasks while receiving/sending USB.
    while (true)
    {
        // Handle the USB driver
        USBDeviceTasks();

        // Do not try to use the USB peripheral until enumerated.
        if (USBGetDeviceState() == CONFIGURED_STATE)
        {
            // Process the commands received through the USB and send the responses.
            USB_Parser_Task();
        }

        // If LTC3335 Counter Test is active, count edges on /IRQ over time.
        LTC3335_Counter_Test_Task();

        // Indicate the system state through the LED.
        Status_Task();
    }
}

// Handler for single interrupt as per section 8 of PIC16F1459 datasheet.
void __interrupt isr(void)
{
    PIC16F_Counter_ISR();
    PIC16F_Timer_ISR();
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
