/*
 Linear Technology DC2343A Demonstration Board.
 PIC16 Driver File for data storage in Flash Memory..
 All datasheet references in this file refer to Microchip Technology Inc. document 40001639B.pdf.

 @verbatim
 This code uses the high endurance region of flash to store nonvolatile data.
  * In order to keep the code module simple, data stored in flash memory can not span a flash page.
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 434 $
 $Date: 2014-06-16 10:32:37 -0400 (Mon, 16 Jun 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

 */

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <xc.h>         // Defines the registers in the PIC.
#include "Typedefs.h"
#include "DC2343A.h"
#include "PIC16F_Flash.h"
#include "LTC3335.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// This must be the 6 bits of each byte stored in flash, so that they can be read directly as const variables.
#define PIC16F_FLASH_RETLW  0x34

// Place the flash data at the start of the last two flash pages in memory.
#define PIC16F_FLASH_DATA_ADDRESS   (_ROMSIZE - 2 * _FLASH_ERASE_SIZE)

// Place the FW revision immediately before the flash data so that it's easy to manually find in memory.
#define PIC16F_FLASH_FW_REV_ADDRESS (PIC16F_FLASH_DATA_ADDRESS - 4)

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
const PIC16F_FLASH_DATA_TYPE PIC16F_Flash_Data @ PIC16F_FLASH_DATA_ADDRESS =
{
    DC2343_SERIAL_NUM_DEFAULT,
    LTC3335_IPEAK_CONFIGURATION_IPEAK_100MA,
    LTC3335_OUTPUT_VOLTAGE_3_3V + 1,    // +1 as the GUI enum includes HARDWARE_CONTROL as first value in enum.
    DC2343_EN_CONFIGURATION_ON,
    0,
    0,
    0.0,
    {0,0,0,0,0,0},
    DC2343_NONSTANDARD_INDUCTOR_DEFAULT
};

const DC2343_FW_REV_TYPE PIC16F_Flash_Fw_Rev @ PIC16F_FLASH_FW_REV_ADDRESS =
{
    DC2343_FW_REV_MAJOR,
    DC2343_FW_REV_MINOR,
    DC2343_FW_REV_SUB_MINOR,
    DC2343_FW_REV_BETA
};

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void pic16f_flash_read_page(uint16_t block_address, uint16_t* data_ptr);
void pic16f_flash_erase_page(uint16_t block_address);
void pic16f_flash_write_page(uint16_t block_address, uint16_t* data_ptr);

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

void PIC16F_Flash_Init(void)
{
    return;
}

void PIC16F_Flash_Write(uint16_t address, uint8_t *data_ptr, uint8_t num_bytes)
{
    uint16_t block_address  = address & ~(_FLASH_ERASE_SIZE - 1);
    uint16_t temp_bytes[_FLASH_ERASE_SIZE];
    uint8_t* temp_byte_ptr;

    // Read out the bytes before we erase the page.
    pic16f_flash_read_page(block_address, &temp_bytes);

    // Erase the page.
    pic16f_flash_erase_page(block_address);

    // Modify the bytes in the page.
    temp_byte_ptr = (uint8_t*) &temp_bytes[address - block_address];
    while(num_bytes > 0)
    {
        *temp_byte_ptr++ = *data_ptr++;
        *temp_byte_ptr++ = PIC16F_FLASH_RETLW;
        num_bytes--;
    }

    // Write page back to flash.
    pic16f_flash_write_page(block_address, &temp_bytes);

    return;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Read page according to datasheet Figure 11-1.
void pic16f_flash_read_page(uint16_t address, uint16_t* data_ptr)
{
    uint8_t num_bytes = _FLASH_ERASE_SIZE;

    DC2343_DISABLE_INTERRUPTS;
    while (num_bytes > 0)
    {
        // Set address register to read.
        PMADRL = (address & 0x00FF);
        PMADRH = ((address & 0xFF00) >> 8);

        // Start read.
        PMCON1bits.CFGS = 0;
        PMCON1bits.RD = 1;
        NOP();
        NOP();

        // Retrieve data.
        *data_ptr = ((PMDATH << 8) | PMDATL);

        // Advance counters and pointers.
        address++;
        data_ptr++;
        num_bytes--;
    }
    DC2343_ENABLE_INTERRUPTS;

    return;
}

// Erase page according to datasheet Figure 11-4.
void pic16f_flash_erase_page(uint16_t block_address)
{
    DC2343_DISABLE_INTERRUPTS;

    // Set address register to erase.
    PMADRL = (block_address & 0xFF);
    PMADRH = ((block_address & 0xFF00) >> 8);

    // Start erase.
    PMCON1bits.CFGS = 0;
    PMCON1bits.FREE = 1;
    PMCON1bits.WREN = 1;

    // Unlock according to datasheet Figure 11-3.
    PMCON2 = 0x55;
    PMCON2 = 0xAA;
    PMCON1bits.WR = 1;      // Set WR bit to begin erase
    NOP();
    NOP();

    PMCON1bits.WREN = 0;       // Disable writes
    DC2343_ENABLE_INTERRUPTS;

    return;
}

// Write page according to datasheet Figure 11-6.
void pic16f_flash_write_page(uint16_t address, uint16_t* data_ptr)
{
    uint8_t num_bytes = _FLASH_WRITE_SIZE;

    DC2343_DISABLE_INTERRUPTS;
    PMCON1bits.CFGS = 0;
    PMCON1bits.WREN = 1;
    PMCON1bits.LWLO = 1;

    while(num_bytes > 0)
    {
        // Set address register to write.
        PMADRL = (address & 0xFF);
        PMADRH = ((address & 0xFF00) >> 8);

        // Set data.
        PMDATL = *data_ptr;
        PMDATH = ((*data_ptr & 0xFF00) >> 8);

        // Advance counters and pointers.
        address++;
        data_ptr++;
        num_bytes--;

        // Start Flash program memory write the last time through
        if(num_bytes == 0)
        {
            PMCON1bits.LWLO = 0;
        }

        // Unlock according to datasheet Figure 11-3.
        PMCON2 = 0x55;
        PMCON2 = 0xAA;
        PMCON1bits.WR = 1;
        NOP();
        NOP();
    }

    PMCON1bits.WREN = 0;       // Disable writes
    DC2343_ENABLE_INTERRUPTS;

    return;
}
