/*
 Linear Technology DC2343A Demonstration Board.
 PIC16 Driver File for I2C port.
 All datasheet references in this file refer to Microchip Technology Inc. document 40001639B.pdf.

 @verbatim
 This code provides an interface to the I2C port on the PIC16 used to communicate with the LTC3335.
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 750 $
 $Date: 2014-09-17 19:27:28 -0400 (Wed, 17 Sep 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

*/

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <xc.h>             // Defines the registers in the PIC.
#include "Typedefs.h"
#include "DC2343A.h"
#include "PIC16F_I2C.h"
#include "PIC16F_Timer.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#define PIC16F_I2C_BAUD         (400000)                                                                                    // I2C baud rate in Hz
#define pic16f_i2c_clock_delay() PIC16F_TIMER_DELAY_US((uint16_t)((US_PER_S + (PIC16F_I2C_BAUD - 1)) / PIC16F_I2C_BAUD))    // delay for I2C clock period (rounded up)
#define PIC16F_I2C_TIMEOUT      ((uint16_t)(10 * PIC16F_TIMER_TICKS_PER_SEC / MS_PER_S))                                    // I2C timeout in timer ticks

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void pic16f_i2c_recover(void);
bool_t pic16f_i2c_wait(void);

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Initializes I2C module
void PIC16F_I2C_Init(void)
{

    // Turn off I2C port before configuring.
    SSP1CON1bits.SSPEN = 0;

    // Recover the I2C if a slave is holding SDA low.
    pic16f_i2c_recover();

    // Init I/O according to datasheet section 12
    I2C_PIC_HW_SETUP;

    // Setup I2C according to datasheet section 22
    // Configure as I2C Master.
    SSP1CON1bits.SSP1M0 = 0;
    SSP1CON1bits.SSP1M1 = 0;
    SSP1CON1bits.SSP1M2 = 0;
    SSP1CON1bits.SSP1M3 = 1;
    SSP1CON1bits.CKP = 1;

    // Set the baud rate
#if (DC2343_OSCILLATOR_FREQUENCY == 48000000) && (PIC16F_I2C_BAUD == 400000)
    SSP1ADD = 29;               // This will cause PIC to communicate at 400kHz when DC2343_OSCILLATOR_FREQUENCY = 48000000.
#else
    #error This code module is designed for an PIC16 oscillator of 48MHz (FOSC).
#endif

    // Turn on configured I2C port.
    SSP1CON1bits.SSPEN = 1;

    return;
}

// Sends a start or a repeated start condition and waits for it to complete.
bool_t PIC16F_I2C_Start(bool_t repeated)
{
    PIR1bits.SSP1IF=0;          // clear SSP interrupt bit

    // Send the start bit.
    if (repeated == false)
    {
        SSP1CON2bits.SEN = 1;
    }
    else
    {
        SSP1CON2bits.RSEN = 1;
    }

    // Wait for i2c operation to be complete.
    return pic16f_i2c_wait();
}

// Writes a series of bytes to the I2C and waits for them to be sent.
bool_t PIC16F_I2C_Write(uint8_t* data_ptr, uint8_t num_bytes)
{
    bool_t success = true;

    // Write bytes
    while(num_bytes > 0)
    {
        PIR1bits.SSP1IF=0;          // clear SSP interrupt bit

        // Write one byte and adjust pointer/counter
        SSP1BUF = *data_ptr++;
        num_bytes--;

        // Wait for i2c operation to be complete.
        success &= pic16f_i2c_wait();

        // Save result of ACK.
        if (SSP1CON2bits.ACKSTAT == 1)
        {
            success = false;
        }
    }

    // Return true only if all bytes were acknowledged.
    return success;
}

// Reads a series of bytes from the I2C.
bool_t PIC16F_I2C_Read(uint8_t* data_ptr, uint8_t num_bytes)
{
    bool_t success = true;

    while(num_bytes > 0)
    {
        PIR1bits.SSP1IF=0;          // clear SSP interrupt bit

        // Start receiving the byte.
        SSP1CON2bits.RCEN = 1;

        // Wait for i2c operation to be complete.
        success &= pic16f_i2c_wait();

        // Read one byte and adjust pointer/counter
        *data_ptr++ = SSP1BUF;
        num_bytes--;

        PIR1bits.SSP1IF=0;          // clear SSP interrupt bit

        // ACK received bytes, NAK the last.
        SSP1CON2bits.ACKDT = num_bytes ? 0 : 1;
        SSP1CON2bits.ACKEN = 1;

        // Wait for i2c operation to be complete.
        success &= pic16f_i2c_wait();
    }

    return true;
}

// Sends a stop bit and waits for it to complete.
bool_t PIC16F_I2C_Stop(void)
{
    PIR1bits.SSP1IF=0;          // clear SSP interrupt bit

    // send stop bit
    SSPCON2bits.PEN=1;

    // Wait for i2c operation to be complete.
    return pic16f_i2c_wait();
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Bit-bangs i2c to recover the bus from a condition where a slave is holding SDA low, preventing the PIC hardware from sending start and stop conditions.
void pic16f_i2c_recover(void)
{
    // Disable I2C module so bit banging can be done
    SSP1CON1bits.SSPEN = 0;

    // Check if SDA is stuck low
    I2C_SDA_IN_PIN_SETUP;
    if (I2C_SDA_IN_PIN == 0)
    {
        // Setup to bitbang SDA and SCL out
        I2C_SCL_OUT_PIN_SETUP;
        I2C_SDA_OUT_PIN_SETUP;

        // Delay with clock low before sending reset
        pic16f_i2c_clock_delay();

        // Send Start Condition
        I2C_SCL_OUT_PIN = 1;
        pic16f_i2c_clock_delay();
        I2C_SDA_OUT_PIN = 0;
        pic16f_i2c_clock_delay();
        I2C_SCL_OUT_PIN = 0;
        pic16f_i2c_clock_delay();

        // Clock 9 data bytes with SDA high
        for (uint8_t clock_num = 0; clock_num < 9; clock_num++)
        {
            I2C_SDA_OUT_PIN = 1;
            I2C_SCL_OUT_PIN = 1;
            pic16f_i2c_clock_delay();
            I2C_SCL_OUT_PIN = 0;
            pic16f_i2c_clock_delay();
        }

        // Send Start Condition
        I2C_SCL_OUT_PIN = 1;
        pic16f_i2c_clock_delay();
        I2C_SDA_OUT_PIN = 0;
        pic16f_i2c_clock_delay();
        I2C_SCL_OUT_PIN = 0;
        pic16f_i2c_clock_delay();

        // Send Stop Condition
        I2C_SCL_OUT_PIN = 1;
        pic16f_i2c_clock_delay();
        I2C_SDA_OUT_PIN = 1;
        pic16f_i2c_clock_delay();
        I2C_SCL_OUT_PIN = 0;
        pic16f_i2c_clock_delay();
    }

    return;
}

// Waits for I2C operation to complete, or for a timeout to occur.
bool_t pic16f_i2c_wait(void)
{
    uint16_t time_start = PIC16F_Timer_Get16();
    // Wait for interrupt flag to go high indicating transmission is complete
    while(!PIR1bits.SSP1IF)
    {
        if ((PIC16F_Timer_Get16() - time_start) >= PIC16F_I2C_TIMEOUT)
        {
            // timeout
            return false;
        }
    }
    return true;
}
