/*
 Linear Technology DC2343A Demonstration Board.
 PIC16 Driver File for Timer1.
 All datasheet references in this file refer to Microchip Technology Inc. document 40001639B.pdf.

 @verbatim
 This code uses Timer1 of the PIC16 to provide time measurements, time stamps, and delays to the other code modules.

 To extend the range of the timer, a firmware variable is updated upon overflow of the hw value in an ISR.
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 434 $
 $Date: 2014-06-16 10:32:37 -0400 (Mon, 16 Jun 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

 */

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <xc.h>         // Defines the registers in the PIC.
#include "Typedefs.h"
#include "DC2343A.h"
#include "PIC16F_Timer.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
uint16_t pic16f_timer_msw;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

void PIC16F_Timer_Init(void)
{
    // Setup timer according to datasheet section 20

    T1CONbits.TMR1ON = 0;   // Start with timer off
    T1CONbits.TMR1CS = 0;   // Use Instruction Clock for timer.

    // Set the prescaler for Timer0
#if (PIC16F_TIMER_PRESCALE == 8)
    T1CONbits.T1CKPS = 3;
#elif (PIC16F_TIMER_PRESCALE == 4)
    T1CONbits.T1CKPS = 2;
#elif (PIC16F_TIMER_PRESCALE == 2)
    T1CONbits.T1CKPS = 1;
#elif (PIC16F_TIMER_PRESCALE == 1)
    T1CONbits.T1CKPS = 0;
#else
#error Invalid divisor specified for PIC16F_TIMER_PRESCALE.
#endif

    T1CONbits.T1OSCEN = 0;      // Disable dedicated oscillator.
    T1CONbits.nT1SYNC = 0;      // No asynchronous clock.

    // Start with timer cleared.
    pic16f_timer_msw = 0;
    TMR1H = 0;
    TMR1L = 0;

    // Configure Interrupt accoding to section 8 of the datasheet.
    PIR1bits.TMR1IF = 0;                // Clear the interrupt flag.
    PIE1bits.TMR1IE = 1;                // Enable the interrupt.
    INTCONbits.PEIE = 1;                // Enable the peripheral interrupts (TMR1 being a peripheral and all).

    // Start the Timer
    T1CONbits.TMR1ON = 1;

}

// Update the 16 bit counter from the hw timer and return the updated value
void PIC16F_Timer_ISR(void)
{
    // Check flag as there's only one interrupt vector in PIC16.
    if (PIR1bits.TMR1IF == 1)
    {
        // Update most significant word of timer.
        pic16f_timer_msw++;

        // Clear the interrupt flag.
        PIR1bits.TMR1IF = 0;
    }

    return;
}

// Get the 16 bit counter from the hw timer
uint16_t PIC16F_Timer_Get16(void)
{
    uint16_t low_byte;
    uint16_t high_byte;

    do
     {
         // Get the most significant byte first, and save so we can detect if overflow occurred during read
        high_byte = TMR1H;
        low_byte = TMR1L;
         // If the msb changed while reading the lsb, then read everything again.
     } while(TMR1H != high_byte);

    return ((uint16_t) high_byte << 8) + low_byte;
}

// Get the 32 bit counter from the hw/fw timer
uint32_t PIC16F_Timer_Get32(void)
{
    uint16_t lsw_copy;
    uint16_t msw_copy;

    do
    {
        // Get the most significant word first, and save so we can detect if overflow occurred during read
        msw_copy = pic16f_timer_msw;
        lsw_copy = PIC16F_Timer_Get16();
        // If the msw changed while reading the lsw, then read everything again.
    } while(pic16f_timer_msw != msw_copy);

    return (((uint32_t) msw_copy) << 16) + lsw_copy;
}

// Wait for the specified number of timer ticks
void PIC16F_Timer_Delay(uint32_t delay_ticks)
{
    if (UPPER_WORD(delay_ticks) == 0)
    {
        uint16_t time_start = PIC16F_Timer_Get16();
        uint16_t time_next;

        do
        {
            time_next = PIC16F_Timer_Get16();
        } while ((time_next - time_start) < LOWER_WORD(delay_ticks));
    }
    else
    {
        uint32_t time_start = PIC16F_Timer_Get32();
        uint32_t time_next;

        do
        {
            time_next = PIC16F_Timer_Get32();
        } while((time_next - time_start) < delay_ticks);
    }

    return;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

