/*
 Linear Technology DC2343A Demonstration Board.
 DC2343A Application File for displaying the status of DC2343A system.

 @verbatim
 This code converts the ADC values in voltage measurements of VBAT and VOUT, and uses these measurements to set
 a state of the system.  If sufficient VBAT is available to communicate with the LTC3335 the green state LED
 will flash slowly to indicate the PCB is operating properly.  If VBAT is not sufficient for communication with
 the LTC3335, the green state LED will flash quickly to notify the user of this condition.

 If the DC2343A is not connected to a GUI, VBAT will be read periodically to update the state.  If the DC2343A is
 connected to a GUI, the GUI will control the rate at which VBAT is red.

 The amber LED is turned on when communication occurs with the LTC3335, and turned off after enough of a delay that
 the lumination can be early seen by a customer.

 To extend the range of the timer, a firmware variable is updated upon overflow of the hw value in an ISR.
 @endverbatim

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 434 $
 $Date: 2014-06-16 10:32:37 -0400 (Mon, 16 Jun 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

 */

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "Typedefs.h"
#include "DC2343A.h"
#include "Status.h"
#include "PIC16F_Timer.h"
#include "PIC16F_ADC.h"
#include "PIC16F_Flash.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#define GUI_STATE_BIT 7

typedef enum
{
    STATUS_STATE_BATTERY_NOT_ATTACHED,
    STATUS_STATE_BATTERY_ATTACHED,
    STATUS_STATE_BATTERY_NOT_ATTACHED_GUI = STATUS_STATE_BATTERY_NOT_ATTACHED | MASK(1, GUI_STATE_BIT),
    STATUS_STATE_BATTERY_ATTACHED_GUI = STATUS_STATE_BATTERY_NOT_ATTACHED | MASK(1, GUI_STATE_BIT),
    STATUS_STATE_NUM_STATES
} STATUS_STATE_TYPE;

#define STATUS_BATTERY_ATTACHED_THRESHOLD          ((uint16_t) (1.7 * STATUS_ADC_COUNTS_PER_VOLT))
#define STATUS_BATTERY_NOT_ATTACHED_HYSTERESIS     ((uint16_t) (0.1 * STATUS_ADC_COUNTS_PER_VOLT))

#define STATUS_ADC_COUNTS_PER_VOLT    1000
#define STATUS_ADC_CAL_DEFAULT        1382L
#define STATUS_ADC_CAL_DIVISOR_SHIFT  8

#define STATUS_AMBER_LED_TIME                       ((uint32_t)(0.02 * PIC16F_TIMER_TICKS_PER_SEC))

#define STATUS_GUI_TIMEOUT                          ((uint32_t)(20 * PIC16F_TIMER_TICKS_PER_SEC))

#define STATUS_STATE_BATTERY_NOT_ATTACHED_FLASH_TIME    ((uint32_t)(0.2 * PIC16F_TIMER_TICKS_PER_SEC))
#define STATUS_STATE_BATTERY_ATTACHED_FLASH_TIME        ((uint32_t)(1.0 * PIC16F_TIMER_TICKS_PER_SEC))

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
STATUS_STATE_TYPE status_state;
uint32_t status_green_led_timer;
uint32_t status_amber_led_timer;
uint32_t status_gui_timer;
uint32_t status_green_led_flash_time;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void status_vbat_test(uint16_t vbat_adc_value);

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Initializes the state and timers used to display the status of the DC2343A System.
void Status_Init(void)
{
    uint32_t timer_new = PIC16F_Timer_Get32();

    LED_GREEN_OUT_PIN = 0;
    LED_AMBER_OUT_PIN = 0;

    status_state = STATUS_STATE_BATTERY_NOT_ATTACHED;

    status_green_led_timer = timer_new;
    status_amber_led_timer = timer_new;
    status_gui_timer = timer_new;

    status_green_led_flash_time = STATUS_STATE_BATTERY_NOT_ATTACHED_FLASH_TIME;
    return;
}

// Task updates the system status state and timers.
void Status_Task(void)
{
    // Read the latest value of the free running timer.
    uint32_t timer_new = PIC16F_Timer_Get32();

    // Flash the green LED as specified by state.
    if((timer_new - status_green_led_timer) > status_green_led_flash_time)
    {
        status_green_led_timer += status_green_led_flash_time;

        if(LED_GREEN_OUT_PIN == 1)
        {
            LED_GREEN_OUT_PIN = 0;
        }
        else
        {
            LED_GREEN_OUT_PIN = 1;

            // If not in a non-GUI state, read the ADCs and use to determine the next state.
            if(status_state == (status_state & ~MASK(1, GUI_STATE_BIT)))
            {
                uint16_t vbat_adc_value;
                ENABLE_ADC_OUT_PIN = 1;
                PIC16F_ADC_Read(VBAT_ADC_CHANNEL, &vbat_adc_value);
                ENABLE_ADC_OUT_PIN = 0;
                vbat_adc_value = Status_Adc_Convert(vbat_adc_value, PIC16F_Flash_Data.vbat_adc_cal);
                status_vbat_test(vbat_adc_value);
            }
            // If in a GUI state, check to see if we've been disconnected long enough to re-enter a non-GUI state.
            else
            {
                if((timer_new - status_gui_timer) > STATUS_GUI_TIMEOUT)
                {
                    status_state &= ~MASK(1, GUI_STATE_BIT);
                }
            }
        }
    }

    // Turn off the amber LED after it's been on long enough for a human to see it.
    if(LED_AMBER_OUT_PIN == 1)
    {
        if((timer_new - status_amber_led_timer) > STATUS_AMBER_LED_TIME)
        {
            LED_AMBER_OUT_PIN = 0;
        }
    }

    return;
}

// Turns on the Amber Comm LED and sets timer to turn it off after brief delay.
void Status_LTC3335_Comm_Set(void)
{
    // Start the timer so that we'll turn off the amber LED after it's been on long enough for a human to see it.
    status_amber_led_timer = PIC16F_Timer_Get32();

    // Set the state depending upon whether VBAT is too high or to low
    LED_AMBER_OUT_PIN = 1;

    return;
}

// Function called when the GUI is commanding ADC measurements making it unecessary for the System Task to command its own ADC
// measurements for the purpose of updating the state.
void Status_GUI_Set(uint16_t vbat_adc_value)
{
    // Reset the GUI timer as we are still getting commands.
    status_gui_timer = PIC16F_Timer_Get32();

    // Set the state depending upon whether VBAT is too high or to low
    status_vbat_test(vbat_adc_value);

    return;
}

// Calculates the calibration factor necessary for the current ADC reading to be intpretted as the desired voltage measurement value.
int8_t Status_Adc_Calibrate(uint16_t adc_value, uint16_t desired_value)
{
    int32_t temp_32;

    if ((adc_value == 0) || (desired_value == 0))  return 0;

    temp_32 = ((1L * desired_value) << STATUS_ADC_CAL_DIVISOR_SHIFT) - (adc_value * STATUS_ADC_CAL_DEFAULT);
    return (temp_32 + adc_value/2) / adc_value;
}

// Performs an ADC conversion and applies a calibration factor.
uint16_t Status_Adc_Convert(uint16_t adc_value, int8_t adc_cal)
{
    uint32_t temp_u32;

    temp_u32 = adc_value * (STATUS_ADC_CAL_DEFAULT + adc_cal);
    return (temp_u32 + (1 << (STATUS_ADC_CAL_DIVISOR_SHIFT-1))) >> STATUS_ADC_CAL_DIVISOR_SHIFT;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Tests if sufficient VBAT is available to communicate with the LTC3335.
void status_vbat_test(uint16_t vbat_adc_value)
{
    // Create a neutral copy of the state, so that the GUI and non-GUI states can be tested the same way
    STATUS_STATE_TYPE state_neutral = status_state & ~MASK(1, GUI_STATE_BIT);

    // If looking for battery to attach, look for VBAT above threshold
    if (state_neutral == STATUS_STATE_BATTERY_NOT_ATTACHED)
    {
        if(vbat_adc_value > STATUS_BATTERY_ATTACHED_THRESHOLD)
        {
            status_state = STATUS_STATE_BATTERY_ATTACHED | (status_state & MASK(1, GUI_STATE_BIT));
            status_green_led_flash_time = STATUS_STATE_BATTERY_ATTACHED_FLASH_TIME;
        }
    }
    // If looking for battery to detach, look for VBAT below threshold - hysteresis
    else
    {
        if(vbat_adc_value < (STATUS_BATTERY_ATTACHED_THRESHOLD - STATUS_BATTERY_NOT_ATTACHED_HYSTERESIS))
        {
            status_state = STATUS_STATE_BATTERY_NOT_ATTACHED | (status_state & MASK(1, GUI_STATE_BIT));
            status_green_led_flash_time = STATUS_STATE_BATTERY_NOT_ATTACHED_FLASH_TIME;
        }        
    }

    return;
}
