/*
 Linear Technology DC2343A Demonstration Board.
 DC2343A Application File for data packed and passed through the USB.

 The pieces of data that are passed through USB are identified by a data id defined by USB_DATA_ID_TYPE.

 The behavior of each data id is defined by 2 functions:
 - bool_t USB_Data_<data_id>_Command(uint8_t board_num, uint8_t* data_ptr) - process a command for this data.
 - bool_t USB_Data_<data_id>_Response(uint8_t board_num, uint8_t* data_ptr) - send a response for this data id.

 Most data ids accept one of more commands, and send a response after a command is received. 

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 827 $
 $Date: 2014-10-15 11:40:07 -0400 (Wed, 15 Oct 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

 */

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "Typedefs.h"
#include "DC2343A.h"
#include "USB_Data.h"
#include "PIC16F_Timer.h"
#include "PIC16F_ADC.h"
#include "PIC16F_Flash.h"
#include "LTC3335.h"
#include "Error.h"
#include "Status.h"
#include <string.h>

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// **************************
// USB_DATA_ID_FW_REVISION
typedef struct
{
    uint8_t rev_major;
    uint8_t rev_minor;
    uint8_t rev_subminor;
    uint8_t rev_beta;
} USB_DATA_ID_FW_REVISION_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_SERIAL_NUMBER
typedef struct
{
    uint8_t serial_number[DC2343_SERIAL_NUM_SIZE];
} USB_DATA_ID_SERIAL_NUMBER_COMMAND_TYPE;

typedef struct
{
    uint8_t serial_number[DC2343_SERIAL_NUM_SIZE];
} USB_DATA_ID_SERIAL_NUMBER_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_ERROR_DATA
typedef struct
{
    uint8_t error_clear;
} USB_DATA_ID_ERROR_DATA_COMMAND_TYPE;

typedef struct
{
    uint8_t error_code_current;
    uint8_t error_data[ERROR_DATA_SIZE];
    uint16_t error_count[ERROR_NUM_CODES];
} USB_DATA_ID_ERROR_DATA_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_VOLTAGE_SELECTION
typedef struct
{
    uint8_t enabled;
    uint8_t voltage_selection;
} USB_DATA_ID_VOLTAGE_SELECTION_COMMAND_TYPE;

typedef struct
{
    uint8_t enabled;
    uint8_t voltage_selection;
} USB_DATA_ID_VOLTAGE_SELECTION_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_PRESCALER_SELECTION
typedef struct
{
    uint8_t prescaler;
} USB_DATA_ID_PRESCALER_SELECTION_COMMAND_TYPE;

typedef struct
{
    uint8_t prescaler;
} USB_DATA_ID_PRESCALER_SELECTION_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_IPEAK_CONFIGURATION
typedef struct
{
    uint8_t ipeak_setting;
} USB_DATA_ID_IPEAK_CONFIGURATION_COMMAND_TYPE;

typedef struct
{
    uint8_t ipeak_setting;
} USB_DATA_ID_IPEAK_CONFIGURATION_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_VOUT_CONFIGURATION
typedef struct
{
    uint8_t vout_setting;
} USB_DATA_ID_VOUT_CONFIGURATION_COMMAND_TYPE;

typedef struct
{
    uint8_t vout_setting;
} USB_DATA_ID_VOUT_CONFIGURATION_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_ALARM_THRESHOLD
typedef struct
{
    uint8_t threshold;
} USB_DATA_ID_ALARM_THRESHOLD_COMMAND_TYPE;

typedef struct
{
    uint8_t threshold;
} USB_DATA_ID_ALARM_THRESHOLD_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_ACCUMULATOR
typedef struct
{
    uint8_t accumulator;
} USB_DATA_ID_ACCUMULATOR_COMMAND_TYPE;

typedef struct
{
    uint8_t accumulator;
} USB_DATA_ID_ACCUMULATOR_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_ALARMS

typedef struct
{
    uint8_t alarms;
} USB_DATA_ID_ALARMS_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_CLEAR_INT
typedef struct
{
    bool_t clear_int;
} USB_DATA_ID_CLEAR_INT_COMMAND_TYPE;

typedef struct
{
    bool_t clear_int;
} USB_DATA_ID_CLEAR_INT_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_COUNTER_TEST
typedef struct
{
    bool_t enabled;
} USB_DATA_ID_COUNTER_TEST_COMMAND_TYPE;

typedef struct
{
    bool_t enabled;
    uint32_t edge_count;
    uint32_t time;
} USB_DATA_ID_COUNTER_TEST_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_IRQ

typedef struct
{
    bool_t high;
} USB_DATA_ID_IRQ_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_PGOOD

typedef struct
{
    bool_t high;
} USB_DATA_ID_PGOOD_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_ADC

typedef struct
{
    uint8_t adc_select;
    uint16_t voltage;
} USB_DATA_ID_ADC_COMMAND_TYPE;

typedef struct
{
    uint16_t vbat;
    uint16_t vout;
} USB_DATA_ID_ADC_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_CAPACITY

typedef struct
{
    float capacity;
} USB_DATA_ID_CAPACITY_COMMAND_TYPE;

typedef struct
{
    float capacity;
} USB_DATA_ID_CAPACITY_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_NONSTANDARD_INDUCTOR
typedef struct
{
    uint8_t nonstandard_inductor[DC2343_NONSTANDARD_INDUCTOR_SIZE];
} USB_DATA_ID_NONSTANDARD_INDUCTOR_COMMAND_TYPE;

typedef struct
{
    uint8_t nonstandard_inductor[DC2343_NONSTANDARD_INDUCTOR_SIZE];
} USB_DATA_ID_NONSTANDARD_INDUCTOR_RESPONSE_TYPE;

// **************************
// USB_DATA_ID_EN_CONFIGURATION
typedef struct
{
    uint8_t en_setting;
} USB_DATA_ID_EN_CONFIGURATION_COMMAND_TYPE;

typedef struct
{
    uint8_t en_setting;
} USB_DATA_ID_EN_CONFIGURATION_RESPONSE_TYPE;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
USB_DATA_TABLE_TYPE const usb_data_table[USB_DATA_ID_NUM_COMMANDS] = {
    {USB_Data_Fw_Revision_Command, USB_Data_Fw_Revision_Response, sizeof(USB_DATA_ID_FW_REVISION_RESPONSE_TYPE)},
    {USB_Data_Serial_Number_Command, USB_Data_Serial_Number_Response, sizeof(USB_DATA_ID_SERIAL_NUMBER_RESPONSE_TYPE)},
    {USB_Data_Error_Data_Command, USB_Data_Error_Data_Response, sizeof(USB_DATA_ID_ERROR_DATA_RESPONSE_TYPE)},
    {USB_Data_Voltage_Selection_Command, USB_Data_Voltage_Selection_Response, sizeof(USB_DATA_ID_VOLTAGE_SELECTION_RESPONSE_TYPE)},
    {USB_Data_Prescaler_Selection_Command, USB_Data_Prescaler_Selection_Response, sizeof(USB_DATA_ID_PRESCALER_SELECTION_RESPONSE_TYPE)},
    {USB_Data_Ipeak_Configuration_Command, USB_Data_Ipeak_Configuration_Response, sizeof(USB_DATA_ID_IPEAK_CONFIGURATION_RESPONSE_TYPE)},
    {USB_Data_Vout_Configuration_Command, USB_Data_Vout_Configuration_Response, sizeof(USB_DATA_ID_VOUT_CONFIGURATION_RESPONSE_TYPE)},
    {USB_Data_Alarm_Threshold_Command, USB_Data_Alarm_Threshold_Response, sizeof(USB_DATA_ID_ALARM_THRESHOLD_RESPONSE_TYPE)},
    {USB_Data_Accumulator_Command, USB_Data_Accumulator_Response, sizeof(USB_DATA_ID_ACCUMULATOR_RESPONSE_TYPE)},
    {USB_Data_Alarms_Command, USB_Data_Alarms_Response, sizeof(USB_DATA_ID_ALARMS_RESPONSE_TYPE)},
    {USB_Data_Clear_Int_Command, USB_Data_Clear_Int_Response, sizeof(USB_DATA_ID_CLEAR_INT_RESPONSE_TYPE)},
    {USB_Data_Counter_Test_Command, USB_Data_Counter_Test_Response, sizeof(USB_DATA_ID_COUNTER_TEST_RESPONSE_TYPE)},
    {USB_Data_Irq_Command, USB_Data_Irq_Response, sizeof(USB_DATA_ID_IRQ_RESPONSE_TYPE)},
    {USB_Data_Pgood_Command, USB_Data_Pgood_Response, sizeof(USB_DATA_ID_PGOOD_RESPONSE_TYPE)},
    {USB_Data_Adc_Command, USB_Data_Adc_Response, sizeof(USB_DATA_ID_ADC_RESPONSE_TYPE)},
    {USB_Data_Capacity_Command, USB_Data_Capacity_Response, sizeof(USB_DATA_ID_CAPACITY_RESPONSE_TYPE)},
    {USB_Data_Nonstandard_Inductor_Command, USB_Data_Nonstandard_Inductor_Response, sizeof(USB_DATA_ID_NONSTANDARD_INDUCTOR_RESPONSE_TYPE)},
    {USB_Data_En_Configuration_Command, USB_Data_En_Configuration_Response, sizeof(USB_DATA_ID_EN_CONFIGURATION_RESPONSE_TYPE)}
};

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

uint16_t USB_Data_Swap_Byte_Order_16(uint16_t data)
{
    uint16_t temp;
    ((uint8_t*)(&temp))[0] = UPPER_BYTE(data);
    ((uint8_t*)(&temp))[1] = LOWER_BYTE(data);

    return temp;
}

uint32_t USB_Data_Swap_Byte_Order_32(uint32_t data)
{
    uint32_t temp;
    ((uint8_t*)&(temp))[0] = UPPER_BYTE(UPPER_WORD(data));
    ((uint8_t*)&(temp))[1] = LOWER_BYTE(UPPER_WORD(data));
    ((uint8_t*)&(temp))[2] = UPPER_BYTE(LOWER_WORD(data));
    ((uint8_t*)&(temp))[3] = LOWER_BYTE(LOWER_WORD(data));

    return temp;
}

// **************************
// USB_DATA_ID_FW_REVISION

bool_t USB_Data_Fw_Revision_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_FW_REVISION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_FW_REVISION_RESPONSE_TYPE*) data_ptr;

    // Send firmware revision.
    response_data_ptr->rev_major =  DC2343_FW_REV_MAJOR;
    response_data_ptr->rev_minor = DC2343_FW_REV_MINOR;
    response_data_ptr->rev_subminor = DC2343_FW_REV_SUB_MINOR;
    response_data_ptr->rev_beta = DC2343_FW_REV_BETA;
    success = true;

    return success;
}

// **************************
// USB_DATA_ID_SERIAL_NUMBER
bool_t USB_Data_Serial_Number_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_SERIAL_NUMBER_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_SERIAL_NUMBER_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.serial_number, (uint8_t*)&(command_data_ptr->serial_number), sizeof((command_data_ptr->serial_number)));
        success = true;
    }

    return success;
}

bool_t USB_Data_Serial_Number_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_SERIAL_NUMBER_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_SERIAL_NUMBER_RESPONSE_TYPE*) data_ptr;
    uint8_t* byte_ptr = response_data_ptr->serial_number;
    
    for(uint8_t byte_num = 0; byte_num < sizeof(response_data_ptr->serial_number); byte_num++)
    {
        byte_ptr[byte_num] = PIC16F_Flash_Data.serial_number[byte_num];
    }

    return success;
}

// **************************
// USB_DATA_ID_ERROR_DATA
bool_t USB_Data_Error_Data_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_ERROR_DATA_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_ERROR_DATA_RESPONSE_TYPE*) data_ptr;

    uint8_t* error_data_ptr;
    response_data_ptr->error_code_current = (uint8_t)Error_Data_Get(&error_data_ptr);
    
    for(uint8_t error_data_num = 0; error_data_num < ERROR_DATA_SIZE; error_data_num++)
    {
        response_data_ptr->error_data[error_data_num] = error_data_ptr[error_data_num];
    }

    for(ERROR_CODE_TYPE error_code = 0; error_code < ERROR_NUM_CODES; error_code++)
    {
        response_data_ptr->error_count[error_code] = USB_Data_Swap_Byte_Order_16(Error_Count_Get(error_code));
    }

    Error_Data_Ptr_Clear();

    return success;
}

// **************************
// USB_DATA_ID_VOLTAGE_SELECTION

bool_t USB_Data_Voltage_Selection_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_VOLTAGE_SELECTION_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_VOLTAGE_SELECTION_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        bool_t enabled;
        LTC3335_OUTPUT_VOLTAGE_TYPE voltage;
        enabled = ((command_data_ptr->enabled) == 0) ?  false : true;
        voltage = command_data_ptr->voltage_selection;
        success = LTC3335_Set_Voltage(enabled, voltage);
    }

    return success;
}

bool_t USB_Data_Voltage_Selection_Response(uint8_t* data_ptr)
{
    bool_t enabled;
    LTC3335_OUTPUT_VOLTAGE_TYPE voltage;
    bool_t success = false;
    USB_DATA_ID_VOLTAGE_SELECTION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_VOLTAGE_SELECTION_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Voltage(&enabled, &voltage) == true)
    {
        response_data_ptr->enabled = (enabled == false) ? 0 : 1;
        response_data_ptr->voltage_selection = voltage;
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_PRESCALER_SELECTION
bool_t USB_Data_Prescaler_Selection_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_PRESCALER_SELECTION_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_PRESCALER_SELECTION_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        success = LTC3335_Set_Prescaler(command_data_ptr->prescaler);
    }

    return success;
}

bool_t USB_Data_Prescaler_Selection_Response(uint8_t* data_ptr)
{
    uint8_t prescaler;
    bool_t success = false;
    USB_DATA_ID_PRESCALER_SELECTION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_PRESCALER_SELECTION_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Prescaler(&prescaler) == true)
    {
        response_data_ptr->prescaler =  prescaler;
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_IPEAK_CONFIGURATION
bool_t USB_Data_Ipeak_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_IPEAK_CONFIGURATION_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_IPEAK_CONFIGURATION_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.ipeak_setting, (uint8_t*)&(command_data_ptr->ipeak_setting), sizeof(command_data_ptr->ipeak_setting));
        success = true;
    }

    return success;
}

bool_t USB_Data_Ipeak_Configuration_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_IPEAK_CONFIGURATION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_IPEAK_CONFIGURATION_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->ipeak_setting = PIC16F_Flash_Data.ipeak_setting;

    return success;
}

// **************************
// USB_DATA_ID_VOUT_CONFIGURATION
bool_t USB_Data_Vout_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_VOUT_CONFIGURATION_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_VOUT_CONFIGURATION_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.vout_setting, (uint8_t*)&(command_data_ptr->vout_setting), sizeof(command_data_ptr->vout_setting));
        success = true;
    }

    return success;
}

bool_t USB_Data_Vout_Configuration_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_VOUT_CONFIGURATION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_VOUT_CONFIGURATION_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->vout_setting = PIC16F_Flash_Data.vout_setting;

    return success;
}

// **************************
// USB_DATA_ID_ALARM_THRESHOLD
bool_t USB_Data_Alarm_Threshold_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_ALARM_THRESHOLD_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_ALARM_THRESHOLD_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        success = LTC3335_Set_Alarm_Threshold(command_data_ptr->threshold);
    }

    return success;
}

bool_t USB_Data_Alarm_Threshold_Response(uint8_t* data_ptr)
{
    uint8_t threshold;
    bool_t success = false;
    USB_DATA_ID_ALARM_THRESHOLD_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_ALARM_THRESHOLD_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Alarm_Threshold(&threshold) == true)
    {
        response_data_ptr->threshold = threshold;
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_ACCUMULATOR
bool_t USB_Data_Accumulator_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_ACCUMULATOR_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_ACCUMULATOR_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        success = LTC3335_Set_Accumulator(command_data_ptr->accumulator);
    }

    return success;
}

bool_t USB_Data_Accumulator_Response(uint8_t* data_ptr)
{
    uint8_t accumulator;
    bool_t success = false;
    USB_DATA_ID_ACCUMULATOR_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_ACCUMULATOR_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Accumulator(&accumulator) == true)
    {
        response_data_ptr->accumulator = accumulator;
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_ALARMS

bool_t USB_Data_Alarms_Response(uint8_t* data_ptr)
{
    LTC3335_ALARM_TYPE alarms;
    bool_t success = false;
    USB_DATA_ID_ALARMS_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_ALARMS_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Alarms(&alarms) == true)
    {
        response_data_ptr->alarms = 0;
        if(alarms.ac_on_time_overflow) response_data_ptr->alarms |= MASK(1,0);
        if(alarms.coulomb_counter_overflow) response_data_ptr->alarms |= MASK(1,1);
        if(alarms.alarm_trip) response_data_ptr->alarms |= MASK(1,2);
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_CLEAR_INT
bool_t USB_Data_Clear_Int_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_CLEAR_INT_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_CLEAR_INT_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        if(command_data_ptr->clear_int != 0)
        {
            success = LTC3335_Clear_Int();
        }
        else
        {
            success = true;
        }
    }

    return success;
}

bool_t USB_Data_Clear_Int_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_CLEAR_INT_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_CLEAR_INT_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->clear_int =  1;

    success = true;

    return success;
}

// **************************
// USB_DATA_ID_COUNTER_TEST
bool_t USB_Data_Counter_Test_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_COUNTER_TEST_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_COUNTER_TEST_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        success = LTC3335_Set_Counter_Test(command_data_ptr->enabled);
        success = LTC3335_Get_Counter_Test(&command_data_ptr->enabled);
    }
    else if(subcommand == USB_DATA_SUBCOMMAND_RESET)
    {
        success = LTC3335_Reset_Counter_Test();
    }
    else if(subcommand == USB_DATA_SUBCOMMAND_REFRESH)
    {
        success = LTC3335_Get_Counter_Test(&command_data_ptr->enabled);
    }

    return success;
}

bool_t USB_Data_Counter_Test_Response(uint8_t* data_ptr)
{
    bool_t enabled;
    uint32_t edge_count;
    uint32_t time;
    bool_t success = false;
    USB_DATA_ID_COUNTER_TEST_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_COUNTER_TEST_RESPONSE_TYPE*) data_ptr;

    if(LTC3335_Get_Counter_Test_Results(&enabled, &edge_count, &time) == true)
    {
        response_data_ptr->enabled = enabled;
        response_data_ptr->edge_count = USB_Data_Swap_Byte_Order_32(edge_count);
        response_data_ptr->time = USB_Data_Swap_Byte_Order_32(time);
        success = true;
    }

    return success;
}

// **************************
// USB_DATA_ID_IRQ

bool_t USB_Data_Irq_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_IRQ_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_IRQ_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->high = LTC3335_IRQ;

    success = true;

    return success;
}

// **************************
// USB_DATA_ID_PGOOD

bool_t USB_Data_Pgood_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_PGOOD_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_PGOOD_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->high = LTC3335_PGOOD;

    success = true;

    return success;
}

// **************************
// USB_DATA_ID_ADC
bool_t USB_Data_Adc_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_ADC_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_ADC_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        uint8_t adc_channel;
        uint16_t cal_address;
        uint16_t adc_value;

        if(command_data_ptr->adc_select == 0)
        {
            cal_address = (uint16_t)(&PIC16F_Flash_Data.vbat_adc_cal);
            adc_channel = VBAT_ADC_CHANNEL;
        }
        else
        {
            cal_address = (uint16_t)(&PIC16F_Flash_Data.vout_adc_cal);
            adc_channel = VOUT_ADC_CHANNEL;
        }

        // Read the current adc value
        ENABLE_ADC_OUT_PIN = 1;
        {
            uint8_t sample_num;
            uint16_t adc_value_sum = 0;
            for(sample_num = 0; sample_num < 4; sample_num++)
            {
                success |= PIC16F_ADC_Read(adc_channel, &adc_value);
                adc_value_sum += adc_value;
            }
            adc_value = (adc_value_sum + 1) >> 2;
        }
        ENABLE_ADC_OUT_PIN = 0;

        // Calculate the calibration factor that will result in the ADC value converting to the desired value.
        if (success == true)
        {
            int8_t new_cal = Status_Adc_Calibrate(adc_value, USB_Data_Swap_Byte_Order_16(command_data_ptr->voltage));
            PIC16F_Flash_Write(cal_address, (uint8_t*)&new_cal, sizeof(new_cal));
        }
    }

    return success;
}

bool_t USB_Data_Adc_Response(uint8_t* data_ptr)
{
    uint16_t vbat_adc_value, vout_adc_value;
    bool_t success = true;
    USB_DATA_ID_ADC_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_ADC_RESPONSE_TYPE*) data_ptr;

    // Read the ADC values
    ENABLE_ADC_OUT_PIN = 1;
    success |= PIC16F_ADC_Read(VBAT_ADC_CHANNEL, &vbat_adc_value);
    success |= PIC16F_ADC_Read(VOUT_ADC_CHANNEL, &vout_adc_value);
    ENABLE_ADC_OUT_PIN = 0;

    // Calibrate the ADC values
    vbat_adc_value = Status_Adc_Convert(vbat_adc_value, PIC16F_Flash_Data.vbat_adc_cal);
    vout_adc_value = Status_Adc_Convert(vout_adc_value, PIC16F_Flash_Data.vout_adc_cal);

    // Use the vbat ADC value to set the system state.
    Status_GUI_Set(vbat_adc_value);

    // Send response
    response_data_ptr->vbat = USB_Data_Swap_Byte_Order_16(vbat_adc_value);
    response_data_ptr->vout = USB_Data_Swap_Byte_Order_16(vout_adc_value);

    return success;
}

// **************************
// USB_DATA_ID_CAPACITY

bool_t USB_Data_Capacity_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_CAPACITY_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_CAPACITY_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.capacity, (uint8_t*)&(command_data_ptr->capacity), sizeof(command_data_ptr->capacity));
        success = true;
    }

    return success;
}

bool_t USB_Data_Capacity_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_CAPACITY_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_CAPACITY_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->capacity = PIC16F_Flash_Data.capacity;

    return success;
}

// **************************
// USB_DATA_ID_NONSTANDARD_INDUCTOR

bool_t USB_Data_Nonstandard_Inductor_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_NONSTANDARD_INDUCTOR_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_NONSTANDARD_INDUCTOR_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.nonstandard_inductor, (uint8_t*)&(command_data_ptr->nonstandard_inductor), sizeof((command_data_ptr->nonstandard_inductor)));
        success = true;
    }

    return success;
}

bool_t USB_Data_Nonstandard_Inductor_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_NONSTANDARD_INDUCTOR_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_NONSTANDARD_INDUCTOR_RESPONSE_TYPE*) data_ptr;
    uint8_t* byte_ptr = response_data_ptr->nonstandard_inductor;

    for(uint8_t byte_num = 0; byte_num < sizeof(response_data_ptr->nonstandard_inductor); byte_num++)
    {
        byte_ptr[byte_num] = PIC16F_Flash_Data.nonstandard_inductor[byte_num];
    }

    return success;
}

// **************************
// USB_DATA_ID_EN_CONFIGURATION
bool_t USB_Data_En_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_EN_CONFIGURATION_COMMAND_TYPE* command_data_ptr = (USB_DATA_ID_EN_CONFIGURATION_COMMAND_TYPE*) data_ptr;

    // Parse the command
    if(subcommand == USB_DATA_SUBCOMMAND_WRITE)
    {
        PIC16F_Flash_Write((uint16_t) &PIC16F_Flash_Data.en_setting, (uint8_t*)&(command_data_ptr->en_setting), sizeof(command_data_ptr->en_setting));
        success = true;
    }

    return success;
}

bool_t USB_Data_En_Configuration_Response(uint8_t* data_ptr)
{
    bool_t success = false;
    USB_DATA_ID_EN_CONFIGURATION_RESPONSE_TYPE* response_data_ptr = (USB_DATA_ID_EN_CONFIGURATION_RESPONSE_TYPE*) data_ptr;

    response_data_ptr->en_setting = PIC16F_Flash_Data.en_setting;

    return success;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
