/*
 Linear Technology DC2343A Demonstration Board.
 DC2343A Application File for data packed and passed through the USB.

 The pieces of data that are passed through USB are identified by a data id defined by USB_DATA_ID_TYPE.

 The behavior of each data id is defined by 2 functions:
 - bool_t USB_Data_<data_id>_Command(uint8_t board_num, uint8_t* data_ptr) - process a command for this data.
 - bool_t USB_Data_<data_id>_Response(uint8_t board_num, uint8_t* data_ptr) - send a response for this data id.

 Most data ids accept one of more commands, and send a response after a command is received.

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 791 $
 $Date: 2014-09-30 10:54:07 -0400 (Tue, 30 Sep 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

*/

#ifndef __USB_DATA__
#define __USB_DATA__

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "Typedefs.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

typedef enum
{
    USB_DATA_ID_FW_REVISION,
    USB_DATA_ID_SERIAL_NUMBER,
    USB_DATA_ID_ERROR_DATA,
    USB_DATA_ID_VOLTAGE_SELECTION,
    USB_DATA_ID_PRESCALER_SELECTION,
    USB_DATA_ID_IPEAK_CONFIGURATION,
    USB_DATA_ID_VOUT_CONFIGURATION,
    USB_DATA_ID_ALARM_THRESHOLD,
    USB_DATA_ID_ACCUMULATOR,
    USB_DATA_ID_ALARMS,
    USB_DATA_ID_CLEAR_INT,
    USB_DATA_ID_COUNTER_TEST,
    USB_DATA_ID_IRQ,
    USB_DATA_ID_PGOOD,
    USB_DATA_ID_ADC,
    USB_DATA_ID_CAPACITY,
    USB_DATA_ID_NONSTANDARD_INDUCTOR,
    USB_DATA_ID_EN_CONFIGURATION,
    USB_DATA_ID_NUM_COMMANDS,
    USB_DATA_ID_NONE = 0xFF
} USB_DATA_ID_TYPE;

typedef enum
{
    USB_DATA_SUBCOMMAND_READ,
    USB_DATA_SUBCOMMAND_WRITE,
    USB_DATA_SUBCOMMAND_RESET,
    USB_DATA_SUBCOMMAND_REFRESH,
    USB_DATA_NUM_SUBCOMMANDS,
    USB_DATA_SUBCOMMAND_NONE = 0xFF
} USB_DATA_SUBCOMMAND_TYPE;

typedef struct
{
    bool_t (*command_function)(USB_DATA_SUBCOMMAND_TYPE, uint8_t*);
    bool_t (*response_function)(uint8_t*);
    uint8_t response_size;
} USB_DATA_TABLE_TYPE;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
extern USB_DATA_TABLE_TYPE const usb_data_table[USB_DATA_ID_NUM_COMMANDS];

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
uint16_t USB_Data_Swap_Byte_Order_16(uint16_t data);
uint32_t USB_Data_Swap_Byte_Order_32(uint32_t data);

#define USB_Data_Fw_Revision_Command NULL           // Read Only
bool_t USB_Data_Fw_Revision_Response(uint8_t* data_ptr);

bool_t USB_Data_Serial_Number_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Serial_Number_Response(uint8_t* data_ptr);

#define USB_Data_Error_Data_Command NULL           // Read Only
bool_t USB_Data_Error_Data_Response(uint8_t* data_ptr);

bool_t USB_Data_Voltage_Selection_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Voltage_Selection_Response(uint8_t* data_ptr);

bool_t USB_Data_Prescaler_Selection_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Prescaler_Selection_Response(uint8_t* data_ptr);

bool_t USB_Data_Ipeak_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Ipeak_Configuration_Response(uint8_t* data_ptr);

bool_t USB_Data_Vout_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Vout_Configuration_Response(uint8_t* data_ptr);

bool_t USB_Data_Alarm_Threshold_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Alarm_Threshold_Response(uint8_t* data_ptr);

bool_t USB_Data_Accumulator_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Accumulator_Response(uint8_t* data_ptr);

#define USB_Data_Alarms_Command NULL                // Read Only
bool_t USB_Data_Alarms_Response(uint8_t* data_ptr);

bool_t USB_Data_Clear_Int_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Clear_Int_Response(uint8_t* data_ptr);

bool_t USB_Data_Counter_Test_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Counter_Test_Response(uint8_t* data_ptr);

#define USB_Data_Irq_Command NULL                   // Read Only
bool_t USB_Data_Irq_Response(uint8_t* data_ptr);

#define USB_Data_Pgood_Command NULL                 // Read Only
bool_t USB_Data_Pgood_Response(uint8_t* data_ptr);

bool_t USB_Data_Adc_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Adc_Response(uint8_t* data_ptr);

bool_t USB_Data_Capacity_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Capacity_Response(uint8_t* data_ptr);

bool_t USB_Data_Nonstandard_Inductor_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_Nonstandard_Inductor_Response(uint8_t* data_ptr);

bool_t USB_Data_En_Configuration_Command(USB_DATA_SUBCOMMAND_TYPE subcommand, uint8_t* data_ptr);
bool_t USB_Data_En_Configuration_Response(uint8_t* data_ptr);

#endif
