/*
 Linear Technology DC2343A Demonstration Board.
 This code contains the USB descriptors used by the DC2343A to identify itself to the connected PC.

 This code is used with a modified version of the Microchip USB driver as the original driver did not support strings in RAM.

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 542 $
 $Date: 2014-07-31 11:57:59 -0400 (Thu, 31 Jul 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

*/

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "Typedefs.h"
#include "PIC16F_Flash.h"
#include "DC2343A.h"
#include "USB_Driver/USB_Descriptor.h"
#include "USB_Driver/usb.h"

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#define USB_DESCRIPTOR_LANGUAGE_STRING         {0x09, 0x04} // USB-IF LANGID defintion specifies 0x0409 as English (United States)
#define USB_DESCRIPTOR_LANGUAGE_STRING_SIZE    2            // sizeof(USB_DESCRIPTOR_LANGUAGE_STRING)

#define USB_DESCRIPTOR_STRING_HEADER_SIZE      2            // 2 bytes for string type identifier and string length

// Structure to track the characters in a string descriptor as they are sent to the USB driver.
typedef struct
{
    uint8_t index;                                          // Index into string.
    uint8_t num;                                            // string number
} USB_DESCRIPTOR_STRING_INDEX_TYPE;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

/* Device Descriptor */
const uint8_t Usb_Descriptor_Device[] =
{
    0x12,                                   // Size of this descriptor in bytes
    USB_DESCRIPTOR_DEVICE,                 // DEVICE descriptor type
    LOWER_BYTE(USB_REVISION),               // USB version in BCD
    UPPER_BYTE(USB_REVISION),               //
    0x00,                                   // Class Code
    0x00,                                   // Subclass code
    0x00,                                   // Protocol code
    USB_EP0_BUFF_SIZE,                      // Max packet size for EP0, see usb_config.h
    LOWER_BYTE(USB_VID_LINEAR),             // Vendor ID
    UPPER_BYTE(USB_VID_LINEAR),             //
    LOWER_BYTE(USB_PID_DC2343),             // Product ID
    UPPER_BYTE(USB_PID_DC2343),             //
    LOWER_BYTE(USB_DEVICE_VERSION),         // Device Version
    UPPER_BYTE(USB_DEVICE_VERSION),         //
    USB_DESCRIPTOR_STRING_MANUFACTURER,    // Index of string description of manufacturer. therefore we point to string_1 array (see below)
    USB_DESCRIPTOR_STRING_PRODUCT,         // Index of string descriptor of the product
    USB_DESCRIPTOR_STRING_SERIAL_NUMBER,   // Index of string descriptor of serial number
    0x01                                    // Number of possible configurations
};

/* Configuration 1 Descriptor */
const uint8_t Usb_Descriptor_Config[] =
{
    /* Configuration Descriptor */
    0x09,//sizeof(USB_CFG_DSC),    // Size of this descriptor in bytes
    USB_DESCRIPTOR_CONFIGURATION,                // CONFIGURATION descriptor type
    32,0,                   // Total length of data for this cfg
    1,                      // Number of interfaces in this cfg
    1,                      // Index value of this configuration
    0,                      // Configuration string index
    _DEFAULT | _SELF,               // Attributes, see usb_device.h
    50,                     // Max power consumption (2X mA)

    /* Interface Descriptor */
    9,//sizeof(USB_INTF_DSC),   // Size of this descriptor in bytes
    USB_DESCRIPTOR_INTERFACE,               // INTERFACE descriptor type
    0,                      // Interface Number
    0,                      // Alternate Setting Number
    2,                      // Number of endpoints in this intf
    0xFF,                    // Class code
    0,                      // Subclass code
    0,                      // Protocol code
    0,                      // Interface string index

    /* Endpoint Descriptor */
    0x07,/*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT,    //Endpoint Descriptor
    _EP01_IN,            //EndpointAddress
    _BULK,                       //Attributes
    0x40,0x00,                  //size
    0x01,                       //Interval

    /* Endpoint Descriptor */
    0x07,/*sizeof(USB_EP_DSC)*/
    USB_DESCRIPTOR_ENDPOINT,    //Endpoint Descriptor
    _EP01_OUT,            //EndpointAddress
    _BULK,                       //Attributes
    0x40,0x00,                  //size
    0x01,                       //Interval
};

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
const char usb_descriptor_string_language_data[USB_DESCRIPTOR_LANGUAGE_STRING_SIZE]            = USB_DESCRIPTOR_LANGUAGE_STRING;
const char usb_descriptor_string_manufacturer_data[DC2343_COMPANY_STRING_SIZE]                  = DC2343_COMPANY_STRING;
const char usb_descriptor_string_product_data[DC2343_IDSTRING_SIZE]                             = DC2343_IDSTRING;

// Length of the strings, all unicode except for the language string, with 2 bytes added for USB string descriptor header
const char usb_descriptor_string_lengths[USB_DESCRIPTOR_NUM_STRINGS] = {                       \
        USB_DESCRIPTOR_LANGUAGE_STRING_SIZE + USB_DESCRIPTOR_STRING_HEADER_SIZE,              \
        DC2343_COMPANY_STRING_SIZE * UNICODE_PER_ASCII + USB_DESCRIPTOR_STRING_HEADER_SIZE,    \
        DC2343_IDSTRING_SIZE * UNICODE_PER_ASCII + USB_DESCRIPTOR_STRING_HEADER_SIZE,          \
        DC2343_SERIAL_NUM_SIZE * UNICODE_PER_ASCII + USB_DESCRIPTOR_STRING_HEADER_SIZE};

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Setup structure to track the characters in a string descriptor as they are sent to the USB driver.
// Return the length of the string, as the USB driver uses this as the limit for a loop.
uint16_t Usb_Descriptor_String_Setup(uint8_t  string_num, uint16_t* usb_mem_ptr)
{
    int8_t string_length;
    USB_DESCRIPTOR_STRING_INDEX_TYPE* string_index_ptr;

    if(string_num < USB_DESCRIPTOR_NUM_STRINGS)
    {
        // use memory already reserved by USB driver to keep track of string descriptor being sent to the USB
        string_index_ptr = (USB_DESCRIPTOR_STRING_INDEX_TYPE*)usb_mem_ptr;

        string_index_ptr->num = string_num;
        string_index_ptr->index = 0;

        string_length = usb_descriptor_string_lengths[string_index_ptr->num];
    }
    else
    {
        string_length = 0;
    }

    return (int16_t) string_length;
}

// Gets one character from a string descriptor.
// Return the indexed character from the string.
char Usb_Descriptor_String_Get_Char(uint16_t* usb_mem_ptr)
{
    char ret_char;
    USB_DESCRIPTOR_STRING_INDEX_TYPE* string_index_ptr;
    int8_t adjusted_index;

    // use memory already reserved by USB driver to keep track of string descriptor being sent to the USB
    string_index_ptr = (USB_DESCRIPTOR_STRING_INDEX_TYPE*)usb_mem_ptr;

    if(string_index_ptr->index == 0)
    {
        ret_char = usb_descriptor_string_lengths[string_index_ptr->num];
    }
    else if(string_index_ptr->index == 1)
    {
        ret_char = USB_DESCRIPTOR_STRING;
    }
    else
    {
        adjusted_index = string_index_ptr->index - USB_DESCRIPTOR_STRING_HEADER_SIZE;

        // Only string 0 is not Unicode
        if(string_index_ptr->num == USB_DESCRIPTOR_STRING_LANGUAGE)
        {
            ret_char = usb_descriptor_string_language_data[adjusted_index];
        }
        else
        {
            // All of the Unicode strings use 0 for their odd bytes
            if(string_index_ptr->index & 1)
            {
                ret_char = 0;
            }
            else
            {
                adjusted_index = adjusted_index / UNICODE_PER_ASCII;
                switch (string_index_ptr->num)
                {
                    case USB_DESCRIPTOR_STRING_MANUFACTURER:
                        ret_char = usb_descriptor_string_manufacturer_data[adjusted_index];
                        break;

                    case USB_DESCRIPTOR_STRING_PRODUCT:
                        ret_char = usb_descriptor_string_product_data[adjusted_index];
                        break;

                    case USB_DESCRIPTOR_STRING_SERIAL_NUMBER:
                        ret_char = PIC16F_Flash_Data.serial_number[adjusted_index];
                        break;

                    default:
                        ret_char = 0;
                        break;
                }
            }
        }
    }

    return ret_char;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
