/*
 Linear Technology DC2343A Demonstration Board.
 DC2343A Application File for receiving commands from and sending responses to the DC2343A GUI.

 This code module accepts and receives bytes from the USB Driver.  If the bytes of data fit the
 packet format defined for the DC2343A, the appropriate function is called from USB_Data.c to
 process the request from USB and send a response.

 http://www.linear.com/product/LTC3335#demoboards

 REVISION HISTORY
 $Revision: 853 $
 $Date: 2014-11-11 16:14:45 -0500 (Tue, 11 Nov 2014) $

 Copyright (c) 2014, Linear Technology Corp.(LTC)
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 The views and conclusions contained in the software and documentation are those
 of the authors and should not be interpreted as representing official policies,
 either expressed or implied, of Linear Technology Corp.

*/

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Includes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include "Typedefs.h"
#include "DC2343A.h"
#include "USB_Driver\usb_device.h"
#include "USB_Parser.h"
#include "USB_Data.h"
#include "PIC16F_Timer.h"
#include "Error.h"
#include <string.h>

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Definitions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

#define USB_MAX_PACKET_SIZE (64)
#define USB_PARSER_EP_NUM  1
#define USB_PARSER_NUM_DIRECTIONS 2          // Two directions for EP1: OUT_FROM_HOST and IN_TO_HOST
#if (USB_PING_PONG_MODE == USB_PING_PONG__FULL_PING_PONG) || (USB_PING_PONG_MODE == USB_PING_PONG__ALL_BUT_EP0)
    #define USB_PARSER_NUM_BUFFERS 2
#else
    #error This file expects ping-pong buffering to be used for EP1.
#endif

typedef enum {
    USB_PARSER_STATE_WAITING,
    USB_PARSER_STATE_COMMAND_RECEIVING_HEADER,
    USB_PARSER_STATE_COMMAND_RECEIVING_DATA,
    USB_PARSER_STATE_COMMAND_PROCESSING,
    USB_PARSER_STATE_RESPONSE_SENDING,
    USB_PARSER_NUM_STATES,
} USB_PARSER_STATE_TYPE;

#define USB_PARSER_START_OF_RESPONSE        0xA123
#define USB_PARSER_START_OF_COMMAND         0xB456

typedef union
{
    struct
    {
        uint16_t start_of_command;
        uint8_t data_id;
        uint8_t subcommand;
        uint16_t num_bytes;
    } command;
    struct
    {
        uint16_t start_of_response;
        uint8_t data_id;
        uint32_t timestamp;
        uint16_t num_bytes;
    } response;
} USB_PARSER_HEADER_TYPE;

#define USB_PARSER_SAFETY_TIMEOUT     (2 * PIC16F_TIMER_TICKS_PER_SEC)

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Data
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
USB_HANDLE usb_parser_handle[USB_PARSER_NUM_DIRECTIONS][USB_PARSER_NUM_BUFFERS];    // Handles to USB Driver EP Buffer Descriptor Tables.
uint8_t usb_parser_active_buffer[USB_PARSER_NUM_DIRECTIONS];                        // Tracks which buffer is active for each EP direction.

// User application buffer for receiving packets from and sending packets to the host.
// See Table 3-3 in datasheet for explanation of why these can not be in a multi-dimensional array.
USB_VOLATILE uint8_t usb_parser_receive_buffer_0[USB_MAX_PACKET_SIZE] @ 0x0A0;
USB_VOLATILE uint8_t usb_parser_receive_buffer_1[USB_MAX_PACKET_SIZE] @ 0x120;
USB_VOLATILE uint8_t usb_parser_transmit_buffer_0[USB_MAX_PACKET_SIZE] @ 0x1A0;
USB_VOLATILE uint8_t usb_parser_transmit_buffer_1[USB_MAX_PACKET_SIZE] @ 0x220;
USB_VOLATILE uint8_t* const usb_parser_buffer_addresses[USB_PARSER_NUM_DIRECTIONS][USB_PARSER_NUM_BUFFERS] = {
        {usb_parser_receive_buffer_0, usb_parser_receive_buffer_1},
        {usb_parser_transmit_buffer_0, usb_parser_transmit_buffer_1}};

// Pointer and byte count in drivers in USB buffer as we fill and empty them.
uint8_t* usb_parser_receive_buffer_ptr;
uint8_t usb_parser_receive_buffer_num_bytes;
uint8_t* usb_parser_transmit_buffer_ptr;
uint8_t usb_parser_transmit_buffer_num_bytes;

USB_PARSER_STATE_TYPE usb_parser_state;
USB_PARSER_HEADER_TYPE usb_parser_header;

uint32_t usb_parser_safety_timer;
USB_PARSER_STATE_TYPE usb_parser_state_last;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Prototypes
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
void usb_parser_state_machine_reset(void);
void usb_parser_state_machine(void);

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Global Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Initializes the USB Parser code module.
void USB_Parser_Init(void)
{

    // Initialize interface to USB Driver.
    usb_parser_active_buffer[OUT_FROM_HOST] = 0;
    usb_parser_handle[OUT_FROM_HOST][0] = NULL;
    usb_parser_handle[OUT_FROM_HOST][1] = NULL;

    usb_parser_active_buffer[IN_TO_HOST] = 0;
    usb_parser_handle[IN_TO_HOST][0] = NULL;
    usb_parser_handle[IN_TO_HOST][1] = NULL;

    usb_parser_receive_buffer_ptr = NULL;
    usb_parser_transmit_buffer_ptr = NULL;

    // Init the parser state machine.
    usb_parser_state_machine_reset();

    usb_parser_safety_timer = PIC16F_Timer_Get32();

    return;
}

// Callback function for the USB Driver.
bool_t USB_Parser_Callback_Event_Handler(USB_EVENT event, void *pdata, uint16_t size)
{
    switch (event)
    {
        case EVENT_CONFIGURED:
            // Enable the parser once in the configured state.
            //Enable the application data endpoint
            USBEnableEndpoint(USB_PARSER_EP_NUM, USB_OUT_ENABLED | USB_IN_ENABLED | USB_HANDSHAKE_ENABLED);

            //Prepare the OUT endpoint for the next packet that the host might try to send
            for (uint8_t buffer_num = 0; buffer_num < USB_PARSER_NUM_DIRECTIONS; buffer_num++)
            {
                usb_parser_handle[OUT_FROM_HOST][buffer_num] = USBRxOnePacket(USB_PARSER_EP_NUM, usb_parser_buffer_addresses[OUT_FROM_HOST][buffer_num], USB_MAX_PACKET_SIZE);
            }
            break;

        case EVENT_SUSPEND:
        case EVENT_RESUME:
            // todo - A USB compliant device should honor the suspend and resume
            break;

        case EVENT_BUS_ERROR:
            // Record the error.
            Error_Data_Set(ERROR_CODE_USB_ERROR, NULL, 0);

            // Reset the parser
            usb_parser_state_machine_reset();

            break;

        default:
            break;
    }

    return true;
}

// Checks USB Driver for bytes, and copies to receive queue for processing by the USB Parser.
void USB_Parser_Task(void)
{
    USB_HANDLE rx_handle = usb_parser_handle[OUT_FROM_HOST][usb_parser_active_buffer[OUT_FROM_HOST]];
    USB_HANDLE tx_handle = usb_parser_handle[IN_TO_HOST][usb_parser_active_buffer[IN_TO_HOST]];

    // If done processing the last buffer, get new bytes from USB if any bytes were received.
    if((usb_parser_receive_buffer_ptr == NULL) && !USBHandleBusy(rx_handle))
    {
        usb_parser_receive_buffer_ptr = usb_parser_buffer_addresses[OUT_FROM_HOST][usb_parser_active_buffer[OUT_FROM_HOST]];
        usb_parser_receive_buffer_num_bytes = USBHandleGetLength(rx_handle);
    }

    // If done sending the last response, get a new buffer from USB so we can transmit more bytes.
    if((usb_parser_transmit_buffer_ptr == NULL) && !USBHandleBusy(tx_handle))
    {
        usb_parser_transmit_buffer_ptr = usb_parser_buffer_addresses[IN_TO_HOST][usb_parser_active_buffer[IN_TO_HOST]];
        usb_parser_transmit_buffer_num_bytes = 0;   // The transmit buffer starts empty.
    }

    // Process USB commands and Build Responses.
    usb_parser_state_machine();

    // If all bytes are processed in this buffer, release it for the USB to get new bytes.
    if((usb_parser_receive_buffer_ptr != NULL) && (usb_parser_receive_buffer_num_bytes == 0))
    {
        USB_VOLATILE uint8_t* buffer = usb_parser_buffer_addresses[OUT_FROM_HOST][usb_parser_active_buffer[OUT_FROM_HOST]];
        rx_handle = USBRxOnePacket(USB_PARSER_EP_NUM, buffer, USB_MAX_PACKET_SIZE);
        usb_parser_active_buffer[OUT_FROM_HOST] = 1 - usb_parser_active_buffer[OUT_FROM_HOST];

        usb_parser_receive_buffer_ptr = NULL;       // USB now owns this buffer.
    }

    // If the transmit buffer isn't empty, release it for the USB to send the bytes.
    if((usb_parser_transmit_buffer_ptr != NULL) && (usb_parser_transmit_buffer_num_bytes != 0))
    {
        USB_VOLATILE uint8_t* buffer = usb_parser_buffer_addresses[IN_TO_HOST][usb_parser_active_buffer[IN_TO_HOST]];
        tx_handle = USBTxOnePacket(USB_PARSER_EP_NUM, buffer, usb_parser_transmit_buffer_num_bytes);
        if(tx_handle != 0)
        {
            usb_parser_handle[IN_TO_HOST][usb_parser_active_buffer[IN_TO_HOST]] = tx_handle;
        }

        usb_parser_active_buffer[IN_TO_HOST] = 1 - usb_parser_active_buffer[IN_TO_HOST];

        usb_parser_transmit_buffer_ptr = NULL;       // USB now owns this buffer.
    }

    return;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Local Functions
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

void usb_parser_state_machine_reset(void)
{
    usb_parser_receive_buffer_num_bytes = 0;
    usb_parser_transmit_buffer_num_bytes = 0;
    usb_parser_state = USB_PARSER_STATE_WAITING;
    usb_parser_state_last = USB_PARSER_STATE_WAITING;
}

void usb_parser_state_machine(void)
{
    // Receive the header, process commands that are received, and wait for the responses to be sent.
    switch (usb_parser_state)
    {
        case USB_PARSER_STATE_WAITING:
check_start:
            if(usb_parser_receive_buffer_num_bytes >= sizeof(usb_parser_header.command))
            {
                usb_parser_state = USB_PARSER_STATE_COMMAND_RECEIVING_HEADER;
                goto receive_header;
            }
            break;

        case USB_PARSER_STATE_COMMAND_RECEIVING_HEADER:
receive_header:
            // Check for the first start of the command byte.
            memcpy((uint8_t*)&usb_parser_header, usb_parser_receive_buffer_ptr, sizeof(usb_parser_header.command));
            usb_parser_header.command.start_of_command = USB_Data_Swap_Byte_Order_16(usb_parser_header.command.start_of_command);
            usb_parser_header.command.data_id = usb_parser_header.command.data_id;
            usb_parser_header.command.subcommand = usb_parser_header.command.subcommand;
            usb_parser_header.command.num_bytes = USB_Data_Swap_Byte_Order_16(usb_parser_header.command.num_bytes);

            if((usb_parser_header.command.start_of_command == USB_PARSER_START_OF_COMMAND) &&
               (usb_parser_header.command.data_id < USB_DATA_ID_NUM_COMMANDS) &&
               (usb_parser_header.command.subcommand < USB_DATA_NUM_SUBCOMMANDS))
            {
                // Advance buffer pointer past the header.
                usb_parser_receive_buffer_ptr += sizeof(usb_parser_header.command);
                usb_parser_receive_buffer_num_bytes -= sizeof(usb_parser_header.command);

                // Receive the data bytes.
                usb_parser_state = USB_PARSER_STATE_COMMAND_RECEIVING_DATA;
                goto receive_data;
            }
            else
            {
                // Throw away bytes.  Header not of proper format.
                usb_parser_state_machine_reset();
                goto check_start;
            }

        case USB_PARSER_STATE_COMMAND_RECEIVING_DATA:
receive_data:
            // Wait until the command data has been received.
            if(usb_parser_receive_buffer_num_bytes >= usb_parser_header.command.num_bytes)
            {
                usb_parser_state = USB_PARSER_STATE_COMMAND_PROCESSING;
                goto process_command;
            }
            break;

        case USB_PARSER_STATE_COMMAND_PROCESSING:
process_command :
        {
            bool_t success;

            if(usb_parser_header.command.subcommand == USB_DATA_SUBCOMMAND_READ)
            {
                // All Data IDs support the read command.
                success = true;
            }
            else if (usb_data_table[usb_parser_header.command.data_id].command_function != NULL)
            {
                // If a command function exists for this Data ID, pass the data to it.
                success = usb_data_table[usb_parser_header.command.data_id].command_function(usb_parser_header.command.subcommand, usb_parser_receive_buffer_ptr);
            }
            else
            {
                // This Data ID doesn't support anything but the read command.
                success = false;
            }

            if(success == true)
            {
                // Advance buffer pointer past the processed data.
                usb_parser_receive_buffer_ptr += usb_parser_header.command.num_bytes;
                usb_parser_receive_buffer_num_bytes -= usb_parser_header.command.num_bytes;

                usb_parser_state = USB_PARSER_STATE_RESPONSE_SENDING;
                goto send_response;
            }
            else
            {
                // Throw away bytes.  Command not of proper format.
                usb_parser_state_machine_reset();
                goto check_start;
            }
        }
            // fallthrough

        case USB_PARSER_STATE_RESPONSE_SENDING:
send_response:
            {
                uint16_t response_size = usb_data_table[usb_parser_header.response.data_id].response_size;
                uint32_t timestamp = PIC16F_Timer_Get32();

                // Wait until there is enough room in the buffer to send this response.
                if((usb_parser_transmit_buffer_ptr != NULL) &&
                  (sizeof(usb_parser_header.response) + response_size) < (USB_MAX_PACKET_SIZE - usb_parser_transmit_buffer_num_bytes))
                {
                    // Build the header.
                    usb_parser_header.response.start_of_response = USB_Data_Swap_Byte_Order_16(USB_PARSER_START_OF_RESPONSE);
                    usb_parser_header.response.data_id = usb_parser_header.command.data_id;
                    usb_parser_header.response.timestamp = USB_Data_Swap_Byte_Order_32(timestamp);
                    usb_parser_header.response.num_bytes = USB_Data_Swap_Byte_Order_16(response_size);

                    // Place header into buffer.
                    memcpy(usb_parser_transmit_buffer_ptr, (uint8_t*)&usb_parser_header, sizeof(usb_parser_header.response));
                    usb_parser_transmit_buffer_ptr += sizeof(usb_parser_header.response);
                    usb_parser_transmit_buffer_num_bytes += sizeof(usb_parser_header.response);

                    // Add the response data.
                    usb_data_table[usb_parser_header.response.data_id].response_function(usb_parser_transmit_buffer_ptr);
                    usb_parser_transmit_buffer_ptr += response_size;
                    usb_parser_transmit_buffer_num_bytes += response_size;

                    // Return to waiting for new commands
                    usb_parser_state = USB_PARSER_STATE_WAITING;
                    goto check_start;
                }
            }
            break;
    }

    // Check to ensure we don't lock up in one state forever.
    {
        uint32_t new_time = PIC16F_Timer_Get32();
        if((usb_parser_state != usb_parser_state_last) || (usb_parser_state == USB_PARSER_STATE_WAITING))
        {
            usb_parser_state_last = usb_parser_state;
            usb_parser_safety_timer = new_time;
        }
        else if((new_time - usb_parser_safety_timer) >= USB_PARSER_SAFETY_TIMEOUT)
        {
            usb_parser_state_machine_reset();
            usb_parser_safety_timer = new_time;
        }
    }

    return;

}
