﻿Option Strict On
Option Explicit On

Imports Microsoft.VisualBasic
Imports System.Windows.Forms
Imports System.Environment

' Event Log Requirements:
'    - There is a listbox and a file for storing log records.
'    - Program still must run, even if file can not be written.
'    - Put stuff on the screen that's nice for customers to read only.
'    - Types of Records:
'        1. Info that's useful to customers.
'        2. Info about how to configure the GUI upon startup.
'        3. Info useful to LTC.
'    - Records can have different behaviors:
'        1. While all records are stored in the file (if it can be written), not all are stored in the listbox.
'        2. Some go to the Listbox, and reloaded into the Listbox upon restart.
'        3. Some go to the Listbox, but aren't reloaded into the Listbox upon restart.
'        4. Some don't go to the Listbox, and aren't reloaded into the Listbox upon restart.

Public Class Event_Log

    Private Const LISTBOX_LIMIT As Integer = 100
    Private filename As String
    Private WithEvents listbox As ListBox
    Private WithEvents clear_button As Button
    Private WithEvents clear_checkbox As CheckBox

    Public Enum Event_Code
        None
        Test
        Event_Log_File
        Clear_Log_On_Exit
        Maintain_Log_On_Exit
        Message
        Exception
        WinUSB_StartDeviceError
        USB_No_Reply
        LTC3335_Comm_Error
        Num_Event_Codes
    End Enum
    Private Event_Code_Count(Event_Code.Num_Event_Codes - 1) As Integer

    Private Class Record
        Public DateTime As DateTime
        Public Event_Code As Event_Code
        Public ListBox_Text As String
        Public FileOnly_Text As String
        Public Reload_Flag As Boolean

        Private Const DATA_DELIMITER As String = "∟"
        Private Const NUM_DATA_FIELDS As Integer = 5

        Public Sub New(Optional ByVal event_code As Event_Code = Event_Code.None, Optional ByVal listbox_text As String = "", _
                       Optional ByVal fileonly_text As String = "", Optional ByVal reload_flag As Boolean = True)
            Me.DateTime = System.DateTime.Now
            Me.Event_Code = event_code
            Me.ListBox_Text = listbox_text
            Me.FileOnly_Text = fileonly_text
            Me.Reload_Flag = reload_flag
        End Sub

        Public Function Get_ListBox_String() As String
            Dim textline As String
            textline = CStr(Me.DateTime) + ": "
            textline += Me.ListBox_Text
            Return textline
        End Function

        Public Function Get_File_String() As String
            Dim textline As String
            textline = CStr(System.DateTime.Now) + DATA_DELIMITER
            textline += [Enum].GetName(GetType(Event_Code), Me.Event_Code) + DATA_DELIMITER
            textline += Me.ListBox_Text + DATA_DELIMITER
            textline += Me.FileOnly_Text + DATA_DELIMITER
            textline += Me.Reload_Flag.ToString + DATA_DELIMITER
            Return textline
        End Function

        Public Sub Set_File_String(ByVal file_string As String)
            Dim record_array() As String = file_string.Split(CChar(DATA_DELIMITER))
            If record_array.Length >= NUM_DATA_FIELDS Then
                Me.DateTime = Date.Parse(record_array(0))
                Me.Event_Code = DirectCast([Enum].Parse(GetType(Event_Code), record_array(1)), Event_Code)
                Me.ListBox_Text = record_array(2)
                Me.FileOnly_Text = record_array(3)
                Me.Reload_Flag = CBool(record_array(4))
            End If
        End Sub

    End Class

    ' Create an event log object, determine if there's a writable event log file, and load contents of file to list box that are meant to be reloaded.
    Public Sub New(ByVal filename As String, ByVal listbox As ListBox, _
                   ByVal clear_button As Button, ByVal clear_checkbox As CheckBox)

        ' Point to the controls in the form
        Me.listbox = listbox
        Me.clear_button = clear_button
        Me.clear_checkbox = clear_checkbox

        ' Clear the count for each event code
        For event_num As Integer = 0 To Event_Code.Num_Event_Codes - 1
            Event_Code_Count(event_num) = 0
        Next event_num

        ' Try reading from the file and closing to see if it is allowed
        Dim reader As System.IO.StreamReader

        Try
            ' Try using this file for error logging.
            Me.filename = filename

            If My.Computer.FileSystem.FileExists(filename) Then
                ' Try loading error file
                reader = New System.IO.StreamReader(filename)

                If (reader.EndOfStream) Then
                    reader.Close()
                    ' If error file exists and it's empty, try writing to it to see if it's possible.
                    Me.add_record(New Record(Event_Code.Event_Log_File, "Event File Created at " + Me.filename, "", False))
                Else
                    ' If error file exists, load contents into the box.
                    While (Not reader.EndOfStream)
                        Dim record As New Record()

                        ' Parse the text from the file into a record
                        record.Set_File_String(reader.ReadLine())

                        ' Update the clear checkbox based up information stored in the log.
                        RemoveHandler clear_checkbox.CheckedChanged, AddressOf clear_checkbox_checkedchanged
                        If record.Event_Code = Event_Code.Clear_Log_On_Exit Then
                            clear_checkbox.Checked = True
                        ElseIf record.Event_Code = Event_Code.Maintain_Log_On_Exit Then
                            clear_checkbox.Checked = False
                        End If
                        AddHandler clear_checkbox.CheckedChanged, AddressOf clear_checkbox_checkedchanged

                        ' Only load the records that are marked to be reloaded.
                        If record.Reload_Flag = True Then
                            Me.listbox.Items.Add(record.Get_ListBox_String)
                            If listbox.Items.Count > LISTBOX_LIMIT Then
                                Me.listbox.Items.RemoveAt(0)
                            End If
                        End If
                    End While
                    reader.Close()

                    ' If error file exists, attempt to write to it.  If we can't, then GUI is probably being run with the wrong permissions.
                    reader.Close()
                    Me.add_record(New Record(Event_Code.Event_Log_File, "Event File Opened at " + Me.filename, "", False))

                End If
                Me.listbox.TopIndex = Me.listbox.Items.Count - 1
            Else
                Try
                    ' If error file doesn't exist, try creating it 
                    Dim writer As New System.IO.StreamWriter(filename)
                    writer.WriteLine()
                    writer.Close()
                    Me.add_record(New Record(Event_Code.Event_Log_File, "Event File Created at " + Me.filename, "", False))

                Catch ex2 As Exception
                    ' Chicken and Egg...Can't Log Exceptions to Error File while creating the Error File 
                    'D3335.Handle_Exception(ex2)

                    ' Clear the error file name to flag that it is not valid file.
                    Me.filename = ""
                End Try
            End If
        Catch ex As Exception
            ' Chicken and Egg...Can't Log Exceptions to Error File while creating the Error File 
            'D3335.Handle_Exception(ex)

            ' Clear the error file name to flag that it is not valid file.
            Me.filename = ""
        End Try

    End Sub

    ' Add a record to the event log file (if writable) and list box.
    Public Sub Add(ByVal event_code As Event_Code, Optional ByVal listbox_text As String = "", Optional ByVal fileonly_text As String = "")
        Me.add_record(New Record(event_code, listbox_text, fileonly_text))
    End Sub

    ' Clear the event log file (if writable) and list box.
    Public Sub Clear()
        ' Try writing to the file and closing to see if it is allowed
        listbox.Items.Clear()

        ' Clear Error Log Data to File if it exists and can be written
        If Me.filename <> "" Then
            My.Computer.FileSystem.WriteAllText(Me.filename, "", False)
        End If

        ' Start new log with the message that the last one was cleared
        Me.add_record(New Record(Event_Code.Message, "Event Log Cleared", , False))

        If clear_checkbox.Checked = True Then
            Me.add_record(New Record(Event_Code.Clear_Log_On_Exit, "Event Log Will Be Cleared on Exit", , False))
        End If

    End Sub

    ' Clear the event log file (if writable) if set to be cleared upon exit.
    Public Sub Close()
        If clear_checkbox.Checked = True Then
            ' Clear Error Log Data to File if it exists and can be written
            If Me.filename <> "" Then
                My.Computer.FileSystem.WriteAllText(Me.filename, "", False)
            End If
            Me.add_record(New Record(Event_Code.Clear_Log_On_Exit, "Event Log Will Be Cleared on Exit", , False))
        End If
    End Sub

    ' Add a record to the event log file (if writable) and list box.
    Private Sub add_record(ByVal record As Record)

        ' Count the number of each error code
        Event_Code_Count(record.Event_Code) += 1

        ' Write Error Log Data to File if it exists and can be written
        If Me.filename <> "" Then
            Dim writer As System.IO.StreamWriter
            writer = System.IO.File.AppendText(Me.filename)

            ' Create the record entry.
            writer.WriteLine(record.Get_File_String)

            writer.Flush()
            writer.Close()
        End If

        ' Display in listbox if necessary
        If record.ListBox_Text <> "" Then

            ' Add Error Log Entry to GUI
            Me.listbox.Items.Add(record.Get_ListBox_String)
            If listbox.Items.Count > LISTBOX_LIMIT Then
                Me.listbox.Items.RemoveAt(0)
            End If
            Me.listbox.TopIndex = Me.listbox.Items.Count - 1
        End If

    End Sub

    ' Allows copying and pasting from the listbox window
    Private Sub listbox_keydown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles listbox.KeyDown

        If e.Control AndAlso e.KeyCode = Keys.C Then
            Dim copy_buffer As New System.Text.StringBuilder
            For Each item As Object In listbox.SelectedItems
                copy_buffer.AppendLine(item.ToString)
            Next
            If copy_buffer.Length > 0 Then
                Clipboard.SetText(copy_buffer.ToString)
            End If
        End If
    End Sub

    ' Clears the event log file (if writable) if set to be cleared upon exit.
    Private Sub clear_button_click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles clear_button.Click
        Me.Clear()
    End Sub

    ' Configures event log file (if writable) to be cleared upon exit.
    Private Sub clear_checkbox_checkedchanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles clear_checkbox.CheckedChanged
        If clear_checkbox.Checked = True Then
            Me.add_record(New Record(Event_Code.Clear_Log_On_Exit, "Event Log Will Be Cleared on Exit", , False))
        Else
            Me.add_record(New Record(Event_Code.Maintain_Log_On_Exit, "Event Log Will Be Maintained on Exit", , False))
        End If
    End Sub

End Class
