' <summary>
' GUI Software File for LTC3335 Nanopower Buck-Boost DC/DC with Integrated Coulomb Counter.
' </summary>
' <remarks>
' This file contains functions for the more complex mathematical functions that would not fit in the DC2343A
' firmware due to Program Memory limitations in the PIC16F1459.
'
' The functions provided in this file:
'   1. Calculates the range and resolution represented by the LTC3335 accumulator in Ah.
'   2. Calculates the optimal LTC3335 prescaler for a given IPeak and Battery Capacity.
'   3. Translates the Counter Test results in edges/second into a battery current estimate in Amps.
'   4. Corrects the coulomb count and battery current estimate for the typical error at a given IPeak, VBAT, and VOUT.
' </remarks>

Option Strict On
Option Explicit On
Imports Microsoft.VisualBasic

''' <summary>
''' Namespace containing constants and functions for the LTC3335 Nanopower Buck-Boost DC/DC with Integrated Coulomb Counter.
''' </summary>
''' <remarks>Implements ISample</remarks>
Namespace LTC3335

    Module LTC3335

        ''' <summary>
        ''' LTC3335 Input Quiescent Current in A
        ''' </summary>
        ''' <remarks></remarks>
        Public IQ As Double = 0.00000068

        ''' <summary>
        ''' LTC3335 full scale ON time in seconds
        ''' </summary>
        Private Const tFS As Single = 0.00001174 

        ''' <summary>
        ''' Voltage Selection Options for the LTC3335
        ''' </summary>
        Public Enum VOLTAGE_SELECTION_OPTIONS
            HARDWARE_CONTROL
            SOFTWARE_CONTROL_1_8V
            SOFTWARE_CONTROL_2_5V
            SOFTWARE_CONTROL_2_8V
            SOFTWARE_CONTROL_3_0V
            SOFTWARE_CONTROL_3_3V
            SOFTWARE_CONTROL_3_6V
            SOFTWARE_CONTROL_4_5V
            SOFTWARE_CONTROL_5_0V
            NUM_VOLTAGE_SELECTION_OPTIONS
        End Enum

        ''' <summary>
        ''' IPeak Selection Options for the LTC3335
        ''' </summary>
        Public Enum IPEAK_SELECTION_OPTIONS
            IPEAK_5MA
            IPEAK_10MA
            IPEAK_15MA
            IPEAK_25MA
            IPEAK_50MA
            IPEAK_100MA
            IPEAK_150MA
            IPEAK_250MA
            NUM_IPEAK_SELECTION_OPTIONS
        End Enum

        ''' <summary>
        ''' Possible states of the EN pin on the DC2343A demo board for the LTC3335.
        ''' </summary>
        Public Enum EN_CONFIGURATION_OPTIONS
            EN_CONFIGURATION_ON
            EN_CONFIGURATION_EXT
            EN_CONFIGURATION_OFF
            NUM_EN_CONFIGURATIONS
        End Enum

        ''' <summary>
        ''' Allowable range for the LTC3335 Prescaler.
        ''' </summary>
        Public Const PRESCALER_SELECTION_MIN As Integer = 0
        Public Const PRESCALER_SELECTION_MAX As Integer = 15

#Region "Basic Operation"
        ''' <summary>
        ''' Returns the VOUT value in V for a voltage setting.
        ''' </summary>
        ''' <param name="vout">An option from the VOLTAGE_SELECTION_OPTIONS enum.</param>
        ''' <returns>The output voltage in Volts.</returns>
        Public Function Get_Vout_V(ByVal vout As VOLTAGE_SELECTION_OPTIONS) As Single

            If vout = VOLTAGE_SELECTION_OPTIONS.HARDWARE_CONTROL Then
                Return -1.0     ' We can not tell what the intended voltage is when we're under hardware control
            Else
                Dim voltage_selection_string As String = vout.ToString
                If voltage_selection_string.IndexOf("SOFTWARE_CONTROL_") = 0 Then
                    voltage_selection_string = voltage_selection_string.Substring(Len("SOFTWARE_CONTROL_"), Len("1_8"))
                    Return CSng(voltage_selection_string.Replace("_", "."))
                Else
                    Return -1.0     ' This shouldn't be possible since all optoins should be HARDWARE_CONTROL or SOFTWARE_CONTROL.
                End If
            End If

        End Function

        ''' <summary>
        ''' Returns the Ipeak value in mA for an ipeak setting.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <returns>The IPEAK value in mA.</returns>
        Public Function Get_IPeak_MA(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS) As Integer
            Dim ipeak_string As String = ipeak.ToString

            ipeak_string = ipeak_string.Substring(Len("IPEAK_"), ipeak_string.IndexOf("MA") - Len("IPEAK_"))

            Return CInt(ipeak_string)

        End Function

        ''' <summary>
        ''' Returns the charge range in Ah for an ipeak setting and prescaler.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="prescaler">The value of the prescaler.</param>
        ''' <returns>The range of the Accumulator in Ah.</returns>
        Public Function Get_Charge_Range(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal prescaler As Integer) As Single
            If (ipeak >= LTC3335.IPEAK_SELECTION_OPTIONS.NUM_IPEAK_SELECTION_OPTIONS) Or (prescaler < LTC3335.PRESCALER_SELECTION_MIN) And (prescaler > LTC3335.PRESCALER_SELECTION_MAX) Then
                Return 0
            Else
                Return CSng(Get_IPeak_MA(ipeak) / 1000 / 2) * (tFS / 3600) * (1UL << (42 - prescaler))   ' in Ah
            End If

        End Function

        ''' <summary>
        ''' Returns the charge resolution in Ah for an ipeak setting and prescaler.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="prescaler">The value of the prescaler.</param>
        ''' <returns>The resolution of the Accumulator in Ah.</returns>
        Public Function Get_Charge_Resolution(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal prescaler As Integer) As Single
            Return Get_Charge_Range(ipeak, prescaler) / 255  ' in Ah
        End Function

        ''' <summary>
        ''' Returns the recommended prescaler for an IPEAK setting and a capacity value in Ah.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="capacity">The battery capacity in Ah.</param>
        ''' <param name="use_software_correction">True is the prescaler should be selected assuming software correction will be used.</param>
        ''' <param name="software_correction_factor">The software correction factor that will be used if use_software_correction is True.</param>
        ''' <returns>The recommended prescaler for the LTC3335.</returns>
        Public Function Get_Prescaler(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal capacity As Single, _
                                      Optional ByVal use_software_correction As Boolean = False, Optional ByVal software_correction_factor As Double = 0.0) As Integer
            Dim numerator As Double = capacity * CDbl(3600.0) * CDbl(2.0)
            Dim denominator As Double = Get_IPeak_MA(ipeak) / 1000.0 * tFS
            If use_software_correction = True Then
                numerator *= (1 + software_correction_factor / 100)
            End If
            Dim result As Double = numerator / denominator
            Dim log2 As Double = Math.Log(result, 2)
            Dim prescaler As Double = 42.0 - log2

            If (prescaler > LTC3335.PRESCALER_SELECTION_MAX) Then
                prescaler = LTC3335.PRESCALER_SELECTION_MAX
            ElseIf (prescaler < LTC3335.PRESCALER_SELECTION_MIN) Then
                prescaler = LTC3335.PRESCALER_SELECTION_MIN
            End If

            Return CInt(Math.Floor(prescaler))

        End Function

        ''' <summary>
        ''' Returns the theoretical maximum output current available for a given IPEAK, VOUT, and VBAT.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="vbat">The battery voltage in V.</param>
        ''' <param name="vout">The output voltage in V.</param>
        ''' <returns>The theoretical maximum output current available from the LTC3335.</returns>       
        Public Function Get_Max_VOUT_Current(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal vbat As Single, ByVal vout As Single) As Single
            Return CSng((Get_IPeak_MA(ipeak) / 2) * (vbat / (vbat + vout)))    ' in mA
        End Function

        ''' <summary>
        ''' Converts the LTC3335 Counter Test results in /IRQ edges per second into mA of battery current.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="edges_per_sec">The number of rising edges of /IRQ per second when the LTC3335 Counter Test is active.</param>
        ''' <param name="use_software_correction">True is the prescaler should be selected assuming software correction will be used.</param>
        ''' <param name="software_correction_factor">The software correction factor that will be used if use_software_correction is True.</param>
        ''' <returns>The estimated battery current in mA.</returns>
        Public Function Convert_Counter_Test_Results(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal edges_per_sec As Single, _
                                             Optional ByVal use_software_correction As Boolean = False, _
                                             Optional ByVal software_correction_factor As Single = 0.0) As Single
            Dim ma As Single

            ma = CSng(Get_IPeak_MA(ipeak) * tFS * edges_per_sec)

            If use_software_correction = True Then
                ma = ma / (1 + software_correction_factor / 100) + CSng(IQ * 1000)
                If ma < 0 Then
                    ma = 0
                End If
            End If

            Return ma
        End Function
#End Region

#Region "Coulomb Counter Correction"

        ''' <summary>
        ''' IPEAK Indicies into COULOMB_COUNTER_CORRECTION_TABLE
        ''' \hideinitializer
        ''' </summary>
        Private IPEAK_INDICIES() As String = {"5mA", "10mA", "15mA", "25mA", "50mA", "100mA", "150mA", "250mA"}

        ''' <summary>
        ''' VOUT Indicies into COULOMB_COUNTER_CORRECTION_TABLE
        ''' \hideinitializer
        ''' </summary>
        Private VOUT_INDICIES() As Single = {1.8, 2.5, 2.8, 3, 3.3, 3.6, 4.5, 5}

        ''' <summary>
        ''' VBAT Indicies into COULOMB_COUNTER_CORRECTION_TABLE
        ''' \hideinitializer
        ''' </summary>
        Private VBAT_INDICIES() As Single = {1.8, 1.9, 2, 2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9, 3, 3.1, 3.2, 3.3, 3.4, 3.5, 3.6, 3.7, 3.8, 3.9, 4, 4.1, 4.2, 4.3, 4.4, 4.5, 4.6, 4.7, 4.8, 4.9, 5, 5.1, 5.2, 5.3, 5.4, 5.5}

        ''' <summary>
        ''' TYP Coulomb Counter Error for a given IPEAK, VOUT, and VBAT as shown in Datasheet Figures G37 - G44
        ''' \hideinitializer
        ''' \image html Typ_Error_Curves.png
        ''' </summary>
        Private COULOMB_COUNTER_CORRECTION_TABLE(,,) As Double = {{{-16.6108451086514, -17.2029191565182, -17.7794694339836, -18.3443937150064, -18.9010756536314, -19.4524163660558, -20.0008660126946, -20.5484553802474, -21.0968274637642, -21.6472690487114, -22.200742293038, -22.7579163092417, -23.3191987464348, -23.8847673724108, -24.4546016557099, -25.0285143476854, -25.6061830645696, -26.1871818695403, -26.7710128547866, -27.3571377235747, -27.9450093723145, -28.5341034726256, -29.1239500534032, -29.7141650828841, -30.3044820507134, -30.8947835500097, -31.4851328594319, -32.0758055252451, -32.6673209433866, -33.2604739415318, -33.8563663611608, -34.4564386396242, -35.0625013922091, -35.6767669942054, -36.3018811629718, -36.9409545400016, -37.5975942729895, -38.2759355978973}, _
        {-14.6813863331943, -15.1953726868007, -15.6967559896174, -16.1883534396683, -16.6726928094388, -17.1520267366043, -17.6283470147588, -18.1033988841425, -18.5786953223711, -19.0555313351636, -19.5349982470708, -20.0179979922036, -20.5052574049617, -20.9973425107616, -21.4946728167651, -21.9975356026079, -22.5061002111275, -23.0204323390921, -23.5405083279284, -24.0662294544507, -24.5974362215885, -25.1339226491156, -25.6754505643778, -26.2217638930217, -26.7726029497232, -27.3277187289153, -27.886887195517, -28.4499235756615, -29.0166966474245, -29.5871430315528, -30.1612814821923, -30.7392271776168, -31.3212060109559, -31.907568880924, -32.4988059825482, -33.0955610978966, -33.6986458868069, -34.3090541776151}, _
        {-13.9069710221644, -14.4366240117394, -14.9450243443703, -15.436439601715, -15.9146991405632, -16.3832157360486, -16.8450072248602, -17.3027181484541, -17.7586413962655, -18.2147398489206, -18.6726680214479, -19.1337937064903, -19.599219617517, -20.069805032035, -20.5461874348013, -21.0288041610341, -21.517914039625, -22.0136190363508, -22.5158858970851, -23.0245677910103, -23.5394259538289, -24.060151330976, -24.5863862208305, -25.1177459179273, -25.6538403561689, -26.1942957520369, -26.7387762478044, -27.2870055547472, -27.8387885963561, -28.3940331515482, -28.9527714978792, -29.5151820547545, -30.0816110266418, -30.6525940462823, -31.2288778179025, -31.8114417604266, -32.4015196506873, -33.0006212666386}, _
        {-13.4563430259988, -13.9827699443209, -14.4871924585236, -14.9738913470079, -15.4467171370998, -15.9091109927378, -16.3641256021608, -16.8144460655959, -17.2624107829463, -17.7100323414792, -18.1590184035133, -18.610792594107, -19.066515388746, -19.5271050010313, -19.9932582703668, -20.4654715496473, -20.9440615929463, -21.4291864432039, -21.9208663199144, -22.4190045068144, -22.9234082395704, -23.4338095934669, -23.9498863710937, -24.4712829900346, -24.9976313705543, -25.5285718232868, -26.0637739369231, -26.602957465899, -27.1459132180828, -27.6925239424634, -28.2427852168381, -28.7968263355, -29.3549311969264, -29.9175591914662, -30.4853660890281, -31.0592249267682, -31.6402468967776, -32.2298022337709}, _
        {-12.8564445099588, -13.3650772678127, -13.8555530149231, -14.3312091482698, -14.7950752088274, -15.2498866981558, -15.6980988949897, -16.1419006718289, -16.5832283115282, -17.0237793238879, -17.4650262622432, -17.9082305400545, -18.3544562474978, -18.8045839680541, -19.2593245951001, -19.7192331484976, -20.1847225911841, -20.6560776457627, -21.1334686110919, -21.6169651788757, -22.106550250254, -22.6021337523924, -23.1035664550721, -23.6106537872802, -24.1231696537997, -24.6408702517993, -25.1635078874238, -25.6908447923839, -26.2226669405464, -26.7587978645242, -27.2991124722661, -27.8435508636473, -28.3921321470593, -28.9449682559994, -29.5022777656617, -30.0643997095264, -30.6318073959502, -31.2051222247562}, _
        {-12.2953010108368, -12.7937705849302, -13.2767970664361, -13.7469534145369, -14.2066081982782, -14.657933331839, -15.1029118098029, -15.5433454424285, -15.98086259092, -16.4169259026983, -16.8528400466715, -17.2897594485055, -17.7286960258949, -18.1705269238338, -18.6160022498862, -19.0657528094569, -19.5202978410625, -19.9800527516013, -20.4453368516251, -20.9163810906091, -21.3933357922228, -21.8762783896009, -22.365221160614, -22.860118963139, -23.3608769703303, -23.8673584058902, -24.3793922793395, -24.8967811212885, -25.4193087187078, -25.9467478501985, -26.4788680212636, -27.0154431995779, -27.5562595502596, -28.1011231711404, -28.6498678280366, -29.2023626900192, -29.7585200646856, -30.3183031334293}, _
        {-10.5147189057445, -11.1681764316899, -11.7637248259292, -12.3111351777677, -12.8191912517151, -13.2957387053286, -13.7477343070562, -14.1812951540787, -14.6017478901537, -15.0136779234575, -15.4209786444291, -15.8269006436125, -16.2341009294993, -16.6446921463728, -17.0602917921497, -17.482071436224, -17.9108059373093, -18.3469226612818, -18.7905506990236, -19.2415700842657, -19.6996610114301, -20.1643530534739, -20.6350743797312, -21.1112009737569, -21.5921058511689, -22.0772082774916, -22.5660229859987, -23.0582093955557, -23.5536208284636, -24.0523537283011, -24.5547968777685, -25.0616806165294, -25.5741260590544, -26.0936943124639, -26.6224356943714, -27.1629389507257, -27.7183804736544, -28.2925735193067}, _
        {-10.1166729719126, -10.6260717085217, -11.1214067029236, -11.6040185529448, -12.0752162788691, -12.5362761493162, -12.988440507122, -13.432916595216, -13.8708753825031, -14.3034503897407, -14.7317365154191, -15.1567888616405, -15.579621559998, -16.0012065974553, -16.4224726422258, -16.8443038696518, -17.2675387880838, -17.6929690647596, -18.1213383516842, -18.5533411115079, -18.9896214434072, -19.430771908962, -19.8773323580371, -20.3297887546598, -20.7885720028997, -21.2540567727479, -21.7265603259972, -22.2063413421193, -22.6935987441463, -23.188470524549, -23.6910325711149, -24.2012974928299, -24.7192134457562, -25.2446629589113, -25.7774617601485, -26.3173576020337, -26.8640290877284, -27.417084496866}}, _
        {{-7.55469704790228, -7.86069301824009, -8.16940202839132, -8.48171270114335, -8.79839893668901, -9.12012596667403, -9.44745640824478, -9.78085631809599, -10.1207012465183, -10.467282291446, -10.8208121525046, -11.1814311850588, -11.5492134542595, -11.9241727890924, -12.3062688364246, -12.6954131150534, -13.0914750697528, -13.4942881253221, -13.9036557406332, -14.319357462678, -14.7411549806166, -15.1687981798246, -15.6020311959409, -16.0405984689151, -16.4842507970557, -16.9327513910772, -17.3858819281482, -17.8434486059388, -18.3052881966684, -18.771274101153, -19.2413224028536, -19.7153979219232, -20.1935202692547, -20.6757699005286, -21.1622941702606, -21.6533133858494, -22.149126861624, -22.6501189728918}, _
        {-6.44721988526797, -6.69277919478709, -6.9455554771533, -7.2055935597208, -7.47289804326271, -7.74743747284762, -8.0291485087166, -8.31794009715989, -8.6136976413938, -8.9162871724374, -9.22555951998949, -9.54135448330527, -9.86350500207322, -10.1918413272919, -10.5261951921468, -10.8664039828871, -11.2123149097023, -11.5637891775996, -11.92070615728, -12.2829675560155, -12.6505015885261, -13.0232671478561, -13.4012579762512, -13.7845068360355, -14.1730896804879, -14.567129824719, -14.9668021165484, -15.3723371073808, -15.7840252230833, -16.2022209348619, -16.6273469301387, -17.0598982834284, -17.500446627215, -17.949644322829, -18.4082286313239, -18.8770258843532, -19.3569556550472, -19.8490349288894}, _
        {-5.84691909749438, -6.14157375266608, -6.42863303756028, -6.7104611757862, -6.98915700553996, -7.26656853999283, -7.54430752767902, -7.8237640128838, -8.1061208960313, -8.39236849407254, -8.68331910087346, -8.97962154760281, -9.28177576312012, -9.59014733436372, -9.90498206673863, -10.2264205445046, -10.554512691164, -10.8892323298498, -11.2304917437137, -11.5781562363139, -11.9320586920029, -12.292014136316, -12.6578342963589, -13.0293421611955, -13.4063865422364, -13.7888566336262, -14.1766965726322, -14.5699200000317, -14.9686246205002, -15.3730067629998, -15.7833759411664, -16.2001694136982, -16.6239667447434, -17.0555043642885, -17.4956901285458, -17.9456178803418, -18.4065820095048, -18.8800920132532}, _
        {-5.49733241917701, -5.81655101192913, -6.12063427137222, -6.41313709382158, -6.69723037940491, -6.97572128317933, -7.25107346624873, -7.52542734688118, -7.80062035162612, -8.07820716643172, -8.35947998776222, -8.6454887737152, -8.93706149513892, -9.23482438674957, -9.53922219824866, -9.85053844544027, -10.1689156613484, -10.4943756473342, -10.8268397242135, -11.1661489833738, -11.5120845378917, -11.8643877736505, -12.2227806004571, -12.5869857031594, -12.9567467927637, -13.3318488575523, -13.7121384142001, -14.0975437588924, -14.4880952184423, -14.8839454014077, -15.2853894492088, -15.6928852872452, -16.1070738760137, -16.5287994622248, -16.9591298299209, -17.3993765515929, -17.851115239298, -18.3162057957767}, _
        {-5.05336125821435, -5.39140790445838, -5.7081013735806, -6.00796349643275, -6.29503195246553, -6.57288568436964, -6.84467031271665, -7.11312355060024, -7.38060061827699, -7.64909965780754, -7.92028714769765, -8.19552331753913, -8.47588756265095, -8.76220385872017, -9.05506617644307, -9.35486389616612, -9.66180722252701, -9.97595259909575, -10.2972281230155, -10.6254589596439, -10.9603927571938, -11.3017250613745, -11.6491247300326, -12.0022593477931, -12.3608206407008, -12.7245498908604, -13.0932633510786, -13.4668776595042, -13.84543525427, -14.2291297881331, -14.6183315431163, -15.0136128451489, -15.4157734787082, -15.8258661014597, -16.245221658899, -16.6754747989924, -17.1185892868178, -17.5768834192062}, _
        {-4.66681689085675, -5.0096077911227, -5.3294317417927, -5.63098274015963, -5.91845138611139, -6.19555162613874, -6.46554749734351, -6.73127987144662, -6.9951931987959, -7.25936225237426, -7.52551887180776, -7.79507870737348, -8.06916796400767, -8.34865014531364, -8.63415279756983, -8.92609425373795, -9.22471037747082, -9.53008130712048, -9.84215819974613, -10.1607899751223, -10.4857500597467, -10.8167631308484, -11.1535318603956, -11.4957636591039, -11.8431974204444, -12.1956302646513, -12.5529442827303, -12.9151332804662, -13.2823295224316, -13.6548304759943, -14.0331255553257, -14.4179228654083, -14.8101759460446, -15.2111105158639, -15.6222512163318, -16.0454483557568, -16.4829046532991, -16.9372019829789}, _
        {-3.27703133984164, -3.69934228241808, -4.08675277679173, -4.44492837896238, -4.77899337356653, -5.09355699658386, -5.39273965804338, -5.68019916472924, -5.95915694288669, -6.23242426092828, -6.50242845213985, -6.77123913738639, -7.04059444781829, -7.31192724757726, -7.58639135650218, -7.8648877728356, -8.14809089592917, -8.43647474895008, -8.73033920158688, -9.02983619275574, -9.33499595330606, -9.64575322872695, -9.96197350185295, -10.2834792155701, -10.610075995522, -10.9415788728161, -11.2778385067292, -11.6187674074136, -11.9643661586036, -12.3147496403212, -12.6701732515815, -13.0310591331001, -13.3980223899981, -13.7718973145079, -14.1537636086804, -14.5449726070899, -14.9471734995406, -15.3623395537725}, _
        {-2.42279680937366, -2.92059341633111, -3.37094118027323, -3.78104131662161, -4.15742216853258, -4.50597137306947, -4.83196802737436, -5.14011485484014, -5.43457037128204, -5.71898105110953, -5.99651349349844, -6.26988658856241, -6.54140368352494, -6.81298474889089, -7.08619854461903, -7.36229478629284, -7.64223631129341, -7.92673124497047, -8.21626516681455, -8.51113327662872, -8.81147256070084, -9.11729395797455, -9.42851452622143, -9.74498960821361, -10.0665449978945, -10.3930091065509, -10.7242451289857, -11.060183209688, -11.4008526090069, -11.7464138693215, -12.0971909812146, -12.4537035496428, -12.816698960109, -13.1871845448345, -13.5664597489306, -13.9561482965705, -14.3582303571611, -14.7750747115151}}, _
        {{-4.64759198730505, -4.86584815626874, -5.07345487428605, -5.27468748870919, -5.4732433444008, -5.6722770818515, -5.87443593529699, -6.08189503083565, -6.29639268454571, -6.51926570060276, -6.75148466939709, -6.99368926565099, -7.24622354653626, -7.5091712497914, -7.78239109183917, -8.06555206590382, -8.3581687401285, -8.65963655569272, -8.96926712492961, -9.28632352944316, -9.61005561822611, -9.93973530577661, -10.2746918702161, -10.6143472514066, -10.9582513490679, -11.3061173208951, -11.6578568806759, -12.0136155964079, -12.3738081884163, -12.7391538274707, -13.1107114329032, -13.489914970725, -13.8786087517446, -14.279082729684, -14.6941077992977, -15.1269710944885, -15.5815112864255, -16.062153881662}, _
        {-3.8292655659983, -4.02138294302442, -4.20288644032579, -4.37765086099247, -4.54904998651008, -4.71998633460771, -4.89292091710512, -5.06990299776051, -5.25259985011787, -5.44232651535457, -5.64007556012902, -5.84654683442791, -6.06217722941414, -6.28717043527388, -6.52152669906451, -6.76507258256192, -7.01749072010823, -7.27834957645898, -7.54713320463111, -7.82327100375001, -8.10616747689759, -8.3952319889593, -8.68990852447187, -8.98970544547104, -9.29422524933882, -9.60319432665099, -9.91649271902489, -10.2341838769667, -10.556544417719, -10.8840938831088, -11.2176244973941, -11.5582309251122, -11.9073400289272, -12.2667406274768, -12.638613253221, -13.0255599102886, -13.430633832325, -13.8573692403403}, _
        {-3.57938302511741, -3.74591327181166, -3.90819736151239, -4.06865985661902, -4.22940662663608, -4.39224418122216, -4.55869900323893, -4.73003688180033, -4.90728224532168, -5.09123749456882, -5.28250233570721, -5.48149311335105, -5.68846214361255, -5.90351704715077, -6.12664008222097, -6.35770747772367, -6.59650876625392, -6.84276611715004, -7.09615366954328, -7.35631686540631, -7.62289178260303, -7.89552446793725, -8.17389027020175, -8.45771317322792, -8.74678512893431, -9.04098539037603, -9.34029984479398, -9.6448403466636, -9.95486405074448, -10.2707927451292, -10.5932321842922, -10.9229914221397, -11.261102145058, -11.608838004963, -11.9677339523493, -12.3396055693392, -12.7265684027319, -13.1310572970531}, _
        {-3.33767731957971, -3.51557317097429, -3.68623606859183, -3.85245593892597, -4.01667252733116, -4.18099617709328, -4.34722860850035, -4.51688369791338, -4.691208256837, -4.87120281099034, -5.05764237937774, -5.25109725335945, -5.45195377572252, -5.66043511975143, -5.87662206829885, -6.10047379285646, -6.33184863262588, -6.57052487358891, -6.81622152757891, -7.06861911135098, -7.3273804256533, -7.59217133429737, -7.862681543229, -8.13864537959918, -8.41986257083456, -8.70621902370843, -8.99770760341146, -9.29444891262216, -9.59671207057822, -9.9049354921467, -10.2197476668949, -10.5419879381616, -10.8727272821272, -11.2132890868846, -11.5652699315103, -11.9305603651349, -12.3113656860137, -12.7102267205981}, _
        {-2.88161570460778, -3.10862244449571, -3.31590571157117, -3.50828963729516, -3.6900287112914, -3.86484024733626, -4.03593684934773, -4.20605887737475, -4.37750691358651, -4.55217422826169, -4.73157924577785, -4.91689801060054, -5.10899665327272, -5.30846385640388, -5.5156433206595, -5.73066623075014, -5.95348372142104, -6.18389934344086, -6.42160152959157, -6.66619606065708, -6.91723853141328, -7.17426681661656, -7.43683353699351, -7.70453852523031, -7.97706129196149, -8.25419349175977, -8.53587138912504, -8.82220832447353, -9.11352718012753, -9.41039284630423, -9.71364468710496, -10.0244290065051, -10.3442315143427, -10.6749097923075, -11.0187257599317, -11.3783781405772, -11.7570349274265, -12.1583658494712}, _
        {-2.39036556763159, -2.67511238377499, -2.9261993932089, -3.15076129516044, -3.35510875742249, -3.54477466426913, -3.72456036437097, -3.89858191871056, -4.07031634849774, -4.24264788308507, -4.41791420788333, -4.59795271227665, -4.78414673753824, -4.97747182474531, -5.17854196269506, -5.38765583581951, -5.60484307210142, -5.82991049098903, -6.06248835131225, -6.30207659919722, -6.54809111598244, -6.7999099661336, -7.05691964515928, -7.31856132752641, -7.58437711457526, -7.85405628243529, -8.12748152994023, -8.40477522654353, -8.68634566023404, -8.972933285451, -9.26565697099937, -9.56606024796609, -9.87615755763425, -10.1984804993989, -10.5361240786835, -10.8927929548537, -11.2728476891333, -11.6813509925206}, _
        {-1.37993027416042, -1.66858473196139, -1.93255716240271, -2.17645892995297, -2.40439274548421, -2.61998170551259, -2.82639833144213, -3.02639360880675, -3.22232602651304, -3.41619061608264, -3.60964799089506, -3.80405338542987, -4.0004856945094, -4.1997765125416, -4.40253917276214, -4.60919778647681, -4.8200162823052, -5.03512744542168, -5.25456195679913, -5.47827743245082, -5.7061874626736, -5.93819065128973, -6.17419965488987, -6.41417022207564, -6.65813023270201, -6.9062087371198, -7.15866499541853, -7.41591751666866, -7.67857309816431, -7.94745586466569, -8.22363630764179, -8.50846032451295, -8.80357825789329, -9.11097393483313, -9.43299370606184, -9.77237548523041, -10.1322777881535, -10.5163087720527}, _
        {-0.373277719255384, -0.883663439915765, -1.31343211843319, -1.67685226243368, -1.98663397699989, -2.25401272250568, -2.48883307244801, -2.69963247128086, -2.89372499224749, -3.0772850952139, -3.2554313845018, -3.43231036672154, -3.6111802086053, -3.79449449483981, -3.9839859858998, -4.18075037588059, -4.38533005033162, -4.59779784408885, -4.81784079910841, -5.04484392229893, -5.27797394335587, -5.51626307259268, -5.75869275877535, -6.00427744695493, -6.25214833630022, -6.50163713793133, -6.75235983275254, -7.00430042928488, -7.2578947215003, -7.51411404665364, -7.77454904311518, -8.04149340820584, -8.3180276560278, -8.60810287529854, -8.91662448718514, -9.24953600313466, -9.61390278270904, -10.0179957914179}}, _
        {{-3.45404879662121, -3.39987397419956, -3.37669148141943, -3.38176130972705, -3.41252013240353, -3.46657675012746, -3.54170753653912, -3.63585188380387, -3.74710764817578, -3.87372659556126, -4.01410984708254, -4.16680332464135, -4.33049319648254, -4.50400132275756, -4.68628070108815, -4.87641091212985, -5.07359356513574, -5.27714774351985, -5.48650545042085, -5.70120705426562, -5.92089673433284, -6.1453179263166, -6.37430876788998, -6.60779754426864, -6.84579813377438, -7.08840545339887, -7.33579090436689, -7.58819781770051, -7.84593689978207, -8.10938167791817, -8.37896394590306, -8.65516920958236, -8.93853213241652, -9.22963198104464, -9.52908807084772, -9.83755521151254, -10.1557191525952, -10.4842920290845}, _
        {-2.98719112763655, -2.95679067410822, -2.94625253450543, -2.95439045022135, -2.98006054762906, -3.02216215165481, -3.07963859935218, -3.15147805347574, -3.23671431605486, -3.33442764196765, -3.44374555251451, -3.56384364899222, -3.69394642626755, -3.83332808635115, -3.98131335197131, -4.13727828014781, -4.30065107576576, -4.4709129051493, -4.64759870963543, -4.83029801914795, -5.01865576577106, -5.21237309732333, -5.41120819093145, -5.61497706660401, -5.82355440080532, -6.03687434002928, -6.25493131437304, -6.477780851111, -6.70554038826847, -6.93839008819548, -7.17657365114073, -7.42039912882519, -7.67023973801605, -7.92653467410049, -8.18978992465949, -8.46057908304161, -8.73954416193682, -9.02739640695032}, _
        {-2.51389693656375, -2.57721703353965, -2.63573692526994, -2.69251061590496, -2.7501706349382, -2.81095330101886, -2.87672398576462, -2.94900237757417, -3.02898774543979, -3.11758420275996, -3.2154259711519, -3.32290264426428, -3.44018445158968, -3.56724752227724, -3.70389914894526, -3.84980305149377, -4.00450464091713, -4.16745628311664, -4.33804256271311, -4.51560554685937, -4.69947004905307, -4.88896889294908, -5.08346817617209, -5.28239253412939, -5.48525040382322, -5.6916592876635, -5.9013710172804, -6.11429701733686, -6.33053356934137, -6.55038707546031, -6.77439932233068, -7.00337274487278, -7.2383956901026, -7.48086768094436, -7.7325246800437, -7.99546435357935, -8.27217133507638, -8.56554248921866}, _
        {-2.28613210471389, -2.37909449762597, -2.45999152004158, -2.53302037818911, -2.60183916006475, -2.6695980232415, -2.73897038267936, -2.81218409853508, -2.89105266397186, -2.97700639296916, -3.07112360813252, -3.17416182850329, -3.28658895736843, -3.40861447007024, -3.54022060181628, -3.68119353548896, -3.83115458945542, -3.98959140537741, -4.15588913602086, -4.32936163306574, -4.50928263491598, -4.69491695450905, -4.88555166712576, -5.08052729820026, -5.27926901112956, -5.48131779508339, -5.68636165281409, -5.89426678846613, -6.10510879538628, -6.31920384393303, -6.53713986928646, -6.75980775925827, -6.9884325421012, -7.22460457431877, -7.47031072847585, -7.72796558100726, -8.00044260002833, -8.29110533314465}, _
        {-2.07429589817394, -2.16428809385874, -2.24449795499915, -2.3184190732859, -2.38910481580784, -2.45919346681619, -2.53093336948908, -2.60620806769564, -2.68656144776028, -2.77322288022701, -2.86713236162363, -2.96896565622608, -3.07915943782258, -3.197936431478, -3.32533055529807, -3.46121206219363, -3.60531268164493, -3.75725076146591, -3.91655640956835, -4.08269663572621, -4.25510049333996, -4.43318422120071, -4.61637638525449, -4.80414302036664, -4.99601277208595, -5.19160203840887, -5.39064011154398, -5.59299431967596, -5.79869516873018, -6.00796148413674, -6.22122555259468, -6.43915826383653, -6.66269425239229, -6.89305703935359, -7.13178417413867, -7.3807523762556, -7.64220267706724, -7.9187655615555}, _
        {-1.93275531567866, -1.99371986413414, -2.0547676085774498, -2.11728896186501, -2.18249237653999, -2.25141479879064, -2.32493212240855, -2.40376964274683, -2.48851251067818, -2.5796161865532, -2.67741689415843, -2.78214207467462, -2.89392084063481, -3.01279442988264, -3.13872665953031, -3.27161437991699, -3.41129792856682, -3.55757158414713, -3.71019402042668, -3.86889876023365, -4.03340462941409, -4.20342621078979, -4.3786842981167, -4.55891635004293, -4.74388694406704, -4.93339823049612, -5.127300386404, -5.32550206958944, -5.52798087253426, -5.73479377636163, -5.94608760479401, -6.16210947811153, -6.38321726711013, -6.60989004705954, -6.8427385516618, -7.08251562700913, -7.3301266855421, -7.58664016000825}, _
        {-0.983142024650931, -1.13778934759304, -1.27710706863827, -1.40442108661556, -1.52269746832865, -1.63456134538449, -1.74231581102377, -1.8479608169493, -1.95321207015562, -2.05951992975823, -2.16808830382278, -2.27989354619436, -2.39570335332668, -2.51609566111145, -2.64147754170755, -2.7721041003702, -2.90809737228058, -3.04946521937447, -3.19612022717205, -3.34789860160699, -3.50457906585556, -3.66590175716595, -3.83158712368774, -4.00135482130082, -4.17494261044488, -4.35212525294855, -4.53273340885867, -4.71667253326961, -4.90394177315242, -5.09465286418425, -5.28904902757725, -5.48752386690841, -5.69064026494812, -5.89914928048995, -6.11400904517981, -6.33640366034492, -6.56776209382336, -6.80977707679323}, _
        {-0.823151054205312, -0.90238278447319, -0.990453866806391, -1.08614994269157, -1.18839507078156, -1.29624520816712, -1.40888169164641, -1.52560471899516, -1.64582683023685, -1.76906638891307, -1.89494106335355, -2.02316130794642, -2.15352384440848, -2.28590514305527, -2.42025490407122, -2.55658953878011, -2.6949856509151, -2.83557351788867, -2.97853057206321, -3.12407488202118, -3.27245863383507, -3.42396161233766, -3.57888468239231, -3.73754327016309, -3.90026084438506, -4.06736239763443, -4.23916792759857, -4.41598591834636, -4.5981068215986, -4.78579653799763, -4.9792898983784, -5.17878414503731, -5.38443241300422, -5.59633721131106, -5.81454390426233, -6.03903419270668, -6.26971959530452, -6.50643492980122}}, _
        {{-0.384600046642817, -0.256100423167814, -0.159098296295583, -0.0908176044338411, -0.0486501320599242, -0.0301521650452941, -0.0330411459802438, -0.0551923294984231, -0.0946354376015446, -0.149551314983858, -0.218268584356825, -0.299260301773774, -0.391140611954333, -0.49266140360926, -0.602708964764865, -0.720300638087683, -0.844581476209137, -0.974820897049978, -1.11040933914507, -1.2508549169679, -1.39578007625514, -1.54491824933138, -1.69811051043363, -1.85530223103594, -2.01653973517401, -2.18196695476982, -2.35182208495619, -2.52643423940146, -2.70622010563397, -2.89168060036675, -3.08339752482214, -3.28203022005635, -3.48831222228399, -3.70304791820297, -3.92710920031858, -4.16143212226868, -4.40701355414791, -4.66490783783241}, _
        {0.296908734396979, 0.334957063870995, 0.360323525333786, 0.373137778955179, 0.373611766931358, 0.362031423244165, 0.338748383420297, 0.304171694290588, 0.25875952374926, 0.203010870513202, 0.13745727388121, 0.062654523493201, -0.0208256309104069, -0.112403769729595, -0.211501093745375, -0.317547714360566, -0.429990943840561, -0.548303585554023, -0.671992224213676, -0.800605516117034, -0.933742479387097, -1.07106078421318, -1.21228504309159, -1.35721510106637, -1.50573432597011, -1.65781789866463, -1.81354110328168, -1.97308761746381, -2.13675780260501, -2.30497699409153, -2.47830379154244, -2.65743834905073, -2.84323066542366, -3.03668887442375, -3.23898753500944, -3.45147592157585, -3.67568631419551, -3.91334228885912}, _
        {0.596629505083413, 0.572931517763212, 0.552118388097804, 0.531709963212332, 0.509589740615414, 0.48398185018576, 0.453428036158678, 0.416764639112536, 0.373099577955463, 0.321789331911868, 0.262415922508874, 0.194763895562982, 0.118797303166653, 0.0346366856747768, -0.0574639463087827, -0.157140129944566, -0.263939968171, -0.377347147717859, -0.496803957119671, -0.621734304729214, -0.751566736730987, -0.88575745515452, -1.02381333588795, -1.1653149466915, -1.30993956521082, -1.45748419699038, -1.60788859348718, -1.76125827008389, -1.91788752410255, -2.07828245281781, -2.24318397147054, -2.4135908312812, -2.59078263746332, -2.7763428672368, -2.97218188784171, -3.18055997455134, -3.4041103286858, -3.64586209562563}, _
        {0.777812591973621, 0.72364107823249, 0.679488083440446, 0.641727371059961, 0.607217914104833, 0.573274523715174, 0.537638477732353, 0.498448149274054, 0.454209635309146, 0.403767385232792, 0.346274829441326, 0.281165007907243, 0.208121198754305, 0.127047546832406, 0.0380396922925135, -0.0586446008382184, -0.162614816081534, -0.273377055933129, -0.390363413287562, -0.512961342863386, -0.640543032628169, -0.772494775223361, -0.90824633938944, -1.04730034139097, -1.18926161644156, -1.33386659012868, -1.48101264983916, -1.63078751618367, -1.78349861442218, -1.93970244588864, -2.10023395941626, -2.26623592276233, -2.43918829403343, -2.62093759311012, -2.81372627307247, -3.02022209162459, -3.2435474825198, -3.48730892698585}, _
        {1.02497792207782, 0.943085889843214, 0.876150520699583, 0.819920417444891, 0.770686276753093, 0.725248772764129, 0.680886440673676, 0.635323560323101, 0.586698039789246, 0.533529298974438, 0.474686153196173, 0.409354696777054, 0.337006186634767, 0.257364925871823, 0.170376147365377, 0.0761738973572568, -0.0249510809562654, -0.132583463834636, -0.246215463403057, -0.365278944062718, -0.489177538900891, -0.617318766101018, -0.749146145352916, -0.884171314262892, -1.0220061447639, -1.16239485952549, -1.3052461483643, -1.45066528465376, -1.5989862417347, -1.750803809325, -1.90700570993004, -2.06880471525284, -2.237770762604, -2.41586307131188, -2.60546225913294, -2.80940245866165, -3.0310034337406, -3.27410269587087}, _
        {1.2576850528034, 1.16338160088655, 1.08365094500142, 1.01467102462538, 0.953093758328738, 0.896017140941151, 0.840957340717745, 0.785820796505391, 0.728876314909064, 0.66872716745806, 0.604283187772263, 0.534732868728334, 0.459515459626149, 0.378293063354913, 0.290922733559412, 0.197428571806445, 0.0979738247509498, -0.00716701869775793, -0.117636130209566, -0.233020244867493, -0.35287856400122, -0.476770658021021, -0.60428436925137, -0.735063714764663, -0.86883678921515, -1.00544366767235, -1.14486430845508, -1.28724645596496, -1.43293354352046, -1.58249259619019, -1.73674213362691, -1.8967800729015, -2.06401163133614, -2.24017722933841, -2.42738039323499, -2.62811565810532, -2.84529647061536, -3.08228309185134}, _
        {2.02812744479357, 1.88005846325024, 1.74689759938849, 1.62610029496147, 1.51537050142726, 1.41264761001475, 1.31609338179001, 1.22407887772209, 1.13517138874945, 1.04812136584602, 0.961849350087067, 0.87543290271574, 0.788093535208946, 0.699183639343512, 0.608173417262316, 0.514637811540486, 0.418243435251462, 0.318735502033074, 0.215924756153869, 0.109674402578994, -0.000112962963497942, -0.113508423916601, -0.230569812828173, -0.351354781284861, -0.475933869845969, -0.604403577977227, -0.736899433984861, -0.873609064949245, -1.01478526665887, -1.16075907354429, -1.31195282861179, -1.46889325337757, -1.63222451780126, -1.80272131022, -1.98130190728231, -2.1690412438819, -2.36718398309147, -2.57715758609698}, _
        {2.41787502239237, 2.22361416823922, 2.05399099565641, 1.90458012209902, 1.77145600674836, 1.65116390269113, 1.54069080909811, 1.43743642340326, 1.33918409348328, 1.244071769836, 1.15056295775988, 1.05741766953261, 0.963663376590621, 0.868565961707955, 0.771600671175126, 0.67242306697842, 0.570839978978787, 0.466780457091054, 0.360266723462623, 0.25138512465294, 0.140257083812345, 0.0270100528609571, -0.0882515353319553, -0.205475314769087, -0.324690034146368, -0.446034604673272, -0.569787147894034, -0.696394043509047, -0.826498977195356, -0.960971988427531, -1.10093851829887, -1.24780845734238, -1.4033051933516, -1.56949465920136, -1.74881438066961, -1.94410252425706, -2.15862694500914, -2.39611423433622}}, _
        {{-0.837124105279059, -0.437206675778157, -0.116994934033301, 0.133561156034332, 0.323592797616286, 0.46135537126785, 0.554271445158342, 0.608973785322938, 0.631348365912288, 0.626577379444846, 0.599182247056572, 0.55306662875258, 0.491559433657596, 0.417457830267075, 0.333070256698047, 0.24025943093995, 0.140485361105622, 0.0348483556820973, -0.0758679662185031, -0.191153664607807, -0.310729468370885, -0.43450376501468, -0.562529590417633, -0.694961618578473, -0.832013151365387, -0.973913108265505, -1.12086301613298, -1.27299399893994, -1.43032376752321, -1.59271360933532, -1.75982537819313, -1.93107848402604, -2.10560688262653, -2.28221606539845, -2.45934004910558, -2.63499836562228, -2.80675305168099, -2.97166563862242}, _
        {0.717289314072509, 0.921374060496504, 1.0761450342637, 1.18785831082202, 1.26219262659416, 1.30427600074091, 1.3187123569239, 1.30960814506981, 1.28059896313226, 1.23487617885662, 1.17521355154209, 1.10399385380562, 1.02323549334493, 0.934619134701845, 0.83951432102562, 0.739006095836075, 0.633921624787053, 0.524856817429505, 0.412202948974912, 0.29617328205844, 0.176829688502387, 0.0541092710792235, -0.0721490147249622, -0.20217773894728, -0.336254375384613, -0.474674679830557, -0.617726068312068, -0.765660995326105, -0.918670332076498, -1.07685674471049, -1.24020807255573, -1.4085707063565, -1.58162296651123, -1.75884848130858, -1.93950956516415, -2.12262059685782, -2.3069213977699, -2.49085061011771}, _
        {1.16480436839144, 1.26832512858988, 1.34539654338791, 1.39876001046669, 1.4309135504183, 1.44412349148995, 1.44043615432786, 1.42168953672269, 1.38952499835233, 1.34539894552788, 1.29059451593673, 1.22623326338757, 1.15328684255448, 1.07258869372138, 0.984845727526316, 0.890650009705812, 0.790490445839268, 0.684764466093174, 0.573789709965549, 0.457815711030307, 0.337035581681548, 0.211597697877906, 0.0816173838868579, -0.0528114029708568, -0.191604387576912, -0.334675292271143, -0.481924152107025, -0.633225630107582, -0.788417332520826, -0.94728812407541, -1.10956644323657, -1.27490861746138, -1.44288717845464, -1.61297917742471, -1.78455450033861, -1.95686418317834, -2.12902872719612, -2.30002641417003}, _
        {1.42600954900215, 1.48045677408854, 1.51844418359806, 1.54124485589794, 1.55002844674075, 1.5458663848345, 1.52973706741201, 1.5025310558022, 1.46505627099846, 1.41804318923069, 1.36215003753373, 1.29796798931843, 1.22602635994125, 1.14679780227458, 1.0607035022767, 0.968118374561893, 0.869376257970583, 0.764775111139262, 0.654582208070718, 0.539039333704046, 0.418367979484756, 0.292774538934854, 0.162455503222819, 0.0276026567338707, -0.111591727360098, -0.254929691530448, -0.402202398321519, -0.55318493478088, -0.707631116888994, -0.865268293989079, -1.02579215321742, -1.1888615239328, -1.35409318214673, -1.52105665495338, -1.68926902495927, -1.85818973471338, -2.02721539113711, -2.195674569954}, _
        {1.77460633368107, 1.78565644948179, 1.78622050717595, 1.77688663478027, 1.75820326707611, 1.73068093332895, 1.69479404500637, 1.65098268349829, 1.59965438783485, 1.54118594240661, 1.47592516468261, 1.40419269293006, 1.32628377393299, 1.24247005071157, 1.15300135024097, 1.05810747117052, 0.957999971542701, 0.852873956512159, 0.742909866064831, 0.628275262736921, 0.509126619333941, 0.38561110664989, 0.257868381186025, 0.126032372870227, -0.00976692722421779, -0.149401679159444, -0.292744406413897, -0.439666208163523, -0.590034971562535, -0.743713584024274, -0.900558145502508, -1.06041618077208, -1.22312485170996, -1.38850916957623, -1.55638020729503, -1.72653331173547, -1.89874631599267, -2.07277775166862}, _
        {2.07843653765652, 2.07423625547457, 2.0575182540524, 2.02952790760899, 1.99140206215165, 1.94417345344029, 1.88877512495073, 1.82604484584014, 1.75672952891, 1.68148964857138, 1.6009036588083, 1.51547241114244, 1.42562357259694, 1.33171604366092, 1.23404437625349, 1.13284319168796, 1.0282915986361, 0.920517611092226, 0.809602566337457, 0.695585542903842, 0.578467778538645, 0.458217088168517, 0.33477228186355, 0.208047582801659, 0.0779370452325758, -0.0556810275577622, -0.192939665283172, -0.333978812692983, -0.478940911607882, -0.627966482955554, -0.78118970880687, -0.938734014411192, -1.10070765023274, -1.26719927398581, -1.43827353267119, -1.61396664461141, -1.79428198148701, -1.97918565037206}, _
        {2.90605650530386, 2.83688184830976, 2.75962298102007, 2.67562748381305, 2.58607345579335, 2.4919788587328, 2.39421086101279, 2.29349518156673, 2.19042543382096, 2.08547246963817, 1.97899372325802, 1.87124255523988, 1.76237759640461, 1.65247209177668, 1.54152324452589, 1.42946155990966, 1.31616018921467, 1.2014442736992, 1.0851002885349, 0.966885386748572, 0.846536743164883, 0.723780898347467, 0.598343102541353, 0.469956659615094, 0.338372271002341, 0.203367379644105, 0.0647555139307254, -0.0776043683562904, -0.223798536102198, -0.373849140916857, -0.527704873193251, -0.685231618164675, -0.846203111963934, -1.01029159768002, -1.17705848141725, -1.34594498835254, -1.51626281879385, -1.68718480423808}, _
        {3.76846190952883, 3.53541713376515, 3.32673853912041, 3.13908473433354, 2.96938026726903, 2.81480550563207, 2.67278651768546, 2.54098495296387, 2.41728792298935, 2.29979788198721, 2.18682250760143, 2.0768645816099, 1.96861187063976, 1.86092700688338, 1.75283736881328, 1.64352496189778, 1.53231629931667, 1.4186722826762, 1.3021780827249, 1.18253302006894, 1.05954044588753, 0.933097622648422, 0.803185604823369, 0.669859119603594, 0.533236447615056, 0.393489303634324, 0.250832717303423, 0.105514913845937, -0.0421928052179261, -0.192006181355961, -0.343638118308228, -0.496808801371454, -0.651255816684657, -0.806744270511643, -0.963076908528109, -1.12010423510393, -1.27773463258908, -1.43594448059752}}, _
        {{-0.725818738628433, -0.382680397504075, -0.087934297066937, 0.162456163169509, 0.372301350856944, 0.545191025417253, 0.684499962646569, 0.793393579320366, 0.874833557798311, 0.931583470628581, 0.96621440515289, 0.981110588111065, 0.978475010245718, 0.960335050907065, 0.928548102657595, 0.884807195876705, 0.830646623365523, 0.767447564951572, 0.696443712093376, 0.618726892485405, 0.535252694662555, 0.446846092604966, 0.354207070342697, 0.257916246560448, 0.158440499202339, 0.0561385900764808, -0.0487332105402416, -0.156015499297483, -0.265640115166613, -0.377624514836195, -0.492066148107158, -0.609136833288136, -0.729077132590661, -0.852190727524645, -0.978838794293465, -1.10943437918935, -1.24443677398874, -1.38434589134746}, _
        {0.467051916771538, 0.707393457725033, 0.908726091796524, 1.07490218962137, 1.2094698380243, 1.31568639016517, 1.3965320156835, 1.45472325084476, 1.49272654868523, 1.5127718291573, 1.51686602927522, 1.50680665326033, 1.48419532268628, 1.45045132662459, 1.40682517179008, 1.35441213268603, 1.29416580174973, 1.22691163949783, 1.1533605246717, 1.07412230438282, 0.989719344258199, 0.900600078585638, 0.807152560459266, 0.709718011924798, 0.608604374124935, 0.50409985744489, 0.396486491657621, 0.28605367606907, 0.173111729663932, 0.0580054412507414, -0.0588723803927174, -0.177067356373855, -0.296049987540087, -0.415202104333534, -0.533803316645138, -0.65101746367002, -0.765879063761453, -0.877279764285827}, _
        {0.75056742721253, 0.925969236064162, 1.07846534232323, 1.20913098936519, 1.31904703889707, 1.40929650343189, 1.48096107876111, 1.53511767642887, 1.57283495620553, 1.59516985856096, 1.60316413713859, 1.59784089122845, 1.58020109824123, 1.55122014618157, 1.51184436612182, 1.4629875646755, 1.40552755647095, 1.34030269662491, 1.2681084132161, 1.18969373975879, 1.10575784767643, 1.01694657877521, 0.92384897771757, 0.826993824495988, 0.726846166906247, 0.623803853021409, 0.518194063665122, 0.410269844885253, 0.300206640427502, 0.188098824209031, 0.0739562327920167, -0.0422993021427978, -0.16083742132248, -0.28192270540835, -0.40591814252236, -0.533288595773539, -0.664604270784324, -0.800544183217188}, _
        {0.954392603471234, 1.0990885729272, 1.22577971728816, 1.33495972176037, 1.42717783471871, 1.50303297034786, 1.56316781128317, 1.60826291125248, 1.63903079771794, 1.65621007451648, 1.6605595245021, 1.65285221218667, 1.63386958638152, 1.60439558283894, 1.56521072689342, 1.51708623610321, 1.46077812289154, 1.3970212971882, 1.3265236690708, 1.24996025140625, 1.16796726249212, 1.0811362286981, 0.990008087107246, 0.895067288157653, 0.796735898283517, 0.695367702556824, 0.591242307328647, 0.484559242870468, 0.375432066015621, 0.263882462800804, 0.149834351107346, 0.0331079833026413, -0.0865859511184137, -0.209652222892057, -0.336616960344296, -0.468133546749397, -0.604988517688547, -0.748107458408558}, _
        {1.28964519409334, 1.40616305553561, 1.50398378807165, 1.58445854165826, 1.64885523741619, 1.69836131302962, 1.73408646814514, 1.75706540977036, 1.76826059767427, 1.7685649897852, 1.75880478759062, 1.73974218153609, 1.71207809642445, 1.67645493681489, 1.6334593324223, 1.58362488351621, 1.52743490632002, 1.46532517841021, 1.39768668411538, 1.32486835991548, 1.24717983984096, 1.16489420087186, 1.07825070833699, 0.987457561313155, 0.89269463802419, 0.794116241240099, 0.691853843676418, 0.586018833393094, 0.476705259193798, 0.363992576025026, 0.247948390375262, 0.128631205674107, 0.00609316769150681, -0.119617190063259, -0.248452200942233, -0.380362319759783, -0.515293377393348, -0.653183835384358}, _
        {1.62940871679046, 1.72867641103068, 1.80452271046784, 1.85996436506913, 1.89769775438874, 1.92011603394143, 1.92932628157595, 1.92716664384797, 1.91522348239448, 1.89484852030677, 1.86717598850383, 1.83313977210594, 1.79349055680813, 1.7488129752537, 1.69954275340762, 1.64598385693005, 1.58832563754976, 1.52665997943771, 1.46099844558036, 1.39128942415332, 1.31743527489474, 1.23930947547869, 1.15677376788882, 1.06969530479166, 0.977963795910265, 0.881508654397495, 0.780316143209651, 0.674446521479791, 0.564051190891435, 0.449389842051811, 0.330847600865377, 0.208952174907405, 0.0843909997974226, -0.0419716144275997, -0.169077336939155, -0.295656669744476, -0.420211801312497, -0.540999460200556}, _
        {2.29069800364528, 2.29226899252825, 2.29068887950365, 2.28559102396385, 2.27666097310897, 2.2636339038995, 2.24629206501132, 2.22446221878768, 2.1980130831939, 2.16685277376914, 2.13092624558176, 2.09021273518152, 2.04472320255286, 1.99449777306916, 1.9396031794453, 1.88013020369167, 1.81619111906731, 1.74791713203338, 1.67545582420655, 1.59896859431248, 1.51862810013916, 1.43461570049034, 1.34711889713899, 1.25632877678062, 1.16243745298677, 1.06563550815839, 0.966109435479295, 0.864039080869549, 0.759595084938811, 0.652936324939833, 0.544207356721877, 0.433535856684065, 0.321030063728846, 0.2067762212154, 0.0908360189129906, -0.0267559650455187, -0.145994822210326, -0.26690787186131}, _
        {2.73083300830359, 2.73319906870444, 2.72188765806146, 2.69885626506614, 2.66587298931343, 2.62452429334029, 2.57622275466051, 2.52221481780516, 2.46358854635687, 2.40128137498849, 2.33608786150002, 2.26866743885598, 2.19955216722214, 2.12915448600325, 2.05777496587993, 1.98561006084602, 1.91275986024571, 1.83923584081087, 1.764968618698, 1.68981570152586, 1.61356924041225, 1.53596378201143, 1.4566840205514, 1.3753725498708, 1.29163761545655, 1.20506086648066, 1.11520510783772, 1.02162205218186, 0.923860071964322, 0.821471951470243, 0.714022638856093, 0.601096998186988, 0.482307561473561, 0.357302280709468, 0.225772279908671, 0.0874596071421276, -0.0578350134244943, -0.21024442949356}}, _
        {{-1.53677236000301, -1.15101345237726, -0.808264640357834, -0.506199036543634, -0.242489081272616, -0.0148138636970927, 0.179133557139934, 0.341638831228831, 0.474958996517479, 0.581315157835449, 0.662885165817995, 0.721796295830078, 0.760117926890686, 0.779854220596551, 0.782936800046642, 0.771217428765901, 0.746460689629572, 0.71033666378699, 0.664413609586131, 0.610150641497173, 0.548890409037019, 0.481851775693078, 0.410122497847444, 0.334651903701069, 0.256243572197731, 0.175548011948139, 0.0930553401540273, 0.00908796153231783, -0.0762067527609602, -0.162863786206449, -0.251107797997429, -0.341360444115747, -0.434247698407756, -0.530607173660034, -0.631495442675678, -0.738195359349916, -0.852223379746078, -0.975336883171558}, _
        {-0.469275930410717, -0.0596732280620245, 0.280112750151744, 0.558828636986576, 0.784418396331652, 0.964061194458829, 1.1042092712722, 1.21062581155739, 1.28842281623145, 1.34209897359217, 1.37557753056806, 1.39224416396745, 1.39498485172844, 1.3862237441682, 1.36796103523283, 1.34181083374671, 1.30903903466222, 1.27060119030911, 1.2271803816444, 1.17922508950163, 1.12698706584091, 1.07055920499783, 1.00991341493356, 0.944938488484254, 0.875477974610696, 0.801368049647685, 0.722475388553868, 0.638735036161135, 0.550188278424349, 0.457020513670757, 0.359599123849762, 0.258511345782303, 0.154602142410539, 0.0490120740473508, -0.0567848303737009, -0.160943202049648, -0.261208461057467, -0.354878945105138}, _
        {-0.137232259422134, 0.194772921345361, 0.479797736038933, 0.722106379854338, 0.925702834285666, 1.09433759715824, 1.2315144126596, 1.34049700137249, 1.42431579030639, 1.48577464292987, 1.5274575892028, 1.55173555560815, 1.56077309518445, 1.55653511755762, 1.54079361897325, 1.51513441232859, 1.48096385720473, 1.43951558989863, 1.39185725345542, 1.33889722770018, 1.2813913592705, 1.21994969164808, 1.15504319519121, 1.08701049716673, 1.01606461178223, 0.942299670217938, 0.865697650659097, 0.786135108327915, 0.703389905515724, 0.617147941615099, 0.527009883151909, 0.432497893817427, 0.333062364500584, 0.228088643319847, 0.116903765655472, -0.00121681581833855, -0.127042501101553, -0.261380812835849}, _
        {0.0976939844162956, 0.377633594340892, 0.623062167865925, 0.836111256408017, 1.01888785457124, 1.17346838207349, 1.30189266567199, 1.40615792109011, 1.4882127349428, 1.54995104666316, 1.59320613042837, 1.6197445770858, 1.63126027607921, 1.62936839737463, 1.6155993733869, 1.59139288090531, 1.55809182301994, 1.51693631104776, 1.46905764645879, 1.41547230280204, 1.3570759076318, 1.29463722443362, 1.22879213455049, 1.16003761910893, 1.08872574094514, 1.01505762653099, 0.939077447900225, 0.860666404574625, 0.779536705489983, 0.695225550922334, 0.607089114413933, 0.514296524699468, 0.415823847632232, 0.310448068110045, 0.196741072001473, 0.0730636280719494, -0.062440630090185, -0.211851222147413}, _
        {0.458668883808187, 0.67623010507802, 0.869395038997112, 1.03915690407429, 1.18656435264723, 1.31271280569249, 1.41873578763632, 1.50579626116465, 1.57507796203376, 1.62777673388054, 1.66509186303303, 1.68821741332066, 1.69833356088484, 1.69659792898921, 1.68413692283019, 1.66203706434717, 1.63133632703325, 1.59301547074519, 1.5479893765143, 1.49709838135648, 1.44109961308283, 1.38065832510999, 1.3163392312705, 1.24859784062328, 1.17777179226402, 1.10407219013555, 1.02757493783823, 0.948212073440486, 0.865763104289034, 0.779846341819429, 0.689910236366362, 0.595224711974183, 0.494872501207176, 0.387740479960126, 0.272511002268509, 0.147653235119105, 0.0114144932602702, -0.138188425987575}, _
        {0.789721894660445, 0.972648091812317, 1.13146604322813, 1.26804634987114, 1.38414814232633, 1.48142198740926, 1.56141279477341, 1.62556272351874, 1.67521408879923, 1.71161226843116, 1.73590860950111, 1.74916333497372, 1.75234845030008, 1.74635065002533, 1.73197422439712, 1.70994396597303, 1.68090807622933, 1.64544107216801, 1.60404669292596, 1.55716080638186, 1.50515431576498, 1.44833606626278, 1.38695575162904, 1.32120682079185, 1.25122938446167, 1.17711312173921, 1.09890018672355, 1.01658811512015, 0.930132730848737, 0.839451052651454, 0.744424200700756, 0.644900303207521, 0.540697403028965, 0.43160636427665, 0.317393778924632, 0.197804873417176, 0.0725664152771044, -0.058610380286364}, _
        {1.33380004585662, 1.50683238420621, 1.65142819063661, 1.77036233662535, 1.86620810423776, 1.94134433734465, 1.99796259283951, 2.03807429185493, 2.06351787097898, 2.07596593347291, 2.07693240048736, 2.06777966227946, 2.04972572942959, 2.02385138405825, 1.99110733104268, 1.95232134923381, 1.90820544267385, 1.85936299181152, 1.80629590472013, 1.74941176831413, 1.68903099956588, 1.62539399672227, 1.55866829052194, 1.48895569541182, 1.41629946076418, 1.34069142209314, 1.26207915227191, 1.18037311274926, 1.09545380476654, 1.00717892057461, 0.915390494650303, 0.819922054913597, 0.720605773944492, 0.617279620199493, 0.50979450922852, 0.398021454892124, 0.281858720577881, 0.161238970417518}, _
        {1.91063945703566, 1.94807619285749, 1.98223866740548, 2.01234513112736, 2.03777169324272, 2.0580403638559, 2.0728070960722, 2.08184982811328, 2.08505652543202, 2.08241322282819, 2.07399206656363, 2.05993935647751, 2.04046358810171, 2.01582349477656, 1.98631608976473, 1.95226470836817, 1.91400705004318, 1.87188322051432, 1.82622377389139, 1.77733775478395, 1.72550074041645, 1.67094288274417, 1.61383695056797, 1.55428637164994, 1.49231327482837, 1.42784653213361, 1.36070980090262, 1.29060956589512, 1.21712318140819, 1.13968691339191, 1.05758398156473, 0.969932601528534, 0.875674026884276, 0.773560591346833, 0.662143750860913, 0.539762125715598, 0.404529542660712, 0.254323077020974}}}

        ''' \cond
        ''' Finds the index for value as a fractional value indexing indicies_list.  If value is outside values in indicies_list, the max or min index is returned.
        ''' \endcond
        Private Function get_index(ByRef indicies_list() As Single, ByVal value As Single) As Single
            If (value <= indicies_list(0)) Then
                Return 0
            ElseIf (value >= indicies_list(UBound(indicies_list))) Then
                Return UBound(indicies_list)
            Else
                For index As Integer = 0 To UBound(indicies_list)
                    If (indicies_list(index) = value) Then
                        Return index
                    ElseIf (indicies_list(index) > value) Then
                        Return (index - 1) + (value - indicies_list(index - 1)) / (indicies_list(index) - indicies_list(index - 1))
                    End If
                Next index
            End If
        End Function

        ''' \cond
        ''' Performs 2d interpolation 
        ''' \endcond
        Private Function interp_2d(ByVal y1 As Double, ByVal y2 As Double, ByVal x1 As Single, ByVal x2 As Single, ByVal x As Single) As Double
            Return y1 + (y2 - y1) * (x - x1) / (x2 - x1)
        End Function

        ''' <summary>
        ''' Returns typical coulomb count error for a given IPEAK, VOUT, and VBAT.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="vbat">The battery voltage in V.</param>
        ''' <param name="vout">The output voltage in V.</param>
        ''' <returns>The typical error which can be used to correct the coulomb count in software.</returns>
        Public Function Get_Software_Correction_Factor(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal vout As Single, ByVal vbat As Single) As Double

            ' Translate IPEAK Selection into index into table
            ' This assumes the order of IPEAK_INDICIES matches LTC3335.IPEAK_SELECTION_OPTIONS
            Dim ipeak_index As Integer = CInt(ipeak)

            ' Translate Vout into index into table
            Dim vout_index As Single = get_index(VOUT_INDICIES, vout)

            ' Find index into table that's lower than vbat
            Dim vbat_index As Single = get_index(VBAT_INDICIES, vbat)

            ' if vout is an exact match, only interpolate over vbat
            If vout_index = Int(Math.Floor(vout_index)) Then
                ' if vbat is an exact match, no interpolation is necessary
                If vbat_index = Int(Math.Floor(vbat_index)) Then
                    Return COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(vout_index), CInt(vbat_index))
                Else
                    ' only interpolate for the vbat value
                    Return interp_2d(COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(vout_index), CInt(Math.Floor(vbat_index))), _
                                     COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(vout_index), CInt(Math.Ceiling(vbat_index))), _
                                     CSng(Math.Floor(vbat_index)), _
                                     CSng(Math.Ceiling(vbat_index)), _
                                     vbat_index)
                End If
            Else
                ' if vbat is an exact match, only interpolate for the vout value
                If vbat_index = Int(Math.Floor(vbat_index)) Then
                    Return interp_2d(COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Floor(vout_index)), CInt(vbat_index)), _
                                     COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Ceiling(vout_index)), CInt(vbat_index)), _
                                     CSng(Math.Floor(vout_index)), _
                                     CSng(Math.Ceiling(vout_index)), _
                                     vout_index)
                Else
                    ' interpolate for vbat first, then vout
                    Dim error_vout_floor As Double = interp_2d(COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Floor(vout_index)), CInt(Math.Floor(vbat_index))), _
                                                               COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Floor(vout_index)), CInt(Math.Ceiling(vbat_index))), _
                                                               CSng(Math.Floor(vbat_index)),
                                                               CSng(Math.Ceiling(vbat_index)),
                                                               vbat_index)

                    Dim error_vout_ceil As Double = interp_2d(COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Ceiling(vout_index)), CInt(Math.Floor(vbat_index))), _
                                                              COULOMB_COUNTER_CORRECTION_TABLE(ipeak_index, CInt(Math.Ceiling(vout_index)), CInt(Math.Ceiling(vbat_index))), _
                                                              CSng(Math.Floor(vbat_index)),
                                                              CSng(Math.Ceiling(vbat_index)),
                                                              vbat_index)

                    Return interp_2d(error_vout_floor, error_vout_ceil, CSng(Math.Floor(vout_index)), CSng(Math.Ceiling(vout_index)), vout_index)
                End If
            End If

        End Function

        ''' <summary>
        ''' Returns typical coulomb count error for a given IPEAK, VOUT, and VBAT.
        ''' </summary>
        ''' <param name="ipeak">An option from the IPEAK_SELECTION_OPTIONS enum.</param>
        ''' <param name="vout_setting">An option from the VOLTAGE_SELECTION_OPTIONS enum.</param>
        ''' <param name="vout">The output voltage in V.</param>
        ''' <returns>The typical error which can be used to correct the coulomb count in software.</returns>
        Public Function Get_Software_Correction_Factor(ByVal ipeak As LTC3335.IPEAK_SELECTION_OPTIONS, ByVal vout_setting As LTC3335.VOLTAGE_SELECTION_OPTIONS, ByVal vbat As Single) As Double

            Dim vout As Single = Get_Vout_V(vout_setting)

            If vout < 0 Then
                Return 0
            Else
                Return Get_Software_Correction_Factor(ipeak, vout, vbat)
            End If

        End Function

#End Region

    End Module

End Namespace
