﻿Option Strict On
Option Explicit On

Imports System
Imports System.IO
Imports System.IO.Pipes
Imports System.Text
Imports Microsoft.VisualBasic

Public Class PipeServer

    Private COMMAND_POLL_RATE As Integer = 500  ' ms
    Private RESPONSE_RATE As Integer = 5        ' ms

    Public Enum PIPE_STATE
        PIPE_START
        PIPE_LISTEN
        PIPE_CLOSE
    End Enum

    Private pipe_server As NamedPipeServerStream
    Private pipe_stream As StreamString
    Private pipe_name As String

    Private WithEvents pipeserver_backgroundworker As ComponentModel.BackgroundWorker
    Private WithEvents pipeserver_timer As Windows.Forms.Timer

    Public state As PIPE_STATE
    Public command_string As String
    Public response_string As String

    Private commands_received As Integer
    Private responses_sent As Integer

    Private popup_detect As Boolean

    Public Sub New()
        Try
            pipe_name = PipeServer_Commands.PIPE_NAME
            Me.pipe_server = New NamedPipeServerStream(pipe_name, PipeDirection.InOut, 1, IO.Pipes.PipeTransmissionMode.Message, IO.Pipes.PipeOptions.Asynchronous)
            Me.pipe_stream = New StreamString(Me.pipe_server)

            pipeserver_backgroundworker = New ComponentModel.BackgroundWorker
            pipeserver_timer = New Windows.Forms.Timer
            Me.pipeserver_timer.Start()

            state = PIPE_STATE.PIPE_START
            command_string = ""
            response_string = ""

            commands_received = 0
            responses_sent = 0

            ' Initialize to detect all popups
            popup_detect = True

            ' Initialize the debug displays
            Server_Command_Process()
        Catch ex As Exception

        End Try
    End Sub

    Public Sub Close()
        Me.pipe_server.Disconnect()
        Me.pipe_server.Close()
    End Sub

    Private Sub pipeserver_timer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles pipeserver_timer.Tick
        ' Run named pipe thread periodically to check for input from manufacturing program
        If pipeserver_backgroundworker.IsBusy <> True Then
            pipeserver_timer.Stop()
            pipeserver_backgroundworker.RunWorkerAsync()
        End If
    End Sub

    Private Sub pipeserver_backgroundworker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles pipeserver_backgroundworker.DoWork
        Try
            Server_State_Machine()
        Catch ex As Exception
            D3335.Handle_Exception(ex)
        End Try

    End Sub

    Private Sub pipeserver_backgroundworker_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles pipeserver_backgroundworker.RunWorkerCompleted
        pipeserver_timer.Interval = Server_Command_Process()
        pipeserver_timer.Start()
    End Sub

    Public Sub Server_State_Machine()
        ' State Machine for managing named pipe

        If (state = PIPE_STATE.PIPE_START) Then
            ' Wait for a client to connect
            If Me.pipe_server.IsConnected = False Then
                Me.pipe_server.WaitForConnection()
            End If

            If Me.pipe_server.IsConnected = True Then
                state = PIPE_STATE.PIPE_LISTEN
            End If
        ElseIf (state = PIPE_STATE.PIPE_LISTEN) Then
            ' Check for the client to disconnect
            If Me.pipe_server.IsConnected = False Then
                state = PIPE_STATE.PIPE_CLOSE
            Else
                Try
                    ' Send response to the client if necessary
                    If (response_string <> "") Then
                        Me.pipe_stream.WriteString(response_string)
                        responses_sent += 1
                        response_string = ""
                    End If

                    ' Listen for new commands from client
                    command_string = Me.pipe_stream.ReadString()
                    If (command_string <> "") Then
                        commands_received += 1
                    End If

                Catch ex As Exception
                    ' Most likely the connection was closed.
                    command_string = ""
                End Try
            End If
        ElseIf (state = PIPE_STATE.PIPE_CLOSE) Then
            ' Wait for a client to connect
            Me.pipe_server.Disconnect()
            state = PIPE_STATE.PIPE_START
            response_string = ""
            command_string = ""
        End If

    End Sub

    Public Function Server_Command_Process() As Integer

        ' Display state of the connection in the debug textbox
        If state = PIPE_STATE.PIPE_START Then
            'D3335.Debug_TextBox.Text = "Waiting for Connection"
        ElseIf state = PIPE_STATE.PIPE_CLOSE Then
            'D3335.Debug_TextBox.Text = "Closing Connection"
        ElseIf state = PIPE_STATE.PIPE_LISTEN Then

            ' Look for popups that need to be addressed by the manufacturing test program.
            response_string += response_popups()

            If (command_string = "") Then
                'D3335.Debug_TextBox.Text = "Listening for Commands"
            Else
                'D3335.Debug_TextBox.Text = "Command " + command_string + " Received."

                ' Determine the command type, and call the appropriate function to process it
                Dim command_array() As String = command_string.Split(CChar(COMMAND_DELIMITER))
                For command_num As Integer = 0 To command_array.Length - 1
                    If command_array(command_num) <> "" Then
                        Dim data_array() As String = command_array(command_num).Split(CChar(DATA_DELIMITER))

                        If data_array.Length > 0 Then
                            Try
                                Dim command As Integer = CInt(DirectCast([Enum].Parse(GetType(PipeCommand), data_array(0)), PipeCommand))

                                If ((command >= COMMAND_NOT_A_CONTROL_START) And
                                        (command <= COMMAND_NOT_A_CONTROL_END)) Then
                                    response_string += command_not_a_control(command, data_array)
                                ElseIf ((command >= COMMAND_BUTTON_START) And
                                        (command <= COMMAND_BUTTON_END)) Then
                                    response_string += command_button(command, data_array)
                                ElseIf ((command >= COMMAND_READ_LABEL_START) And
                                        (command <= COMMAND_READ_LABEL_END)) Then
                                    response_string += command_read_label(command, data_array)
                                ElseIf ((command >= COMMAND_SET_TEXTBOX_START) And
                                        (command <= COMMAND_SET_TEXTBOX_END)) Then
                                    response_string += command_set_textbox(command, data_array)
                                ElseIf ((command >= COMMAND_READ_TEXTBOX_START) And
                                        (command <= COMMAND_READ_TEXTBOX_END)) Then
                                    response_string += command_read_textbox(command, data_array)
                                ElseIf ((command >= COMMAND_SET_COMBOBOX_START) And
                                        (command <= COMMAND_SET_COMBOBOX_END)) Then
                                    response_string += command_set_combobox(command, data_array)
                                ElseIf ((command >= COMMAND_READ_COMBOBOX_START) And
                                        (command <= COMMAND_READ_COMBOBOX_END)) Then
                                    response_string += command_read_combobox(command, data_array)
                                ElseIf ((command >= COMMAND_SET_CHECKBOX_START) And
                                        (command <= COMMAND_SET_CHECKBOX_END)) Then
                                    response_string += command_set_checkbox(command, data_array)
                                ElseIf ((command >= COMMAND_READ_CHECKBOX_START) And
                                        (command <= COMMAND_READ_CHECKBOX_END)) Then
                                    response_string += command_read_checkbox(command, data_array)
                                End If
                            Catch ex As Exception
                                response_string += COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                            End Try
                        End If
                    End If
                Next
            End If
        End If

        If response_string <> "" Then
            Return RESPONSE_RATE
        Else
            Return COMMAND_POLL_RATE
        End If

    End Function

    Private Function response_popups() As String
        Dim return_string As String = ""  ' start off assuming nothing needs to be sent.

        ' If detecting popups look for window to become visible so that it can be reported to mfg program
        If (popup_detect = True) Then
            If (Popup.Condition <> Popup.Conditions.None) Then
                return_string += COMMAND_DELIMITER + PipeCommand.Response_Popup.ToString()
                return_string += DATA_DELIMITER + Popup.Condition.ToString()
                popup_detect = False   ' Only send one response each time the popup is opened.
            End If
        ElseIf (Popup.Visible = False) Then
            ' If popup was previously detected, wait for window to be closed before trying to detect them again.
            popup_detect = True
        End If

        Return return_string

    End Function

    Private Function command_not_a_control(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then

            Select Case (command)
                Case PipeCommand.Quit
                    D3335.Close()
                Case PipeCommand.GUI_Version
                    return_string += DATA_DELIMITER + Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString()
                Case PipeCommand.Reset_Counter_Test
                    D3335.Battery_Current_Value_Label.Text = ""
                    D3335.USB_Comm.SendCommand(D3335_USB.USB_DATA_ID.COUNTER_TEST, New D3335_USB.RESET_COMMAND)
                Case PipeCommand.Battery_Selection_Show
                    D3335.Double_Click_CallBack(D3335.battery_cc_graphic.battery_capacity_ah, D3335.battery_cc_graphic.battery_coulombs_discharged_ah, D3335.battery_cc_graphic.alarm_ah)
                Case PipeCommand.Ipeak_Selection_Show
                    D3335.Open_IPEAK_Setting_MouseClick(D3335.LTC3335_IPK0_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                Case PipeCommand.Vout_Selection_Show
                    D3335.Open_VOUT_Setting_MouseClick(D3335.LTC3335_OUT0_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                Case PipeCommand.En_Selection_Show
                    D3335.Open_EN_Setting_MouseClick(D3335.LTC3335_EN_TextBox, New MouseEventArgs(0, 0, 0, 0, 0))
                Case PipeCommand.Battery_Capacity_Ah
                    If D3335.Battery_Selection.Visible = False Then
                        return_string += DATA_DELIMITER + D3335.battery_cc_graphic.battery_capacity_ah.ToString()
                    Else
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.battery_cc_graphic.battery_capacity_ah.ToString()
                    End If
                Case PipeCommand.Battery_Coulombs_Discharged_Ah
                    If D3335.Battery_Selection.Visible = False Then
                        return_string += DATA_DELIMITER + D3335.battery_cc_graphic.battery_coulombs_discharged_ah.ToString()
                    Else
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.battery_cc_graphic.battery_coulombs_discharged_ah.ToString()
                    End If
                Case PipeCommand.Battery_Alarm_Ah
                    If D3335.Battery_Selection.Visible = False Then
                        return_string += DATA_DELIMITER + D3335.battery_cc_graphic.alarm_ah.ToString()
                    Else
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.battery_cc_graphic.alarm_ah.ToString()
                    End If
                Case Else
                    return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
            End Select
        Else
        ' Command had the wrong number of data fields
        return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_button(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then
            Dim e As New System.EventArgs
            Dim sender As New System.Object

            Select Case (command)
                Case PipeCommand.ClickButtonPopupOK
                    Popup.OK_Button_Click(sender, e)
                Case PipeCommand.ClickButtonBatteryCurrentEstimate
                    D3335.Battery_Current_Estimate_Button_Click(sender, e)
                Case PipeCommand.ClickBatterySelectionOK
                    D3335.Battery_Selection.OK_Button_Click(sender, e)
                Case PipeCommand.ClickBatterySelectionCancel
                    D3335.Battery_Selection.Cancel_Button_Click(sender, e)
                Case PipeCommand.ClickIpeakSelectionOK
                    D3335.Ipeak_Selection.OK_Button_Click(sender, e)
                Case PipeCommand.ClickIpeakSelectionCancel
                    D3335.Ipeak_Selection.Cancel_Button_Click(sender, e)
                Case PipeCommand.ClickVoutSelectionOK
                    D3335.Vout_Selection.OK_Button_Click(sender, e)
                Case PipeCommand.ClickVoutSelectionCancel
                    D3335.Vout_Selection.Cancel_Button_Click(sender, e)
                Case PipeCommand.ClickEnSelectionOK
                    D3335.En_Selection.OK_Button_Click(sender, e)
                Case PipeCommand.ClickEnSelectionCancel
                    D3335.En_Selection.Cancel_Button_Click(sender, e)
                Case Else
                    return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
            End Select
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_read_label(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then
            Select Case (command)
                Case PipeCommand.GetLabelTextCurrentEstimate
                    return_string += DATA_DELIMITER + D3335.Battery_Current_MA.ToString
                Case Else
                    return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
            End Select
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_set_textbox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 2 Then
            Try
                Dim sender As New System.Object
                Dim e As New System.EventArgs
                Dim test As Single = CByte(data_array(1))       ' Test that argument can be turned into a byte

                Select Case (command)
                    Case PipeCommand.SetTextBoxAlarmThreshold
                        If D3335.Battery_Selection.Visible = False Then
                            D3335.Alarm_Threshold_TextBox.Text = data_array(1)
                            D3335.Alarm_Threshold_TextBox_LostFocus(sender, e)
                        Else
                            D3335.Battery_Selection.Alarm_Threshold_TextBox.Text = data_array(1)
                            D3335.Battery_Selection.Alarm_Threshold_TextBox_LostFocus(sender, e)
                        End If
                    Case PipeCommand.SetTextBoxAccumulator
                        If D3335.Battery_Selection.Visible = False Then
                            D3335.Accumulator_TextBox.Text = data_array(1)
                            D3335.Accumulator_TextBox_LostFocus(sender, e)
                        Else
                            D3335.Battery_Selection.Accumulator_TextBox.Text = data_array(1)
                            D3335.Battery_Selection.Accumulator_TextBox_LostFocus(sender, e)
                        End If
                    Case PipeCommand.SetTextBoxBatteryCapacity
                        D3335.Battery_Selection.Battery_Capacity_TextBox.Text = data_array(1)
                        D3335.Battery_Selection.Battery_Capacity_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxBatteryPercentDischarged
                        D3335.Battery_Selection.Battery_Percent_Discharged_TextBox.Text = data_array(1)
                        D3335.Battery_Selection.Battery_Percent_Discharged_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxBatteryPercentAlarm
                        D3335.Battery_Selection.Battery_Alarm_Percent_Discharged_TextBox.Text = data_array(1)
                        D3335.Battery_Selection.Battery_Alarm_Percent_Discharged_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxAccumulatorPoll_Period
                        D3335.Accumulator_Poll_Period_TextBox.Text = data_array(1)
                        D3335.Accumulator_Poll_Period_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxADCPoll_Period
                        D3335.ADC_Poll_Period_TextBox.Text = data_array(1)
                        D3335.ADC_Poll_Period_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxSerialNumber
                        D3335.SN_TextBox.Text = data_array(1)
                        D3335.SN_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxVBAT
                        D3335.Battery_Voltage_TextBox.Text = data_array(1)
                        D3335.Battery_Voltage_TextBox_LostFocus(sender, e)
                    Case PipeCommand.SetTextBoxVOUT
                        D3335.Output_Voltage_TextBox.Text = data_array(1)
                        D3335.Output_Voltage_TextBox_LostFocus(sender, e)
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                return_string += PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_read_textbox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then
            Try
                Select Case (command)
                    Case PipeCommand.ReadTextBoxAlarmThreshold
                        If D3335.Battery_Selection.Visible = False Then
                            return_string += DATA_DELIMITER + D3335.Alarm_Threshold_TextBox.Text
                        Else
                            return_string += DATA_DELIMITER + D3335.Battery_Selection.Alarm_Threshold_TextBox.Text
                        End If
                    Case PipeCommand.ReadTextBoxAccumulator
                        If D3335.Battery_Selection.Visible = False Then
                            return_string += DATA_DELIMITER + D3335.Accumulator_TextBox.Text
                        Else
                            return_string += DATA_DELIMITER + D3335.Battery_Selection.Accumulator_TextBox.Text
                        End If
                    Case PipeCommand.ReadTextBoxBatteryCapacity
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.Battery_Capacity_TextBox.Text
                    Case PipeCommand.ReadTextBoxBatteryPercentDischarged
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.Battery_Percent_Discharged_TextBox.Text
                    Case PipeCommand.ReadTextBoxBatteryPercentAlarm
                        return_string += DATA_DELIMITER + D3335.Battery_Selection.Battery_Alarm_Percent_Discharged_TextBox.Text
                    Case PipeCommand.ReadTextBoxSoftwareCorrectionFactor
                        If D3335.Battery_Selection.Visible = False Then
                            return_string += DATA_DELIMITER + D3335.Correction_Factor_Value_TextBox.Text
                        Else
                            return_string += DATA_DELIMITER + D3335.Battery_Selection.Correction_Factor_Value_TextBox.Text
                        End If
                    Case PipeCommand.ReadTextBoxAccumulatorPoll_Period
                        return_string += DATA_DELIMITER + D3335.Accumulator_Poll_Period_TextBox.Text
                    Case PipeCommand.ReadTextBoxADCPoll_Period
                        return_string += DATA_DELIMITER + D3335.ADC_Poll_Period_TextBox.Text
                    Case PipeCommand.ReadTextBoxVBAT
                        return_string += DATA_DELIMITER + D3335.Battery_Voltage_TextBox.Text
                    Case PipeCommand.ReadTextBoxVOUT
                        return_string += DATA_DELIMITER + D3335.Output_Voltage_TextBox.Text
                    Case PipeCommand.ReadTextBoxnIRQ
                        return_string += DATA_DELIMITER + D3335.LTC3335_nIRQ_Textbox.Text
                    Case PipeCommand.ReadTextBoxPGOOD
                        return_string += DATA_DELIMITER + D3335.LTC3335_PGOOD_TextBox.Text
                    Case PipeCommand.ReadTextBoxCounterTestResult
                        return_string += DATA_DELIMITER + D3335.Counter_Test_Result_TextBox.Text
                    Case PipeCommand.ReadTextBoxSerialNumber
                        return_string += DATA_DELIMITER + D3335.SN_TextBox.Text
                    Case PipeCommand.ReadTextBoxFirmwareRevision
                        return_string += DATA_DELIMITER + D3335.FW_Rev_TextBox.Text
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                return_string += PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_set_combobox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 2 Then
            Try
                Dim sender As New System.Object
                Dim e As New System.EventArgs

                Select Case (command)
                    Case PipeCommand.SelectComboBoxItemVoltage
                        D3335.Voltage_Selection_CenteredComboBox.SelectedItem = data_array(1)
                    Case PipeCommand.SelectComboBoxItemPrescaler
                        If D3335.Battery_Selection.Visible = False Then
                            D3335.Prescaler_Selection_CenteredComboBox.SelectedItem = data_array(1)
                        Else
                            D3335.Battery_Selection.Prescaler_Selection_CenteredComboBox.SelectedItem = data_array(1)
                        End If
                    Case PipeCommand.SelectComboBoxItemIpeakSelection
                        D3335.Ipeak_Selection.Ipeak_Selection_ComboBox.SelectedItem = data_array(1)
                    Case PipeCommand.SelectComboBoxItemVoutSelection
                        D3335.Vout_Selection.Set_Selection(DirectCast([Enum].Parse(GetType(LTC3335.VOLTAGE_SELECTION_OPTIONS), data_array(1)), LTC3335.VOLTAGE_SELECTION_OPTIONS))
                    Case PipeCommand.SelectComboBoxItemEnSelection
                        D3335.En_Selection.Set_Selection(DirectCast([Enum].Parse(GetType(LTC3335.EN_CONFIGURATION_OPTIONS), data_array(1)), LTC3335.EN_CONFIGURATION_OPTIONS))
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                ' Command had the wrong format for the expected data fields
                return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_read_combobox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then
            Try
                Dim sender As New System.Object
                Dim e As New System.EventArgs

                Select Case (command)
                    Case PipeCommand.ReadComboBoxItemVoltage
                        return_string += DATA_DELIMITER + D3335.Voltage_Selection_CenteredComboBox.SelectedItem.ToString
                    Case PipeCommand.ReadComboBoxItemPrescaler
                        If D3335.Battery_Selection.Visible = False Then
                            return_string += DATA_DELIMITER + D3335.Prescaler_Selection_CenteredComboBox.SelectedItem.ToString
                        Else
                            return_string += DATA_DELIMITER + D3335.Battery_Selection.Prescaler_Selection_CenteredComboBox.SelectedItem.ToString
                        End If
                    Case PipeCommand.ReadComboBoxItemIpeakSelection
                        return_string += DATA_DELIMITER + D3335.Ipeak_Selection.Ipeak_Selection_ComboBox.SelectedItem.ToString
                    Case PipeCommand.ReadComboBoxItemVoutSelection
                        return_string += DATA_DELIMITER + D3335.Vout_Selection.selection.ToString
                    Case PipeCommand.ReadComboBoxItemEnSelection
                        return_string += DATA_DELIMITER + D3335.En_Selection.selection.ToString
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                ' Command had the wrong format for the expected data fields
                return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_set_checkbox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 2 Then
            Try
                Dim checked As Boolean = CBool(data_array(1))
                Select Case (command)
                    Case PipeCommand.SetCheckBoxManufacturingMode
                        D3335.Manufacturing_Enable_CheckBox.Checked = checked
                    Case PipeCommand.SetCheckBoxSoftwareCorrection
                        If D3335.Battery_Selection.Visible = False Then
                            D3335.Software_Correction_CheckBox.Checked = checked
                        Else
                            D3335.Battery_Selection.Software_Correction_CheckBox.Checked = checked
                        End If
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                ' Command had the wrong format for the expected data fields
                return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

    Private Function command_read_checkbox(ByVal command As Integer, ByVal data_array() As String) As String
        Dim return_string As String = COMMAND_DELIMITER + data_array(0)  ' start off assuming the command is good, and the command number will be returned
        If data_array.Length = 1 Then
            Try
                Select Case (command)
                    Case PipeCommand.ReadCheckBoxManufacturingMode
                        return_string += DATA_DELIMITER + D3335.Manufacturing_Enable_CheckBox.Checked.ToString
                    Case PipeCommand.ReadCheckBoxSoftwareCorrection
                        If D3335.Battery_Selection.Visible = False Then
                            return_string += DATA_DELIMITER + D3335.Software_Correction_CheckBox.Checked.ToString
                        Else
                            return_string += DATA_DELIMITER + D3335.Battery_Selection.Software_Correction_CheckBox.Checked.ToString
                        End If
                    Case Else
                        return_string = COMMAND_DELIMITER + PipeCommand.Response_UnknownCommand.ToString()
                End Select
            Catch ex As Exception
                return_string += PipeCommand.Response_IncorrectFormatofArguments.ToString()
            End Try
        Else
            ' Command had the wrong number of data fields
            return_string = COMMAND_DELIMITER + PipeCommand.Response_IncorrectNumberofArguments.ToString()
        End If
        Return return_string
    End Function

End Class

' Defines the data protocol for reading and writing strings on our stream 
Public Class StreamString
    Private ioStream As Stream
    Private streamEncoding As UnicodeEncoding

    Public Sub New(ByVal ioStream As Stream)
        Me.ioStream = ioStream
        Me.streamEncoding = New UnicodeEncoding(False, False)
    End Sub

    Public Function ReadString() As String
        Dim len As Integer = 0
        len = CType(ioStream.ReadByte(), Integer) * 256
        len += CType(ioStream.ReadByte(), Integer)
        Dim inBuffer As Array = Array.CreateInstance(GetType(Byte), len)
        Me.ioStream.Read(CType(inBuffer, Byte()), 0, len)

        Return streamEncoding.GetString(CType(inBuffer, Byte()))
    End Function

    Public Function WriteString(ByVal outString As String) As Integer
        Dim outBuffer() As Byte = streamEncoding.GetBytes(outString)
        Dim len As Integer = outBuffer.Length
        If len > UInt16.MaxValue Then
            len = CType(UInt16.MaxValue, Integer)
        End If
        Me.ioStream.WriteByte(CType(len \ 256, Byte))
        Me.ioStream.WriteByte(CType(len And 255, Byte))
        Me.ioStream.Write(outBuffer, 0, outBuffer.Length)
        Me.ioStream.Flush()

        Return outBuffer.Length + 2
    End Function
End Class
