﻿Option Strict On
Option Explicit On

Imports WinUSBAccess.WinUSBAccess
Imports System
Imports System.Collections.Generic
Imports System.Runtime.InteropServices
Imports System.Runtime.InteropServices.Marshal
Imports System.Drawing

'''<summary>
''' Demonstrates how to use the WinUSBAccess library to catalog, communicate with, and manage the comings and
''' goings of devices that share a Device Interface GUID.
''' Catalogs all attached devices with a specific device interface GUID.
''' Enables sending and receiving data via bulk, interrupt, and control transfers.  Uses
''' RegisterDeviceNotification() and WM_DEVICE_CHANGE messages to detect when a device is attached or removed.
''' For bulk and interrupt transfers, the application uses background workers to read data back from the USB
''' channels.
''' </summary>

Partial Class D3335_USB
    Inherits Form

#Region "WinUSBAccess Interface"

    ' The Device Interface GUID from the LTC INF file
    Private Const LTC_DEVICE_INTERFACE_GUID_STRING As String = "{27876b85-3b46-453a-8477-afbcd86533c0}"

    ' Instantiate LTCDevices as a new DeviceManagement class object.  This class instance has a property that
    ' is the Device Interface GUID of the devices it manages.  This class instance can register for device
    ' attach/detach messages for devices with the Device Interface GUID, and has a property for holding the
    ' device notification handle.  Finally, this class instance can have a catalog of all the currently
    ' attached devices with the Device Interface GUID.
    Private LTCDevices As DeviceManagement

    ' This is a local instance of the devInfo structure to keep track of the current active device.
    ' Initialized to 0.
    Private CurrentDevice As WinUsbDevice.devInfo

    Private WithEvents readpipe_backgroundworker As ComponentModel.BackgroundWorker
    Private WithEvents writepipe_backgroundworker As ComponentModel.BackgroundWorker
    Private WithEvents removal_timer As Windows.Forms.Timer
    Private Const REMOVAL_TIMEOUT As Integer = 500
    Private USB_Receive_Buffer As New List(Of Byte)     ' List for accumulating bytes received from USB
    Private flush_pipe As Boolean = True                ' When the GUI first launches, it should flush the USB as anything received was meant for some other GUI.

    Public Sub New(Optional ByVal debug_panel As Panel = Nothing)

        ' Create objects for communication with DC2343 USB
        Me.LTCDevices = New DeviceManagement
        Me.CurrentDevice = New WinUsbDevice.devInfo(0)

        removal_timer = New Windows.Forms.Timer
        removal_timer.Stop()
        removal_timer.Interval = REMOVAL_TIMEOUT

        readpipe_backgroundworker = New ComponentModel.BackgroundWorker
        USB_Comm_List_In = New List(Of DC2343_USB_RESPONSE)

        writepipe_backgroundworker = New ComponentModel.BackgroundWorker
        USB_Comm_List_Out = New List(Of DC2343_USB_COMMAND)

        For response_num As Integer = 0 To NUM_RESPONSES - 1
            timestamps(response_num) = New Timestamp
        Next response_num

        init_debug(debug_panel)

        ' Set the Device Interface GUID for the LTCDevices instance of the DeviceManagement class
        LTCDevices.DeviceInterfaceGUIDvalue = New Guid(LTC_DEVICE_INTERFACE_GUID_STRING)

        Me.Attempt_To_Connect()

    End Sub

    Public Sub Attempt_To_Connect()
        ' Register for attach/detach messages from the Device Interface GUID, ignore success
        LTCDevices.RegisterForDeviceNotifications(Me.Handle)

        ' This both invokes a catalog of all the connected devices with the Device Interface GUID and
        ' returns a count.  CatalogDevices will return a -1 if the GUID is invalid, or a device count of
        ' 0, if the GUID is valid but there are no devices of that Device Interface GUID attached.
        If LTCDevices.CatalogDevices() > 0 Then
            device_connected()
        End If
    End Sub

    Public Sub Disconnect_And_Stop()
        If Is_Connected() Then
            WinUsbDevice.StopDevice(CurrentDevice)
        End If
        LTCDevices.StopReceivingDeviceNotifications()
    End Sub

    Public Function Is_Connected() As Boolean
        Dim BlankDevice As WinUsbDevice.devInfo = New WinUsbDevice.devInfo(0)
        Return Not CurrentDevice.Equals(BlankDevice)
    End Function

    ''' <summary>
    ''' Overrides WndProc to enable checking for and handling WM_DEVICECHANGE messages.
    ''' When a WM_DEVICECHANGE (Msg = 0x219) message has arrived, it indicates that a device has been
    ''' attached or removed.  A WM_DEVICECHANGE message with a Wparam of DeviceManagement.DBT_DEVICEARRIVAL
    ''' (0x8000) or DeviceManagement.DBT_DEVICEREMOVECOMPLETE (0x8004) will have an Lparam that is a pointer
    ''' to a structure with the path name of the noobie or dearly departed device.
    ''' </summary>
    ''' <param name="m">
    ''' A WM_DEVICECHANGE message with information about the device.  Specifically the Wparam element
    ''' indicates whether the device departed or arrived, and the Lparam has the path name of the device.
    ''' </param>
    Protected Overrides Sub WndProc(ByRef m As Message)
        Try
            'The OnDeviceChange routine processes WM_DEVICECHANGE messages.
            If m.Msg = WinUSBAccess.WinUSBAccess.DeviceManagement.WM_DEVICECHANGE Then
                If (m.WParam.ToInt32 = DeviceManagement.DBT_DEVICEARRIVAL) Then
                    ' If Wparam = DeviceManagement.DBT_DEVICEARRIVAL, a device has been attached.  Find out if
                    ' it's the 'current' device.  If it is the 'current' device, stop the removal timer.  If the
                    ' removal timer had already timed out, the 'current' device was reset to 0, so the compare
                    ' will fail.
                    If LTCDevices.AllDevices.Count = 0 Then
                        If LTCDevices.CatalogDevices() > 0 Then
                            device_connected()
                        End If
                    ElseIf (LTCDevices.DevicepathnameCompare(CurrentDevice, LTCDevices.IdentifyDeviceChanged(m))) Then
                        ' todo - Handle multiple devices
                        If (WinUsbDevice.StartDevice(CurrentDevice) = False) Then
                            device_error(Device_Event_Codes.FAILED_TO_START, Reflection.MethodInfo.GetCurrentMethod().ToString())
                        End If
                    Else
                        ' todo - Handle multiple devices
                        Dim FreshDevice As New WinUsbDevice.devInfo(0)
                        LTCDevices.DevInfofromPathname(LTCDevices.IdentifyDeviceChanged(m), FreshDevice)
                        LTCDevices.AllDevices.Add(FreshDevice)
                        LTCDevices.AllDevices.IndexOf(FreshDevice)
                    End If
                    removal_timer.Stop()

                ElseIf (m.WParam.ToInt32 = DeviceManagement.DBT_DEVICEREMOVECOMPLETE) Then
                    ' If Wparam = DeviceManagement.DBT_DEVICEREMOVECOMPLETE, a device has been removed, find out
                    ' if it's the 'current' device.  If it is the 'current' device start the removal timer.  When
                    ' the removal timer interval expires, clear the 'current' device.
                    If (LTCDevices.DevicepathnameCompare(CurrentDevice, LTCDevices.IdentifyDeviceChanged(m))) Then
                        removal_timer.Start()
                    Else
                        Dim i As Integer = LTCDevices.IndexOfPathName(LTCDevices.IdentifyDeviceChanged(m))
                        LTCDevices.DeletefromCatalog(i)
                    End If
                End If
            End If
            'Let the base form process the message.
            MyBase.WndProc(m)

        Catch ex As Exception
            handle_exception(ex)
        End Try

    End Sub

    ''' <summary>
    ''' The RemovalTimer was started when the 'current' device detached.  If this event fires, the current
    ''' device has been gone for > RemovalTimeout.  Permanently remove the 'current' device from the device
    ''' catalog and clear the 'current device'.
    ''' </summary>
    ''' <param name="sender">
    ''' ? MS won't share
    ''' </param>
    ''' <param name="e">
    ''' </param>
    ''' <remarks>
    ''' The elapsed time before this event triggers is set using the RemovalTimeout constant.
    ''' </remarks>
    Private Sub removal_timer_tick(ByVal sender As Object, ByVal e As EventArgs) Handles removal_timer.Tick
        device_disconnected()
        removal_timer.Stop()
    End Sub

    Private Sub device_connected()

        ' todo - make this handle multiple devices.  For now, always select the first one.
        CurrentDevice = LTCDevices.AllDevices(0)

        If (WinUsbDevice.StartDevice(CurrentDevice) = False) Then
            device_error(Device_Event_Codes.FAILED_TO_START, Reflection.MethodInfo.GetCurrentMethod().ToString())
        End If

        If Not Event_CallBacks(Callback_Events.DEVICE_CONNECTED) Is Nothing Then
            Event_CallBacks(Callback_Events.DEVICE_CONNECTED)(True)
        End If

        ' Flush the pipe if connected the first time
        If (Me.Is_Connected = True) And (flush_pipe = True) Then
            Dim buffer(USB_MAX_PACKET_SIZE - 1) As Byte
            Dim bytesRead As UInt32
            Dim ReturnedCode As Integer = WinUsbDevice.ReadPipe(CurrentDevice, CurrentDevice.bulkInPipe, USB_MAX_PACKET_SIZE, buffer, bytesRead)
            If ReturnedCode > 0 Then
                ReturnedCode = WinUsbDevice.ReadPipe(CurrentDevice, CurrentDevice.bulkInPipe, USB_MAX_PACKET_SIZE, buffer, bytesRead)
            End If
            flush_pipe = False      ' Don't flush again, as it just causes unnecessary delays when a new board is connected.
        End If

        If readpipe_backgroundworker.IsBusy <> True Then
            readpipe_backgroundworker.RunWorkerAsync()
        End If

    End Sub

    Private Sub device_disconnected()

        Try
            ' If you are here the current device has been gone for more than the RemovalTimeout, and its time to 
            ' part ways.
            If LTCDevices.AllDevices.Contains(CurrentDevice) Then
                Dim i As Integer = LTCDevices.AllDevices.IndexOf(CurrentDevice) ' Get index of CurrentDevice

                WinUsbDevice.StopDevice(CurrentDevice)      ' Stop the current device
                LTCDevices.AllDevices.RemoveAt(i)         ' Delete the CurrentDevice from the catalog
                CurrentDevice = New WinUsbDevice.devInfo(0) ' Clear the local CurrentDevice devInfo structure

                If Not Event_CallBacks(Callback_Events.DEVICE_DISCONNECTED) Is Nothing Then
                    Event_CallBacks(Callback_Events.DEVICE_DISCONNECTED)(False)
                End If

            End If
        Catch ex As Exception
            handle_exception(ex)
        End Try
    End Sub

    Private Sub device_error(ByVal Event_Code As Device_Event_Codes, ByVal error_msg As String)

        If Not Event_CallBacks(Callback_Events.DEVICE_ERROR) Is Nothing Then
            Dim error_obj As Device_Error_Structure
            error_obj.Event_Code = Event_Code
            error_obj.error_msg = error_msg
            Event_CallBacks(Callback_Events.DEVICE_ERROR)(error_obj)
        End If
    End Sub

    Private Sub handle_exception(ByVal ex As Exception)
        ' Don't generate a bunch of errors if the device is unplugged in the middle of communication.
        If Me.Is_Connected() = True Then
            If Not Event_CallBacks(Callback_Events.EXCEPTION_HANDLED) Is Nothing Then
                Event_CallBacks(Callback_Events.EXCEPTION_HANDLED)(ex)
            End If
        End If
    End Sub

#End Region

#Region "DC2343 Interface"
    Private Const USB_MAX_PACKET_SIZE As Integer = 64
    Friend Const USB_PARSER_START_OF_RESPONSE As UInt16 = &HA123
    Private Const USB_PARSER_START_OF_COMMAND As UInt16 = &HB456

    Private Const USB_MAX_COMMANDS As Integer = 100
    Private Const USB_MAX_RESPONSES As Integer = 100

    Private USB_Comm_List_In As List(Of DC2343_USB_RESPONSE)       ' List for accumulating responses from USB
    Private USB_Comm_List_Out As List(Of DC2343_USB_COMMAND)       ' List for accumulating commands to USB
    Private timestamps(NUM_RESPONSES - 1) As Timestamp

    ' Variables and controls used to display statistics about USB Comm.
    Private response_count(USB_DATA_ID.NUM_DATA_IDS - 1) As Integer
    Private command_count(USB_DATA_ID.NUM_DATA_IDS - 1) As Integer

    Private debug_display As Boolean
    Private dataid_header_label As Label
    Private response_header_label As Label
    Private command_header_label As Label
    Private dataid_label(USB_DATA_ID.NUM_DATA_IDS - 1) As Label
    Private response_label(USB_DATA_ID.NUM_DATA_IDS - 1) As Label
    Private command_label(USB_DATA_ID.NUM_DATA_IDS - 1) As Label

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Friend Structure COMMAND_HEADER_STRUCT
        <FieldOffset(0)> Friend start_of_command As UInt16
        <FieldOffset(2)> Friend data_id As Byte
        <FieldOffset(3)> Friend subcommand As Byte
        <FieldOffset(4)> Friend number_of_bytes As UInt16
    End Structure

    <StructLayout(LayoutKind.Explicit, Pack:=1)> _
    Friend Structure RESPONSE_HEADER_STRUCT
        <FieldOffset(0)> Friend start_of_response As UInt16
        <FieldOffset(2)> Friend data_id As Byte
        <FieldOffset(3)> Friend timestamp As UInt32
        <FieldOffset(7)> Friend number_of_bytes As UInt16
    End Structure

    Public Enum SUBCOMMAND
        READ
        WRITE
        RESET
        REFRESH
        NUM_SUBCOMMANDS
        NONE = &HFF
    End Enum

    Private Enum USB_PARSER_STATE_TYPE
        USB_PARSER_STATE_WAITING
        USB_PARSER_STATE_RESPONSE_RECEIVING_HEADER
        USB_PARSER_STATE_RESPONSE_RECEIVING_DATA
        USB_PARSER_STATE_RESPONSE_PROCESSING
        USB_PARSER_STATE_RESPONSE_RECEIVED
        USB_PARSER_NUM_STATES
    End Enum

    Private Sub writepipe_backgroundworker_dowork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles writepipe_backgroundworker.DoWork

        Try
            ' Get data to send 
            Dim command As DC2343_USB_COMMAND = DirectCast(e.Argument, DC2343_USB_COMMAND)

            'Build the header for the command to this Data ID
            Dim header As COMMAND_HEADER_STRUCT
            header.start_of_command = USB_PARSER_START_OF_COMMAND
            header.data_id = CByte(command.data_id)

            ' Update counts for debugging
            command_count(command.data_id) += 1

            'The rest of the header depends upon the subcommand
            Dim success As Boolean = True
            Dim data(0) As Byte
            If (command.data.GetType() Is GetType(READ_COMMAND)) Then
                ' If this is a read command, no need for a function call.  Just request the data.
                header.subcommand = CByte(SUBCOMMAND.READ)
                header.number_of_bytes = 0
            Else
                ' A reset, refresh, or write command requires calling a function to get the data.
                If (command.data.GetType() Is GetType(RESET_COMMAND)) Then
                    header.subcommand = CByte(SUBCOMMAND.RESET)
                ElseIf (command.data.GetType() Is GetType(REFRESH_COMMAND)) Then
                    header.subcommand = CByte(SUBCOMMAND.REFRESH)
                Else
                    header.subcommand = CByte(SUBCOMMAND.WRITE)
                End If

                ' Call function to get data for command to this Data ID.
                Select Case command.data_id

                    Case USB_DATA_ID.FW_REVISION
                        success = Send_Fw_Revision_Command(command.data, data)
                    Case USB_DATA_ID.SERIAL_NUMBER
                        success = Send_Serial_Number_Command(command.data, data)
                    Case USB_DATA_ID.ERROR_DATA
                        success = Send_Error_Data_Command(command.data, data)
                    Case USB_DATA_ID.VOLTAGE_SELECTION
                        success = Send_Voltage_Selection_Command(command.data, data)
                    Case USB_DATA_ID.PRESCALER_SELECTION
                        success = Send_Prescaler_Selection_Command(command.data, data)
                    Case USB_DATA_ID.IPEAK_CONFIGURATION
                        success = Send_Ipeak_Selection_Command(command.data, data)
                    Case USB_DATA_ID.VOUT_CONFIGURATION
                        success = Send_Vout_Configuration_Command(command.data, data)
                    Case USB_DATA_ID.ALARM_THRESHOLD
                        success = Send_Alarm_Threshold_Command(command.data, data)
                    Case USB_DATA_ID.ACCUMULATOR
                        success = Send_Accumulator_Command(command.data, data)
                    Case USB_DATA_ID.ALARMS
                        success = Send_Alarms_Command(command.data, data)
                    Case USB_DATA_ID.CLEAR_INT
                        success = Send_Clear_Int_Command(command.data, data)
                    Case USB_DATA_ID.COUNTER_TEST
                        success = Send_Counter_Test_Command(command.data, data)
                    Case USB_DATA_ID.IRQ
                        success = Send_Irq_Command(command.data, data)
                    Case USB_DATA_ID.PGOOD
                        success = Send_Pgood_Command(command.data, data)
                    Case USB_DATA_ID.ADC
                        success = Send_Adc_Command(command.data, data)
                    Case USB_DATA_ID.CAPACITY
                        success = Send_Capacity_Command(command.data, data)
                    Case USB_DATA_ID.NONSTANDARD_INDUCTOR
                        success = Send_Nonstandard_Inductor_Command(command.data, data)
                    Case USB_DATA_ID.EN_CONFIGURATION
                        success = Send_En_Configuration_Command(command.data, data)
                    Case Else
                        Throw New Exception("Invalid Command")
                End Select

                ' Populate the number of bytes in the header given the length of data returned by the function call.
                header.number_of_bytes = CUShort(data.Length)

            End If

            If success = False Then
                Throw New Exception("Invalid Data: " + command.data_id.ToString())
            End If

            ' Make a single buffer big enough for the header and data to pass to WritePipe function.
            Dim data_bytes(SizeOf(header) + header.number_of_bytes - 1) As Byte
            Dim index As Integer = 0

            ' Copy header to buffer in NBO.
            success = success And Put_Network_Byte_Order(data_bytes, index, header.start_of_command)
            success = success And Put_Network_Byte_Order(data_bytes, index, header.data_id)
            success = success And Put_Network_Byte_Order(data_bytes, index, header.subcommand)
            success = success And Put_Network_Byte_Order(data_bytes, index, header.number_of_bytes)

            ' If there is data, copy to buffer.  The command funtion will have already written in NBO, so just copy it byte by byte.
            If (header.number_of_bytes <> 0) Then
                Array.Copy(data, 0, data_bytes, SizeOf(header), data.Length)
            End If

            If success = False Then
                Throw New Exception("Error Formatting Data")
            End If

            'Send the data
            Dim ReturnedCode As Integer
            Do While ReturnedCode <= 0
                ReturnedCode = WinUsbDevice.WritePipe(CurrentDevice, CurrentDevice.bulkOutPipe, data_bytes, CUInt(data_bytes.Length))
            Loop

            ' Return the entire object
            e.Result = command

        Catch ex As Exception
            e.Result = ex
        End Try

    End Sub

    Private Sub writepipe_backgroundworker_runworkercompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles writepipe_backgroundworker.RunWorkerCompleted

        ' Either a successfully sent command or an exception was returned by the background worker.
        If (e.Result.GetType() Is GetType(DC2343_USB_COMMAND)) Then
            ' Pass sent command to GUI
            Dim command As DC2343_USB_COMMAND = DirectCast(e.Result, DC2343_USB_COMMAND)
            If Not Event_CallBacks(Callback_Events.COMMAND_SENT) Is Nothing Then
                Event_CallBacks(Callback_Events.COMMAND_SENT)(command)
            End If
        Else
            ' Handle the exception if returned from the background worker
            Dim ex As Exception = DirectCast(e.Result, Exception)
            handle_exception(ex)
        End If

        ' Remove message whether it was successfully sent or not.
        USB_Comm_List_Out.RemoveAt(0)

        ' Start sending more commands if queued up.
        If (USB_Comm_List_Out.Count <> 0) Then
            writepipe_backgroundworker.RunWorkerAsync(USB_Comm_List_Out(0))
        End If

    End Sub

    Private Sub readpipe_backgroundworker_dowork(ByVal sender As Object, ByVal e As ComponentModel.DoWorkEventArgs) Handles readpipe_backgroundworker.DoWork

        Dim usb_parser_state As USB_PARSER_STATE_TYPE = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_WAITING

        Try
            Dim header As New RESPONSE_HEADER_STRUCT
            Dim response As New DC2343_USB_RESPONSE

            ' Read data from bulk IN channel until a valid response is received.
            Do While usb_parser_state <> USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVED

                ' Process the bytes received.
                Select Case usb_parser_state

                    Case USB_PARSER_STATE_TYPE.USB_PARSER_STATE_WAITING
check_start:
                        While (USB_Receive_Buffer.Count >= SizeOf(USB_PARSER_START_OF_RESPONSE))
                            If (USB_Receive_Buffer(0) = (USB_PARSER_START_OF_RESPONSE >> 8)) AndAlso (USB_Receive_Buffer(1) = (USB_PARSER_START_OF_RESPONSE And &HFF)) Then
                                usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVING_HEADER
                                GoTo receive_header
                            Else
                                USB_Receive_Buffer.RemoveAt(0)
                            End If
                        End While

                    Case USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVING_HEADER
receive_header:
                        If (USB_Receive_Buffer.Count >= SizeOf(header)) Then
                            Dim success As Boolean = True
                            Dim index As Integer = 0
                            Dim header_bytes() As Byte = USB_Receive_Buffer.GetRange(0, SizeOf(header)).ToArray
                            success = success And Get_Network_Byte_Order(header_bytes, index, header.start_of_response)
                            success = success And Get_Network_Byte_Order(header_bytes, index, header.data_id)
                            success = success And Get_Network_Byte_Order(header_bytes, index, header.timestamp)
                            success = success And Get_Network_Byte_Order(header_bytes, index, header.number_of_bytes)
                            success = success And ((header.data_id < USB_DATA_ID.NUM_DATA_IDS) OrElse _
                                                   ((header.data_id >= RESPONSE_STREAM_FIRST) AndAlso (header.data_id < USB_DATA_ID.STREAM_LAST)))
                            If (success = True) Then
                                USB_Receive_Buffer.RemoveRange(0, SizeOf(header))
                                usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVING_DATA
                                GoTo receive_data
                            Else
                                USB_Receive_Buffer.RemoveAt(0)
                                usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_WAITING
                                GoTo check_start
                            End If
                        End If

                    Case USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVING_DATA
receive_data:
                        If (USB_Receive_Buffer.Count >= header.number_of_bytes) Then
                            usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_PROCESSING
                            GoTo receive_response
                        End If

                    Case USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_PROCESSING
receive_response:
                        Dim success As Boolean = True
                        Dim data_bytes() As Byte = USB_Receive_Buffer.GetRange(0, header.number_of_bytes).ToArray

                        ' Process the header for this response.
                        response.data_id = CType(header.data_id, USB_DATA_ID)
                        Dim timestamp_index As Integer
                        If (header.data_id < USB_DATA_ID.NUM_DATA_IDS) Then
                            timestamp_index = header.data_id
                        Else
                            timestamp_index = USB_DATA_ID.NUM_DATA_IDS + (header.data_id - RESPONSE_STREAM_FIRST)
                        End If
                        timestamps(timestamp_index).Update(header.timestamp)
                        response.time = timestamps(timestamp_index).time

                        ' Update counts for debugging
                        response_count(header.data_id) += 1

                        ' Process the data for this response.
                        Select Case header.data_id

                            Case CByte(USB_DATA_ID.FW_REVISION)
                                Dim output_obj As New FW_REVISION_RESPONSE()
                                success = Receive_Fw_Revision_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.SERIAL_NUMBER)
                                Dim output_obj As New SERIAL_NUMBER_RESPONSE()
                                success = Receive_Serial_Number_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.ERROR_DATA)
                                Dim output_obj As New ERROR_DATA_RESPONSE()
                                success = Receive_Error_Data_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.VOLTAGE_SELECTION)
                                Dim output_obj As New VOLTAGE_SELECTION_RESPONSE()
                                success = Receive_Voltage_Selection_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.PRESCALER_SELECTION)
                                Dim output_obj As New PRESCALER_SELECTION_RESPONSE()
                                success = Receive_Prescaler_Selection_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.IPEAK_CONFIGURATION)
                                Dim output_obj As New Ipeak_Selection_RESPONSE()
                                success = Receive_Ipeak_Selection_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.VOUT_CONFIGURATION)
                                Dim output_obj As New VOUT_CONFIGURATION_RESPONSE()
                                success = Receive_Vout_Configuration_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.ALARM_THRESHOLD)
                                Dim output_obj As New ALARM_THRESHOLD_RESPONSE()
                                success = Receive_Alarm_Threshold_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.ACCUMULATOR)
                                Dim output_obj As New ACCUMULATOR_RESPONSE()
                                success = Receive_Accumulator_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.ALARMS)
                                Dim output_obj As New ALARMS_RESPONSE()
                                success = Receive_Alarms_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.CLEAR_INT)
                                Dim output_obj As New CLEAR_INT_RESPONSE()
                                success = Receive_Clear_Int_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.COUNTER_TEST)
                                Dim output_obj As New COUNTER_TEST_RESPONSE()
                                success = Receive_Counter_Test_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.IRQ)
                                Dim output_obj As New IRQ_RESPONSE()
                                success = Receive_Irq_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.PGOOD)
                                Dim output_obj As New PGOOD_RESPONSE()
                                success = Receive_Pgood_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.ADC)
                                Dim output_obj As New ADC_RESPONSE()
                                success = Receive_Adc_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.CAPACITY)
                                Dim output_obj As New CAPACITY_RESPONSE()
                                success = Receive_Capacity_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.NONSTANDARD_INDUCTOR)
                                Dim output_obj As New NONSTANDARD_INDUCTOR_RESPONSE()
                                success = Receive_Nonstandard_Inductor_Response(data_bytes, output_obj)
                                response.data = output_obj

                            Case CByte(USB_DATA_ID.EN_CONFIGURATION)
                                Dim output_obj As New EN_CONFIGURATION_RESPONSE()
                                success = Receive_En_Configuration_Response(data_bytes, output_obj)
                                response.data = output_obj

                        End Select

                        ' If this command processed successfully, pass the response back to the GUI.
                        If (success = True) Then
                            USB_Receive_Buffer.RemoveRange(0, header.number_of_bytes)
                            usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVED
                        Else
                            USB_Receive_Buffer.RemoveAt(0)
                            usb_parser_state = USB_PARSER_STATE_TYPE.USB_PARSER_STATE_WAITING
                            GoTo check_start
                        End If
                End Select

                ' Read new bytes from WinUSB and add to bytes received previously.
                If usb_parser_state <> USB_PARSER_STATE_TYPE.USB_PARSER_STATE_RESPONSE_RECEIVED Then
                    Dim buffer(USB_MAX_PACKET_SIZE - 1) As Byte
                    Dim bytesRead As UInt32
                    Dim ReturnedCode As Integer = WinUsbDevice.ReadPipe(CurrentDevice, CurrentDevice.bulkInPipe, USB_MAX_PACKET_SIZE, buffer, bytesRead)
                    If ReturnedCode > 0 Then
                        For byte_num As Integer = 0 To CInt((bytesRead - 1))
                            USB_Receive_Buffer.Add(buffer(byte_num))
                        Next byte_num
                    End If
                End If

            Loop

            ' Return the entire object
            e.Result = response

        Catch ex As Exception
            Try
                USB_Receive_Buffer.RemoveAt(0)
            Catch ex2 As Exception
                ' I'd rather pass the first ex than the second ex2
            End Try
            e.Result = ex
        End Try
    End Sub

    Private Sub readpipe_backgroundworker_runworkercompleted(ByVal sender As Object, ByVal e As ComponentModel.RunWorkerCompletedEventArgs) Handles readpipe_backgroundworker.RunWorkerCompleted

        ' Either a successfully received response or an exception was returned by the background worker.
        If (e.Result.GetType() Is GetType(DC2343_USB_RESPONSE)) Then
            ' Pass received response to GUI
            Dim response As DC2343_USB_RESPONSE = DirectCast(e.Result, DC2343_USB_RESPONSE)
            If Not Event_CallBacks(Callback_Events.RESPONSE_RECEIVED) Is Nothing Then
                Event_CallBacks(Callback_Events.RESPONSE_RECEIVED)(response)
            Else
                ' If GUI not receiving responses through callback, then store in list for polled commands.
                If USB_Comm_List_In.Count >= USB_MAX_RESPONSES Then
                    USB_Comm_List_In.RemoveAt(0)
                End If
                USB_Comm_List_In.Add(response)
            End If
        Else
            ' Handle the exception if returned from the background worker
            Dim ex As Exception = DirectCast(e.Result, Exception)
            handle_exception(ex)
        End If

        ' Start receiving more bytes
        If Me.Is_Connected() = True Then
            readpipe_backgroundworker.RunWorkerAsync()
        End If

    End Sub

    Private Sub init_debug(Optional ByVal debug_panel As Panel = Nothing)

        For dataid_num As Integer = 0 To USB_DATA_ID.NUM_DATA_IDS - 1
            response_count(dataid_num) = 0
            command_count(dataid_num) = 0
        Next dataid_num

        If debug_panel Is Nothing Then
            debug_display = False
        Else
            debug_display = True

            ' Build the header labels
            dataid_header_label = New Label
            response_header_label = New Label
            command_header_label = New Label

            With dataid_header_label
                .Parent = debug_panel
                .Location = New Point(0, 0)
                .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
                .AutoSize = True
                .Text = "Data ID"
                .TextAlign = ContentAlignment.MiddleCenter
            End With
            D3335.Set_Font(DirectCast(dataid_header_label, Control))

            With command_header_label
                .Parent = debug_panel
                .Location = New Point(dataid_header_label.Location.X + dataid_header_label.Width, dataid_header_label.Location.Y)
                .Font = dataid_header_label.Font
                .AutoSize = dataid_header_label.AutoSize
                .Text = "Commands"
                .TextAlign = ContentAlignment.MiddleCenter
            End With
            D3335.Set_Font(DirectCast(command_header_label, Control))

            With response_header_label
                .Parent = debug_panel
                .Location = New Point(command_header_label.Location.X + command_header_label.Width, command_header_label.Location.Y)
                .Font = dataid_header_label.Font
                .AutoSize = dataid_header_label.AutoSize
                .Text = "Responses"
                .TextAlign = ContentAlignment.MiddleCenter
            End With
            D3335.Set_Font(DirectCast(response_header_label, Control))

            For dataid_num As Integer = 0 To USB_DATA_ID.NUM_DATA_IDS - 1
                dataid_label(dataid_num) = New Label
                response_label(dataid_num) = New Label
                command_label(dataid_num) = New Label

                With dataid_label(dataid_num)
                    .Parent = debug_panel
                    .Location = New Point(dataid_header_label.Location.X, If(dataid_num = 0, dataid_header_label.Location.Y + dataid_header_label.Height, dataid_label(dataid_num - 1).Location.Y + dataid_label(dataid_num - 1).Height))
                    .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Bold)
                    .Height = dataid_header_label.Height
                    .Width = dataid_header_label.Width
                    .Text = dataid_num.ToString
                    .TextAlign = ContentAlignment.MiddleCenter
                End With
                D3335.Set_Font(DirectCast(dataid_label(dataid_num), Control))

                With command_label(dataid_num)
                    .Parent = debug_panel
                    .Location = New Point(dataid_label(dataid_num).Location.X + dataid_label(dataid_num).Width, dataid_label(dataid_num).Location.Y)
                    .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
                    .Height = command_header_label.Height
                    .Width = command_header_label.Width
                    .Text = 0.ToString()
                    .TextAlign = ContentAlignment.MiddleCenter
                End With
                D3335.Set_Font(DirectCast(command_label(dataid_num), Control))

                With response_label(dataid_num)
                    .Parent = debug_panel
                    .Location = New Point(command_label(dataid_num).Location.X + command_label(dataid_num).Width, command_label(dataid_num).Location.Y)
                    .Font = New Font("Linear Helv Cond", 9.75!, FontStyle.Regular)
                    .Height = response_header_label.Height
                    .Width = response_header_label.Width
                    .Text = 0.ToString()
                    .TextAlign = ContentAlignment.MiddleCenter
                End With
                D3335.Set_Font(DirectCast(response_label(dataid_num), Control))

            Next dataid_num

        End If

    End Sub

    Public Sub Update_Debug()

        If debug_display = True Then

            For dataid_num As Integer = 0 To USB_DATA_ID.NUM_DATA_IDS - 1

                With command_label(dataid_num)
                    .Text = command_count(dataid_num).ToString()
                End With

                With response_label(dataid_num)
                    .Text = response_count(dataid_num).ToString()
                End With

            Next dataid_num

        End If
    End Sub

#End Region

End Class
